@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,15 +6,18 @@
 
 #include <config.h>
 
+#include <cfgrpt/config_report.h>
 #include <config/command_mgr.h>
 #include <d2/d2_controller.h>
 #include <d2/d2_process.h>
 #include <d2/parser_context.h>
+#include <stats/stats_mgr.h>
 
 #include <stdlib.h>
 
 using namespace isc::config;
 using namespace isc::process;
+using namespace isc::stats;
 namespace ph = std::placeholders;
 
 namespace isc {
@@ -79,6 +82,19 @@ D2Controller::registerCommands() {
 
     CommandMgr::instance().registerCommand(VERSION_GET_COMMAND,
         std::bind(&D2Controller::versionGetHandler, this, ph::_1, ph::_2));
+
+    // Register statistic related commands.
+    CommandMgr::instance().registerCommand("statistic-get",
+        std::bind(&StatsMgr::statisticGetHandler, ph::_1, ph::_2));
+
+    CommandMgr::instance().registerCommand("statistic-get-all",
+        std::bind(&StatsMgr::statisticGetAllHandler, ph::_1, ph::_2));
+
+    CommandMgr::instance().registerCommand("statistic-reset",
+        std::bind(&StatsMgr::statisticResetHandler, ph::_1, ph::_2));
+
+    CommandMgr::instance().registerCommand("statistic-reset-all",
+        std::bind(&StatsMgr::statisticResetAllHandler, ph::_1, ph::_2));
 }
 
 void
@@ -95,6 +111,10 @@ D2Controller::deregisterCommands() {
         CommandMgr::instance().deregisterCommand(CONFIG_TEST_COMMAND);
         CommandMgr::instance().deregisterCommand(CONFIG_WRITE_COMMAND);
         CommandMgr::instance().deregisterCommand(SHUT_DOWN_COMMAND);
+        CommandMgr::instance().deregisterCommand("statistic-get");
+        CommandMgr::instance().deregisterCommand("statistic-get-all");
+        CommandMgr::instance().deregisterCommand("statistic-reset");
+        CommandMgr::instance().deregisterCommand("statistic-reset-all");
         CommandMgr::instance().deregisterCommand(STATUS_GET_COMMAND);
         CommandMgr::instance().deregisterCommand(VERSION_GET_COMMAND);
 
@@ -103,8 +123,6 @@ D2Controller::deregisterCommands() {
     }
 }
 
-
-
 isc::data::ConstElementPtr
 D2Controller::parseFile(const std::string& file_name) {
     isc::data::ConstElementPtr elements;
@@ -122,6 +140,9 @@ D2Controller::parseFile(const std::string& file_name) {
 D2Controller::~D2Controller() {
 }
 
+// Refer to config_report so it will be embedded in the binary.
+const char* const* d2_config_report = isc::detail::config_report;
+
 std::string
 D2Controller::getVersionAddendum() {
     std::stringstream stream;
@@ -131,5 +152,5 @@ D2Controller::getVersionAddendum() {
 
 }
 
-}; // end namespace isc::d2
-}; // end namespace isc
+} // end namespace isc::d2
+} // end namespace isc
