@@ -658,129 +658,130 @@ namespace isc { namespace d2 {
         S_86_13 = 86,                            // $@13
         S_global_object = 87,                    // global_object
         S_88_14 = 88,                            // $@14
-        S_sub_dhcpddns = 89,                     // sub_dhcpddns
-        S_90_15 = 90,                            // $@15
-        S_dhcpddns_params = 91,                  // dhcpddns_params
-        S_dhcpddns_param = 92,                   // dhcpddns_param
-        S_ip_address = 93,                       // ip_address
-        S_94_16 = 94,                            // $@16
-        S_port = 95,                             // port
-        S_dns_server_timeout = 96,               // dns_server_timeout
-        S_ncr_protocol = 97,                     // ncr_protocol
-        S_98_17 = 98,                            // $@17
-        S_ncr_protocol_value = 99,               // ncr_protocol_value
-        S_ncr_format = 100,                      // ncr_format
-        S_101_18 = 101,                          // $@18
-        S_user_context = 102,                    // user_context
-        S_103_19 = 103,                          // $@19
-        S_comment = 104,                         // comment
-        S_105_20 = 105,                          // $@20
-        S_forward_ddns = 106,                    // forward_ddns
-        S_107_21 = 107,                          // $@21
-        S_reverse_ddns = 108,                    // reverse_ddns
-        S_109_22 = 109,                          // $@22
-        S_ddns_mgr_params = 110,                 // ddns_mgr_params
-        S_not_empty_ddns_mgr_params = 111,       // not_empty_ddns_mgr_params
-        S_ddns_mgr_param = 112,                  // ddns_mgr_param
-        S_ddns_domains = 113,                    // ddns_domains
-        S_114_23 = 114,                          // $@23
-        S_sub_ddns_domains = 115,                // sub_ddns_domains
-        S_116_24 = 116,                          // $@24
-        S_ddns_domain_list = 117,                // ddns_domain_list
-        S_not_empty_ddns_domain_list = 118,      // not_empty_ddns_domain_list
-        S_ddns_domain = 119,                     // ddns_domain
-        S_120_25 = 120,                          // $@25
-        S_sub_ddns_domain = 121,                 // sub_ddns_domain
-        S_122_26 = 122,                          // $@26
-        S_ddns_domain_params = 123,              // ddns_domain_params
-        S_ddns_domain_param = 124,               // ddns_domain_param
-        S_ddns_domain_name = 125,                // ddns_domain_name
-        S_126_27 = 126,                          // $@27
-        S_ddns_key_name = 127,                   // ddns_key_name
-        S_128_28 = 128,                          // $@28
-        S_dns_servers = 129,                     // dns_servers
-        S_130_29 = 130,                          // $@29
-        S_sub_dns_servers = 131,                 // sub_dns_servers
-        S_132_30 = 132,                          // $@30
-        S_dns_server_list = 133,                 // dns_server_list
-        S_dns_server = 134,                      // dns_server
-        S_135_31 = 135,                          // $@31
-        S_sub_dns_server = 136,                  // sub_dns_server
-        S_137_32 = 137,                          // $@32
-        S_dns_server_params = 138,               // dns_server_params
-        S_dns_server_param = 139,                // dns_server_param
-        S_dns_server_hostname = 140,             // dns_server_hostname
-        S_141_33 = 141,                          // $@33
-        S_dns_server_ip_address = 142,           // dns_server_ip_address
-        S_143_34 = 143,                          // $@34
-        S_dns_server_port = 144,                 // dns_server_port
-        S_tsig_keys = 145,                       // tsig_keys
-        S_146_35 = 146,                          // $@35
-        S_sub_tsig_keys = 147,                   // sub_tsig_keys
-        S_148_36 = 148,                          // $@36
-        S_tsig_keys_list = 149,                  // tsig_keys_list
-        S_not_empty_tsig_keys_list = 150,        // not_empty_tsig_keys_list
-        S_tsig_key = 151,                        // tsig_key
-        S_152_37 = 152,                          // $@37
-        S_sub_tsig_key = 153,                    // sub_tsig_key
-        S_154_38 = 154,                          // $@38
-        S_tsig_key_params = 155,                 // tsig_key_params
-        S_tsig_key_param = 156,                  // tsig_key_param
-        S_tsig_key_name = 157,                   // tsig_key_name
-        S_158_39 = 158,                          // $@39
-        S_tsig_key_algorithm = 159,              // tsig_key_algorithm
-        S_160_40 = 160,                          // $@40
-        S_tsig_key_digest_bits = 161,            // tsig_key_digest_bits
-        S_tsig_key_secret = 162,                 // tsig_key_secret
-        S_163_41 = 163,                          // $@41
-        S_control_socket = 164,                  // control_socket
-        S_165_42 = 165,                          // $@42
-        S_control_socket_params = 166,           // control_socket_params
-        S_control_socket_param = 167,            // control_socket_param
-        S_control_socket_type = 168,             // control_socket_type
-        S_169_43 = 169,                          // $@43
-        S_control_socket_name = 170,             // control_socket_name
-        S_171_44 = 171,                          // $@44
-        S_hooks_libraries = 172,                 // hooks_libraries
-        S_173_45 = 173,                          // $@45
-        S_hooks_libraries_list = 174,            // hooks_libraries_list
-        S_not_empty_hooks_libraries_list = 175,  // not_empty_hooks_libraries_list
-        S_hooks_library = 176,                   // hooks_library
-        S_177_46 = 177,                          // $@46
-        S_sub_hooks_library = 178,               // sub_hooks_library
-        S_179_47 = 179,                          // $@47
-        S_hooks_params = 180,                    // hooks_params
-        S_hooks_param = 181,                     // hooks_param
-        S_library = 182,                         // library
-        S_183_48 = 183,                          // $@48
-        S_parameters = 184,                      // parameters
-        S_185_49 = 185,                          // $@49
-        S_loggers = 186,                         // loggers
-        S_187_50 = 187,                          // $@50
-        S_loggers_entries = 188,                 // loggers_entries
-        S_logger_entry = 189,                    // logger_entry
-        S_190_51 = 190,                          // $@51
-        S_logger_params = 191,                   // logger_params
-        S_logger_param = 192,                    // logger_param
-        S_name = 193,                            // name
-        S_194_52 = 194,                          // $@52
-        S_debuglevel = 195,                      // debuglevel
-        S_severity = 196,                        // severity
-        S_197_53 = 197,                          // $@53
-        S_output_options_list = 198,             // output_options_list
-        S_199_54 = 199,                          // $@54
-        S_output_options_list_content = 200,     // output_options_list_content
-        S_output_entry = 201,                    // output_entry
-        S_202_55 = 202,                          // $@55
-        S_output_params_list = 203,              // output_params_list
-        S_output_params = 204,                   // output_params
-        S_output = 205,                          // output
-        S_206_56 = 206,                          // $@56
-        S_flush = 207,                           // flush
-        S_maxsize = 208,                         // maxsize
-        S_maxver = 209,                          // maxver
-        S_pattern = 210,                         // pattern
-        S_211_57 = 211                           // $@57
+        S_global_object_comma = 89,              // global_object_comma
+        S_sub_dhcpddns = 90,                     // sub_dhcpddns
+        S_91_15 = 91,                            // $@15
+        S_dhcpddns_params = 92,                  // dhcpddns_params
+        S_dhcpddns_param = 93,                   // dhcpddns_param
+        S_ip_address = 94,                       // ip_address
+        S_95_16 = 95,                            // $@16
+        S_port = 96,                             // port
+        S_dns_server_timeout = 97,               // dns_server_timeout
+        S_ncr_protocol = 98,                     // ncr_protocol
+        S_99_17 = 99,                            // $@17
+        S_ncr_protocol_value = 100,              // ncr_protocol_value
+        S_ncr_format = 101,                      // ncr_format
+        S_102_18 = 102,                          // $@18
+        S_user_context = 103,                    // user_context
+        S_104_19 = 104,                          // $@19
+        S_comment = 105,                         // comment
+        S_106_20 = 106,                          // $@20
+        S_forward_ddns = 107,                    // forward_ddns
+        S_108_21 = 108,                          // $@21
+        S_reverse_ddns = 109,                    // reverse_ddns
+        S_110_22 = 110,                          // $@22
+        S_ddns_mgr_params = 111,                 // ddns_mgr_params
+        S_not_empty_ddns_mgr_params = 112,       // not_empty_ddns_mgr_params
+        S_ddns_mgr_param = 113,                  // ddns_mgr_param
+        S_ddns_domains = 114,                    // ddns_domains
+        S_115_23 = 115,                          // $@23
+        S_sub_ddns_domains = 116,                // sub_ddns_domains
+        S_117_24 = 117,                          // $@24
+        S_ddns_domain_list = 118,                // ddns_domain_list
+        S_not_empty_ddns_domain_list = 119,      // not_empty_ddns_domain_list
+        S_ddns_domain = 120,                     // ddns_domain
+        S_121_25 = 121,                          // $@25
+        S_sub_ddns_domain = 122,                 // sub_ddns_domain
+        S_123_26 = 123,                          // $@26
+        S_ddns_domain_params = 124,              // ddns_domain_params
+        S_ddns_domain_param = 125,               // ddns_domain_param
+        S_ddns_domain_name = 126,                // ddns_domain_name
+        S_127_27 = 127,                          // $@27
+        S_ddns_key_name = 128,                   // ddns_key_name
+        S_129_28 = 129,                          // $@28
+        S_dns_servers = 130,                     // dns_servers
+        S_131_29 = 131,                          // $@29
+        S_sub_dns_servers = 132,                 // sub_dns_servers
+        S_133_30 = 133,                          // $@30
+        S_dns_server_list = 134,                 // dns_server_list
+        S_dns_server = 135,                      // dns_server
+        S_136_31 = 136,                          // $@31
+        S_sub_dns_server = 137,                  // sub_dns_server
+        S_138_32 = 138,                          // $@32
+        S_dns_server_params = 139,               // dns_server_params
+        S_dns_server_param = 140,                // dns_server_param
+        S_dns_server_hostname = 141,             // dns_server_hostname
+        S_142_33 = 142,                          // $@33
+        S_dns_server_ip_address = 143,           // dns_server_ip_address
+        S_144_34 = 144,                          // $@34
+        S_dns_server_port = 145,                 // dns_server_port
+        S_tsig_keys = 146,                       // tsig_keys
+        S_147_35 = 147,                          // $@35
+        S_sub_tsig_keys = 148,                   // sub_tsig_keys
+        S_149_36 = 149,                          // $@36
+        S_tsig_keys_list = 150,                  // tsig_keys_list
+        S_not_empty_tsig_keys_list = 151,        // not_empty_tsig_keys_list
+        S_tsig_key = 152,                        // tsig_key
+        S_153_37 = 153,                          // $@37
+        S_sub_tsig_key = 154,                    // sub_tsig_key
+        S_155_38 = 155,                          // $@38
+        S_tsig_key_params = 156,                 // tsig_key_params
+        S_tsig_key_param = 157,                  // tsig_key_param
+        S_tsig_key_name = 158,                   // tsig_key_name
+        S_159_39 = 159,                          // $@39
+        S_tsig_key_algorithm = 160,              // tsig_key_algorithm
+        S_161_40 = 161,                          // $@40
+        S_tsig_key_digest_bits = 162,            // tsig_key_digest_bits
+        S_tsig_key_secret = 163,                 // tsig_key_secret
+        S_164_41 = 164,                          // $@41
+        S_control_socket = 165,                  // control_socket
+        S_166_42 = 166,                          // $@42
+        S_control_socket_params = 167,           // control_socket_params
+        S_control_socket_param = 168,            // control_socket_param
+        S_control_socket_type = 169,             // control_socket_type
+        S_170_43 = 170,                          // $@43
+        S_control_socket_name = 171,             // control_socket_name
+        S_172_44 = 172,                          // $@44
+        S_hooks_libraries = 173,                 // hooks_libraries
+        S_174_45 = 174,                          // $@45
+        S_hooks_libraries_list = 175,            // hooks_libraries_list
+        S_not_empty_hooks_libraries_list = 176,  // not_empty_hooks_libraries_list
+        S_hooks_library = 177,                   // hooks_library
+        S_178_46 = 178,                          // $@46
+        S_sub_hooks_library = 179,               // sub_hooks_library
+        S_180_47 = 180,                          // $@47
+        S_hooks_params = 181,                    // hooks_params
+        S_hooks_param = 182,                     // hooks_param
+        S_library = 183,                         // library
+        S_184_48 = 184,                          // $@48
+        S_parameters = 185,                      // parameters
+        S_186_49 = 186,                          // $@49
+        S_loggers = 187,                         // loggers
+        S_188_50 = 188,                          // $@50
+        S_loggers_entries = 189,                 // loggers_entries
+        S_logger_entry = 190,                    // logger_entry
+        S_191_51 = 191,                          // $@51
+        S_logger_params = 192,                   // logger_params
+        S_logger_param = 193,                    // logger_param
+        S_name = 194,                            // name
+        S_195_52 = 195,                          // $@52
+        S_debuglevel = 196,                      // debuglevel
+        S_severity = 197,                        // severity
+        S_198_53 = 198,                          // $@53
+        S_output_options_list = 199,             // output_options_list
+        S_200_54 = 200,                          // $@54
+        S_output_options_list_content = 201,     // output_options_list_content
+        S_output_entry = 202,                    // output_entry
+        S_203_55 = 203,                          // $@55
+        S_output_params_list = 204,              // output_params_list
+        S_output_params = 205,                   // output_params
+        S_output = 206,                          // output
+        S_207_56 = 207,                          // $@56
+        S_flush = 208,                           // flush
+        S_maxsize = 209,                         // maxsize
+        S_maxver = 210,                          // maxver
+        S_pattern = 211,                         // pattern
+        S_212_57 = 212                           // $@57
       };
     };
 
@@ -2165,7 +2166,7 @@ switch (yykind)
     // YYDEFACT[STATE-NUM] -- Default reduction number in state STATE-NUM.
     // Performed when YYTABLE does not specify something else to do.  Zero
     // means the default is an error.
-    static const unsigned char yydefact_[];
+    static const short yydefact_[];
 
     // YYPGOTO[NTERM-NUM].
     static const short yypgoto_[];
@@ -2421,7 +2422,7 @@ switch (yykind)
     enum
     {
       yylast_ = 307,     ///< Last index in yytable_.
-      yynnts_ = 151,  ///< Number of nonterminal symbols.
+      yynnts_ = 152,  ///< Number of nonterminal symbols.
       yyfinal_ = 22 ///< Termination state number.
     };
 
@@ -2636,7 +2637,7 @@ switch (yykind)
 
 #line 14 "d2_parser.yy"
 } } // isc::d2
-#line 2640 "d2_parser.h"
+#line 2641 "d2_parser.h"
 
 
 
