@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -7,14 +7,15 @@
 #include <config.h>
 
 #include <asiolink/io_service.h>
+#include <d2srv/testutils/nc_test_utils.h>
 #include <d2/d2_update_mgr.h>
+#include <d2/nc_add.h>
+#include <d2/nc_remove.h>
+#include <d2/simple_add.h>
+#include <d2/simple_remove.h>
+#include <process/testutils/d_test_stubs.h>
 #include <util/time_utilities.h>
-#include <d_test_stubs.h>
-#include <nc_test_utils.h>
 
-#include <boost/function.hpp>
-#include <boost/bind.hpp>
-#include <gtest/gtest.h>
 #include <gtest/gtest.h>
 #include <algorithm>
 #include <vector>
@@ -23,6 +24,8 @@ using namespace std;
 using namespace isc;
 using namespace isc::dhcp_ddns;
 using namespace isc::d2;
+using namespace isc::process;
+using namespace isc::util;
 
 namespace {
 
@@ -95,10 +98,11 @@ public:
         " \"ip-address\" : \"192.168.1.2\" , "
         " \"dhcid\" : \"0102030405060708\" , "
         " \"lease-expires-on\" : \"20130121132405\" , "
-        " \"lease-length\" : 1300 "
+        " \"lease-length\" : 1300, "
+        " \"use-conflict-resolution\" : true "
         "}";
 
-        const char* dhcids[] = { "111111", "222222", "333333", "444444"};
+        const char* dhcids[] = { "111111", "222222", "333333", "444444" };
         canned_count_ = 4;
         for (int i = 0; i < canned_count_; i++) {
             dhcp_ddns::NameChangeRequestPtr ncr = NameChangeRequest::
@@ -118,31 +122,31 @@ public:
                   "\"port\" : 88 , "
                   "\"tsig-keys\": [] ,"
                   "\"forward-ddns\" : {"
-                  "\"ddns-domains\": [ "
-                  "{ \"name\": \"example.com.\" , "
-                  "  \"dns-servers\" : [ "
-                  "  { \"ip-address\": \"127.0.0.1\", \"port\" : 5301  } "
-                  "  ] },"
-                  "{ \"name\": \"org.\" , "
-                  "  \"dns-servers\" : [ "
-                  "  { \"ip-address\": \"127.0.0.1\" } "
+                  " \"ddns-domains\": [ "
+                  "  { \"name\": \"example.com.\" , "
+                  "   \"dns-servers\" : [ "
+                  "    { \"ip-address\": \"127.0.0.1\", \"port\" : 5301  } "
+                  "   ] },"
+                  "  { \"name\": \"org.\" , "
+                  "   \"dns-servers\" : [ "
+                  "    { \"ip-address\": \"127.0.0.1\" } "
+                  "    ] }"
                   "  ] }, "
-                  "] }, "
                   "\"reverse-ddns\" : { "
-                  "\"ddns-domains\": [ "
-                  "{ \"name\": \"1.168.192.in-addr.arpa.\" , "
-                  "  \"dns-servers\" : [ "
-                  "  { \"ip-address\": \"127.0.0.1\", \"port\" : 5301 } "
-                  "  ] }, "
-                  "{ \"name\": \"2.0.3.0.8.B.D.0.1.0.0.2.ip6.arpa.\" , "
-                  "  \"dns-servers\" : [ "
-                  "  { \"ip-address\": \"127.0.0.1\" } "
-                  "  ] } "
-                  "] } }";
+                  " \"ddns-domains\": [ "
+                  "  { \"name\": \"1.168.192.in-addr.arpa.\" , "
+                  "   \"dns-servers\" : [ "
+                  "    { \"ip-address\": \"127.0.0.1\", \"port\" : 5301 } "
+                  "   ] }, "
+                  "  { \"name\": \"2.0.3.0.8.B.D.0.1.0.0.2.ip6.arpa.\" , "
+                  "   \"dns-servers\" : [ "
+                  "    { \"ip-address\": \"127.0.0.1\" } "
+                  "    ] } "
+                  "  ] } }";
 
         // If this configuration fails to parse most tests will fail.
         ASSERT_TRUE(fromJSON(canned_config_));
-        answer_ = cfg_mgr_->parseConfig(config_set_);
+        answer_ = cfg_mgr_->simpleParseConfig(config_set_);
         ASSERT_TRUE(checkAnswer(0));
     }
 
@@ -228,8 +232,8 @@ public:
             // value).  This is roughly ten times the number for the longest
             // test (currently, multiTransactionTimeout).
             if (passes > max_passes) {
-                ADD_FAILURE() << "processALL failed, too many passes: "
-                    << passes <<  ", total handlers executed: " << handlers;
+                FAIL() << "processALL failed, too many passes: "
+                       << passes <<  ", total handlers executed: " << handlers;
             }
         }
     }
@@ -483,7 +487,7 @@ TEST_F(D2UpdateMgrTest, checkFinishedTransaction) {
     // Verify that the list of transactions has decreased by two.
     EXPECT_EQ(canned_count_ - 2, update_mgr_->getTransactionCount());
 
-    // Vefity that the transaction list is correct.
+    // Verify that the transaction list is correct.
     EXPECT_TRUE(update_mgr_->hasTransaction(canned_ncrs_[0]->getDhcid()));
     EXPECT_FALSE(update_mgr_->hasTransaction(canned_ncrs_[1]->getDhcid()));
     EXPECT_TRUE(update_mgr_->hasTransaction(canned_ncrs_[2]->getDhcid()));
@@ -675,6 +679,10 @@ TEST_F(D2UpdateMgrTest, addTransaction) {
     NameChangeTransactionPtr trans = (*pos).second;
     ASSERT_TRUE(trans);
 
+    // Verify the correct type of transaction was created.
+    NameAddTransaction* t = dynamic_cast<NameAddTransaction*>(trans.get());
+    ASSERT_TRUE(t);
+
     // At this point the transaction should have constructed
     // and sent the DNS request.
     ASSERT_TRUE(trans->getCurrentServer());
@@ -728,6 +736,10 @@ TEST_F(D2UpdateMgrTest, removeTransaction) {
     NameChangeTransactionPtr trans = (*pos).second;
     ASSERT_TRUE(trans);
 
+    // Verify the correct type of transaction was created.
+    NameRemoveTransaction* t = dynamic_cast<NameRemoveTransaction*>(trans.get());
+    ASSERT_TRUE(t);
+
     // At this point the transaction should have constructed
     // and sent the DNS request.
     ASSERT_TRUE(trans->getCurrentServer());
@@ -858,4 +870,120 @@ TEST_F(D2UpdateMgrTest, multiTransactionTimeout) {
     }
 }
 
+/// @brief Tests integration of SimpleAddTransaction
+/// This test verifies that update manager can create and manage a
+/// SimpleAddTransaction from start to finish.  It utilizes a fake server
+/// which responds to all requests sent with NOERROR, simulating a
+/// successful addition.  The transaction processes both forward and
+/// reverse changes.
+TEST_F(D2UpdateMgrTest, simpleAddTransaction) {
+    // Put each transaction on the queue.
+    canned_ncrs_[0]->setChangeType(dhcp_ddns::CHG_ADD);
+    canned_ncrs_[0]->setReverseChange(true);
+    canned_ncrs_[0]->setConflictResolution(false);
+    ASSERT_NO_THROW(queue_mgr_->enqueue(canned_ncrs_[0]));
+
+    // Call sweep once, this should:
+    // 1. Dequeue the request
+    // 2. Create the transaction
+    // 3. Start the transaction
+    ASSERT_NO_THROW(update_mgr_->sweep());
+
+    // Get a copy of the transaction.
+    TransactionList::iterator pos = update_mgr_->transactionListBegin();
+    ASSERT_TRUE (pos != update_mgr_->transactionListEnd());
+    NameChangeTransactionPtr trans = (*pos).second;
+    ASSERT_TRUE(trans);
+
+    // Verify the correct type of transaction was created.
+    SimpleAddTransaction* t = dynamic_cast<SimpleAddTransaction*>(trans.get());
+    ASSERT_TRUE(t);
+
+    // At this point the transaction should have constructed
+    // and sent the DNS request.
+    ASSERT_TRUE(trans->getCurrentServer());
+    ASSERT_TRUE(trans->isModelRunning());
+    ASSERT_EQ(1, trans->getUpdateAttempts());
+    ASSERT_EQ(StateModel::NOP_EVT, trans->getNextEvent());
+
+    // Create a server based on the transaction's current server, and
+    // start it listening.
+    FauxServer server(*io_service_, *(trans->getCurrentServer()));
+    server.receive(FauxServer::USE_RCODE, dns::Rcode::NOERROR());
+
+    // Run sweep and IO until everything is done.
+    processAll();
+
+    // Verify that model succeeded.
+    EXPECT_FALSE(trans->didModelFail());
+
+    // Both completion flags should be true.
+    EXPECT_TRUE(trans->getForwardChangeCompleted());
+    EXPECT_TRUE(trans->getReverseChangeCompleted());
+
+    // Verify that we went through success state.
+    EXPECT_EQ(NameChangeTransaction::PROCESS_TRANS_OK_ST,
+              trans->getPrevState());
+    EXPECT_EQ(NameChangeTransaction::UPDATE_OK_EVT,
+              trans->getLastEvent());
+}
+
+/// @brief Tests integration of SimpleRemoveTransaction
+/// This test verifies that update manager can create and manage a
+/// SimpleRemoveTransaction from start to finish.  It utilizes a fake server
+/// which responds to all requests sent with NOERROR, simulating a
+/// successful addition.  The transaction processes both forward and
+/// reverse changes.
+TEST_F(D2UpdateMgrTest, simpleRemoveTransaction) {
+    // Put each transaction on the queue.
+    canned_ncrs_[0]->setChangeType(dhcp_ddns::CHG_REMOVE);
+    canned_ncrs_[0]->setReverseChange(true);
+    canned_ncrs_[0]->setConflictResolution(false);
+    ASSERT_NO_THROW(queue_mgr_->enqueue(canned_ncrs_[0]));
+
+    // Call sweep once, this should:
+    // 1. Dequeue the request
+    // 2. Create the transaction
+    // 3. Start the transaction
+    ASSERT_NO_THROW(update_mgr_->sweep());
+
+    // Get a copy of the transaction.
+    TransactionList::iterator pos = update_mgr_->transactionListBegin();
+    ASSERT_TRUE (pos != update_mgr_->transactionListEnd());
+    NameChangeTransactionPtr trans = (*pos).second;
+    ASSERT_TRUE(trans);
+
+    // Verify the correct type of transaction was created.
+    SimpleRemoveTransaction* t = dynamic_cast<SimpleRemoveTransaction*>(trans.get());
+    ASSERT_TRUE(t);
+
+    // At this point the transaction should have constructed
+    // and sent the DNS request.
+    ASSERT_TRUE(trans->getCurrentServer());
+    ASSERT_TRUE(trans->isModelRunning());
+    ASSERT_EQ(1, trans->getUpdateAttempts());
+    ASSERT_EQ(StateModel::NOP_EVT, trans->getNextEvent());
+
+    // Create a server based on the transaction's current server,
+    // and start it listening.
+    FauxServer server(*io_service_, *(trans->getCurrentServer()));
+    server.receive(FauxServer::USE_RCODE, dns::Rcode::NOERROR());
+
+    // Run sweep and IO until everything is done.
+    processAll();
+
+    // Verify that model succeeded.
+    EXPECT_FALSE(trans->didModelFail());
+
+    // Both completion flags should be true.
+    EXPECT_TRUE(trans->getForwardChangeCompleted());
+    EXPECT_TRUE(trans->getReverseChangeCompleted());
+
+    // Verify that we went through success state.
+    EXPECT_EQ(NameChangeTransaction::PROCESS_TRANS_OK_ST,
+              trans->getPrevState());
+    EXPECT_EQ(NameChangeTransaction::UPDATE_OK_EVT,
+              trans->getLastEvent());
+}
+
 }
