@@ -9,11 +9,16 @@
 #include <d2/parser_context.h>
 #include <d2/tests/parser_unittest.h>
 #include <testutils/io_utils.h>
+#include <testutils/log_utils.h>
 #include <testutils/user_context_utils.h>
+
 #include <gtest/gtest.h>
+
 #include <fstream>
 #include <set>
 
+#include <boost/algorithm/string.hpp>
+
 #include "test_data_files_config.h"
 
 using namespace isc::data;
@@ -49,8 +54,9 @@ void compareJSON(ConstElementPtr a, ConstElementPtr b) {
 /// @param compare whether to compare the output with legacy JSON parser
 void testParser(const std::string& txt, D2ParserContext::ParserType parser_type,
     bool compare = true) {
-    ConstElementPtr test_json;
+    SCOPED_TRACE("\n=== tested config ===\n" + txt + "=====================");
 
+    ConstElementPtr test_json;
     ASSERT_NO_THROW({
             try {
                 D2ParserContext ctx;
@@ -291,8 +297,9 @@ TEST(ParserTest, file) {
 /// @param msg expected content of the exception
 void testError(const std::string& txt,
                D2ParserContext::ParserType parser_type,
-               const std::string& msg)
-{
+               const std::string& msg) {
+    SCOPED_TRACE("\n=== tested config ===\n" + txt + "=====================");
+
     try {
         D2ParserContext ctx;
         ConstElementPtr parsed = ctx.parseString(txt, parser_type);
@@ -531,10 +538,6 @@ TEST(ParserTest, errors) {
               D2ParserContext::PARSER_JSON,
               "<string>:1.3: syntax error, unexpected \",\", "
               "expecting }");
-    testError("{ \"foo\":true, }\n",
-              D2ParserContext::PARSER_JSON,
-              "<string>:1.15: syntax error, unexpected }, "
-              "expecting constant string");
 
     // bad type
     testError("{ \"DhcpDdns\":{\n"
@@ -595,7 +598,7 @@ TEST(ParserTest, errors) {
               "  \"DhcpDdns\":{\n"
               "  \"comment\": \"second\" }}\n",
               D2ParserContext::PARSER_DHCPDDNS,
-              "<string>:2.23: syntax error, unexpected \",\", expecting }");
+              "<string>:3.3-12: syntax error, unexpected DhcpDdns, expecting \",\" or }");
 
     // duplicate of not string entries
     testError("{ \"DhcpDdns\":{\n"
@@ -805,6 +808,64 @@ TEST(ParserTest, duplicateMapEntries) {
     cout << "checked " << cnt << " duplicated map entries\n";
 }
 
+/// @brief Test fixture for trailing commas.
+class TrailingCommasTest : public isc::dhcp::test::LogContentTest {
+public:
+    /// @brief Add a log entry.
+    ///
+    /// @param loc Location of the trailing comma.
+    void addLog(const string& loc) {
+        string log = "DHCP_DDNS_CONFIG_SYNTAX_WARNING DHCP-DDNS server ";
+        log += "configuration syntax warning: " + loc;
+        log += ": Extraneous comma. ";
+        log += "A piece of configuration may have been omitted.";
+        addString(log);
+    }
+};
+
+// Test that trailing commas are allowed.
+TEST_F(TrailingCommasTest, tests) {
+    string txt(R"({
+  "DhcpDdns": {
+    "forward-ddns": {},
+    "ip-address": "127.0.0.1",
+    "loggers": [
+      {
+        "name": "kea-dhcp-ddns",
+        "output_options": [
+          {
+            "output": "stdout"
+          },
+        ],
+        "severity": "DEBUG",
+      },
+    ],
+    "port": 53001,
+    "reverse-ddns": {},
+    "tsig-keys": [
+      {
+        "algorithm": "HMAC-MD5",
+        "name": "d2.md5.key",
+        "secret": "sensitivejdPJI5QxlpnfQ==",
+      },
+    ]
+  },
+})");
+    testParser(txt, D2ParserContext::PARSER_DHCPDDNS, false);
+
+    addLog("<string>:11.12");
+    addLog("<string>:13.28");
+    addLog("<string>:14.8");
+    addLog("<string>:22.45");
+    addLog("<string>:23.8");
+    addLog("<string>:25.4");
+    EXPECT_TRUE(checkFile());
+
+    // Test with many consecutive commas.
+    boost::replace_all(txt, ",", ",,,,");
+    testParser(txt, D2ParserContext::PARSER_DHCPDDNS, false);
 }
-}
-}
+
+}  // namespace test
+}  // namespace d2
+}  // namespace isc
