@@ -1,4 +1,4 @@
-// File created from ../../../src/bin/dhcp4/dhcp4_messages.mes on Fri Aug 14 2020 21:33
+// File created from ../../../src/bin/dhcp4/dhcp4_messages.mes
 
 #ifndef DHCP4_MESSAGES_H
 #define DHCP4_MESSAGES_H
@@ -37,14 +37,15 @@ extern const isc::log::MessageID DHCP4_CONFIG_OPTION_DUPLICATE;
 extern const isc::log::MessageID DHCP4_CONFIG_PACKET_QUEUE;
 extern const isc::log::MessageID DHCP4_CONFIG_RECEIVED;
 extern const isc::log::MessageID DHCP4_CONFIG_START;
+extern const isc::log::MessageID DHCP4_CONFIG_SYNTAX_WARNING;
 extern const isc::log::MessageID DHCP4_CONFIG_UNRECOVERABLE_ERROR;
 extern const isc::log::MessageID DHCP4_CONFIG_UNSUPPORTED_OBJECT;
 extern const isc::log::MessageID DHCP4_CONFIG_UPDATE;
-extern const isc::log::MessageID DHCP4_DB_RECONNECT_ATTEMPT_FAILED;
-extern const isc::log::MessageID DHCP4_DB_RECONNECT_ATTEMPT_SCHEDULE;
 extern const isc::log::MessageID DHCP4_DB_RECONNECT_DISABLED;
+extern const isc::log::MessageID DHCP4_DB_RECONNECT_FAILED;
+extern const isc::log::MessageID DHCP4_DB_RECONNECT_LOST_CONNECTION;
 extern const isc::log::MessageID DHCP4_DB_RECONNECT_NO_DB_CTL;
-extern const isc::log::MessageID DHCP4_DB_RECONNECT_RETRIES_EXHAUSTED;
+extern const isc::log::MessageID DHCP4_DB_RECONNECT_SUCCEEDED;
 extern const isc::log::MessageID DHCP4_DDNS_REQUEST_SEND_FAILED;
 extern const isc::log::MessageID DHCP4_DEACTIVATE_INTERFACE;
 extern const isc::log::MessageID DHCP4_DECLINE_FAIL;
@@ -63,6 +64,7 @@ extern const isc::log::MessageID DHCP4_DHCP4O6_RECEIVING;
 extern const isc::log::MessageID DHCP4_DHCP4O6_RESPONSE_DATA;
 extern const isc::log::MessageID DHCP4_DYNAMIC_RECONFIGURATION;
 extern const isc::log::MessageID DHCP4_DYNAMIC_RECONFIGURATION_FAIL;
+extern const isc::log::MessageID DHCP4_DYNAMIC_RECONFIGURATION_SUCCESS;
 extern const isc::log::MessageID DHCP4_EMPTY_HOSTNAME;
 extern const isc::log::MessageID DHCP4_FLEX_ID;
 extern const isc::log::MessageID DHCP4_GENERATE_FQDN;
@@ -71,10 +73,12 @@ extern const isc::log::MessageID DHCP4_HOOKS_LIBS_RELOAD_FAIL;
 extern const isc::log::MessageID DHCP4_HOOK_BUFFER_RCVD_DROP;
 extern const isc::log::MessageID DHCP4_HOOK_BUFFER_RCVD_SKIP;
 extern const isc::log::MessageID DHCP4_HOOK_BUFFER_SEND_SKIP;
+extern const isc::log::MessageID DHCP4_HOOK_DDNS_UPDATE;
 extern const isc::log::MessageID DHCP4_HOOK_DECLINE_SKIP;
 extern const isc::log::MessageID DHCP4_HOOK_LEASE4_RELEASE_SKIP;
 extern const isc::log::MessageID DHCP4_HOOK_LEASES4_COMMITTED_DROP;
 extern const isc::log::MessageID DHCP4_HOOK_LEASES4_COMMITTED_PARK;
+extern const isc::log::MessageID DHCP4_HOOK_LEASES4_PARKING_LOT_FULL;
 extern const isc::log::MessageID DHCP4_HOOK_PACKET_RCVD_SKIP;
 extern const isc::log::MessageID DHCP4_HOOK_PACKET_SEND_DROP;
 extern const isc::log::MessageID DHCP4_HOOK_PACKET_SEND_SKIP;
@@ -85,15 +89,16 @@ extern const isc::log::MessageID DHCP4_INIT_FAIL;
 extern const isc::log::MessageID DHCP4_INIT_REBOOT;
 extern const isc::log::MessageID DHCP4_LEASE_ADVERT;
 extern const isc::log::MessageID DHCP4_LEASE_ALLOC;
+extern const isc::log::MessageID DHCP4_LEASE_REUSE;
 extern const isc::log::MessageID DHCP4_MULTI_THREADING_INFO;
-extern const isc::log::MessageID DHCP4_MULTI_THREADING_WARNING;
-extern const isc::log::MessageID DHCP4_NCR_CREATE;
 extern const isc::log::MessageID DHCP4_NCR_CREATION_FAILED;
 extern const isc::log::MessageID DHCP4_NOT_RUNNING;
 extern const isc::log::MessageID DHCP4_NO_LEASE_INIT_REBOOT;
 extern const isc::log::MessageID DHCP4_NO_SOCKETS_OPEN;
 extern const isc::log::MessageID DHCP4_OPEN_CONFIG_DB;
 extern const isc::log::MessageID DHCP4_OPEN_SOCKET;
+extern const isc::log::MessageID DHCP4_OPEN_SOCKETS_FAILED;
+extern const isc::log::MessageID DHCP4_OPEN_SOCKETS_NO_RECONNECT_CTL;
 extern const isc::log::MessageID DHCP4_OPEN_SOCKET_FAIL;
 extern const isc::log::MessageID DHCP4_PACKET_DROP_0001;
 extern const isc::log::MessageID DHCP4_PACKET_DROP_0002;
@@ -107,6 +112,8 @@ extern const isc::log::MessageID DHCP4_PACKET_DROP_0009;
 extern const isc::log::MessageID DHCP4_PACKET_DROP_0010;
 extern const isc::log::MessageID DHCP4_PACKET_DROP_0011;
 extern const isc::log::MessageID DHCP4_PACKET_DROP_0012;
+extern const isc::log::MessageID DHCP4_PACKET_DROP_0013;
+extern const isc::log::MessageID DHCP4_PACKET_DROP_0014;
 extern const isc::log::MessageID DHCP4_PACKET_NAK_0001;
 extern const isc::log::MessageID DHCP4_PACKET_NAK_0002;
 extern const isc::log::MessageID DHCP4_PACKET_NAK_0003;
@@ -133,6 +140,7 @@ extern const isc::log::MessageID DHCP4_RELEASE_EXCEPTION;
 extern const isc::log::MessageID DHCP4_RELEASE_FAIL;
 extern const isc::log::MessageID DHCP4_RELEASE_FAIL_NO_LEASE;
 extern const isc::log::MessageID DHCP4_RELEASE_FAIL_WRONG_CLIENT;
+extern const isc::log::MessageID DHCP4_RESERVATIONS_LOOKUP_FIRST_ENABLED;
 extern const isc::log::MessageID DHCP4_RESERVED_HOSTNAME_ASSIGNED;
 extern const isc::log::MessageID DHCP4_RESPONSE_DATA;
 extern const isc::log::MessageID DHCP4_RESPONSE_FQDN_DATA;
@@ -153,6 +161,7 @@ extern const isc::log::MessageID DHCP4_SUBNET_DYNAMICALLY_CHANGED;
 extern const isc::log::MessageID DHCP4_SUBNET_SELECTED;
 extern const isc::log::MessageID DHCP4_SUBNET_SELECTION_FAILED;
 extern const isc::log::MessageID DHCP4_TESTING_MODE_SEND_TO_SOURCE_ENABLED;
+extern const isc::log::MessageID DHCP4_UNKNOWN_ADDRESS_REQUESTED;
 extern const isc::log::MessageID DHCP6_DHCP4O6_PACKET_RECEIVED;
 
 } // namespace dhcp
