@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -12,11 +12,13 @@
 #include <dhcp/tests/iface_mgr_test_config.h>
 #include <dhcpsrv/lease.h>
 #include <dhcp4/tests/dhcp4_client.h>
+#include <util/multi_threading_mgr.h>
 #include <util/range_utilities.h>
 #include <boost/pointer_cast.hpp>
 #include <cstdlib>
 
 using namespace isc::asiolink;
+using namespace isc::util;
 
 namespace isc {
 namespace dhcp {
@@ -235,7 +237,9 @@ Dhcp4Client::appendExtraOptions() {
     if (!extra_options_.empty()) {
         for (OptionCollection::iterator opt = extra_options_.begin();
              opt != extra_options_.end(); ++opt) {
-            context_.query_->addOption(opt->second);
+            // Call base class function so that unittests can add multiple
+            // options with the same code.
+            context_.query_->Pkt::addOption(opt->second);
         }
     }
 }
@@ -449,10 +453,13 @@ Dhcp4Client::includeFQDN(const uint8_t flags, const std::string& fqdn_name,
 
 void
 Dhcp4Client::includeHostname(const std::string& name) {
-    hostname_.reset(new OptionString(Option::V4, DHO_HOST_NAME, name));
+    if (name.empty()) {
+       hostname_.reset();
+    } else {
+        hostname_.reset(new OptionString(Option::V4, DHO_HOST_NAME, name));
+    }
 }
 
-
 HWAddrPtr
 Dhcp4Client::generateHWAddr(const uint8_t htype) const {
     if (htype != HTYPE_ETHER) {
@@ -494,12 +501,10 @@ Dhcp4Client::requestOptions(const uint8_t option1, const uint8_t option2,
 
 Pkt4Ptr
 Dhcp4Client::receiveOneMsg() {
-    // Return empty pointer if server hasn't responded.
-    if (srv_->fake_sent_.empty()) {
+    Pkt4Ptr msg = srv_->receiveOneMsg();
+    if (!msg) {
         return (Pkt4Ptr());
     }
-    Pkt4Ptr msg = srv_->fake_sent_.front();
-    srv_->fake_sent_.pop_front();
 
     // Copy the original message to simulate reception over the wire.
     msg->pack();
@@ -522,18 +527,23 @@ void
 Dhcp4Client::sendMsg(const Pkt4Ptr& msg) {
     srv_->shutdown_ = false;
     if (use_relay_) {
-        msg->setHops(1);
-        msg->setGiaddr(relay_addr_);
-        msg->setLocalAddr(server_facing_relay_addr_);
-        // Insert RAI
-        OptionPtr rai(new Option(Option::V4, DHO_DHCP_AGENT_OPTIONS));
-        // Insert circuit id, if specified.
-        if (!circuit_id_.empty()) {
-            rai->addOption(OptionPtr(new Option(Option::V4, RAI_OPTION_AGENT_CIRCUIT_ID,
-                                                OptionBuffer(circuit_id_.begin(),
-                                                             circuit_id_.end()))));
+        try {
+            msg->setHops(1);
+            msg->setGiaddr(relay_addr_);
+            msg->setLocalAddr(server_facing_relay_addr_);
+            // Insert RAI
+            OptionPtr rai(new Option(Option::V4, DHO_DHCP_AGENT_OPTIONS));
+            // Insert circuit id, if specified.
+            if (!circuit_id_.empty()) {
+                rai->addOption(OptionPtr(new Option(Option::V4, RAI_OPTION_AGENT_CIRCUIT_ID,
+                                                    OptionBuffer(circuit_id_.begin(),
+                                                                 circuit_id_.end()))));
+            }
+            msg->addOption(rai);
+        } catch (...) {
+            // If relay options have already been added in the unittest, ignore
+            // exception on add.
         }
-        msg->addOption(rai);
     }
     // Repack the message to simulate wire-data parsing.
     msg->pack();
@@ -559,6 +569,9 @@ Dhcp4Client::sendMsg(const Pkt4Ptr& msg) {
     } catch (...) {
         // Suppress errors, as the DHCPv4 server does.
     }
+
+    // Make sure the server processed all packets in MT.
+    isc::util::MultiThreadingMgr::instance().getThreadPool().wait(3);
 }
 
 void
