@@ -1,4 +1,4 @@
-// Copyright (C) 2016-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -10,11 +10,16 @@
 #include <dhcpsrv/parsers/simple_parser4.h>
 #include <testutils/gtest_utils.h>
 #include <testutils/io_utils.h>
+#include <testutils/log_utils.h>
 #include <testutils/user_context_utils.h>
+
 #include <gtest/gtest.h>
+
 #include <fstream>
 #include <set>
 
+#include <boost/algorithm/string.hpp>
+
 using namespace isc::data;
 using namespace isc::test;
 using namespace std;
@@ -48,8 +53,9 @@ void compareJSON(ConstElementPtr a, ConstElementPtr b) {
 /// @param compare whether to compare the output with legacy JSON parser
 void testParser(const std::string& txt, Parser4Context::ParserType parser_type,
                 bool compare = true) {
-    ConstElementPtr test_json;
+    SCOPED_TRACE("\n=== tested config ===\n" + txt + "=====================");
 
+    ConstElementPtr test_json;
     ASSERT_NO_THROW({
             try {
                 Parser4Context ctx;
@@ -270,7 +276,6 @@ TEST(ParserTest, file) {
                                "all-keys.json",
                                "all-options.json",
                                "backends.json",
-                               "cassandra.json",
                                "classify.json",
                                "classify2.json",
                                "comments.json",
@@ -331,8 +336,9 @@ TEST(ParserTest, outboundIface) {
 /// @param msg expected content of the exception
 void testError(const std::string& txt,
                Parser4Context::ParserType parser_type,
-               const std::string& msg)
-{
+               const std::string& msg) {
+    SCOPED_TRACE("\n=== tested config ===\n" + txt + "=====================");
+
     try {
         Parser4Context ctx;
         ConstElementPtr parsed = ctx.parseString(txt, parser_type);
@@ -575,10 +581,6 @@ TEST(ParserTest, errors) {
               Parser4Context::PARSER_JSON,
               "<string>:1.3: syntax error, unexpected \",\", "
               "expecting }");
-    testError("{ \"foo\":true, }\n",
-              Parser4Context::PARSER_JSON,
-              "<string>:1.15: syntax error, unexpected }, "
-              "expecting constant string");
 
     // bad type
     testError("{ \"Dhcp4\":{\n"
@@ -646,7 +648,7 @@ TEST(ParserTest, errors) {
               "  \"Dhcp4\":{\n"
               "  \"comment\": \"second\" }}\n",
               Parser4Context::PARSER_DHCP4,
-              "<string>:2.23: syntax error, unexpected \",\", expecting }");
+              "<string>:3.3-9: syntax error, unexpected Dhcp4, expecting \",\" or }");
 
     // duplicate of not string entries
     testError("{ \"Dhcp4\":{\n"
@@ -762,7 +764,8 @@ TEST(ParserTest, mapEntries) {
     loadFile(sample_dir + "reservations.json", sample_json);
     loadFile(sample_dir + "all-keys-netconf.json", sample_json);
     KeywordSet sample_keys = {
-        "hosts-database"
+        "hosts-database",
+        "reservation-mode"
     };
     // Recursively extract keywords.
     static void (*extract)(ConstElementPtr, KeywordSet&) =
@@ -873,6 +876,98 @@ TEST(ParserTest, duplicateMapEntries) {
     cout << "checked " << cnt << " duplicated map entries\n";
 }
 
+/// @brief Test fixture for trailing commas.
+class TrailingCommasTest : public isc::dhcp::test::LogContentTest {
+public:
+    /// @brief Add a log entry.
+    ///
+    /// @param loc Location of the trailing comma.
+    void addLog(const string& loc) {
+        string log = "DHCP4_CONFIG_SYNTAX_WARNING configuration syntax ";
+        log += "warning: " + loc;
+        log += ": Extraneous comma. ";
+        log += "A piece of configuration may have been omitted.";
+        addString(log);
+    }
+};
+
+// Test that trailing commas are allowed.
+TEST_F(TrailingCommasTest, tests) {
+    string txt(R"({
+  "Dhcp4": {
+    "control-socket": {
+      "socket-name": "/tmp/kea-dhcp4-ctrl.sock",
+      "socket-type": "unix",
+    },
+    "hooks-libraries": [
+      {
+        "library": "/usr/local/lib/kea/hooks/libdhcp_dummy.so",
+      },
+    ],
+    "interfaces-config": {
+      "interfaces": [
+        "eth0",
+      ],
+    },
+    "lease-database": {
+      "name": "/tmp/kea-dhcp4.csv",
+      "persist": true,
+      "type": "memfile",
+    },
+    "loggers": [
+      {
+        "debuglevel": 99,
+        "name": "kea-dhcp4",
+        "output_options": [
+          {
+            "output": "stdout",
+          },
+        ],
+        "severity": "DEBUG",
+      },
+    ],
+    "multi-threading": {
+      "enable-multi-threading": false,
+      "packet-queue-size": 0,
+      "thread-pool-size": 0
+    },
+    "subnet4": [
+      {
+        "pools": [
+          {
+            "pool": "192.168.0.0/24",
+          },
+        ],
+        "subnet": "192.168.0.0/24",
+      },
+    ],
+  },
+})");
+    testParser(txt, Parser4Context::PARSER_DHCP4, false);
+
+    addLog("<string>:5.28");
+    addLog("<string>:9.63");
+    addLog("<string>:10.8");
+    addLog("<string>:14.15");
+    addLog("<string>:15.8");
+    addLog("<string>:20.24");
+    addLog("<string>:28.31");
+    addLog("<string>:29.12");
+    addLog("<string>:31.28");
+    addLog("<string>:32.8");
+    addLog("<string>:43.37");
+    addLog("<string>:44.12");
+    addLog("<string>:46.35");
+    addLog("<string>:47.8");
+    addLog("<string>:48.6");
+    addLog("<string>:49.4");
+    EXPECT_TRUE(checkFile());
+
+    // Test with many consecutive commas.
+    boost::replace_all(txt, ",", ",,,,");
+    testParser(txt, Parser4Context::PARSER_DHCP4, false);
 }
-}
-}
+
+}  // namespace test
+}  // namespace dhcp
+}  // namespace isc
