@@ -1,4 +1,4 @@
-// File created from ../../../src/bin/dhcp6/dhcp6_messages.mes on Fri Aug 14 2020 21:36
+// File created from ../../../src/bin/dhcp6/dhcp6_messages.mes
 
 #ifndef DHCP6_MESSAGES_H
 #define DHCP6_MESSAGES_H
@@ -28,15 +28,16 @@ extern const isc::log::MessageID DHCP6_CONFIG_LOAD_FAIL;
 extern const isc::log::MessageID DHCP6_CONFIG_PACKET_QUEUE;
 extern const isc::log::MessageID DHCP6_CONFIG_RECEIVED;
 extern const isc::log::MessageID DHCP6_CONFIG_START;
+extern const isc::log::MessageID DHCP6_CONFIG_SYNTAX_WARNING;
 extern const isc::log::MessageID DHCP6_CONFIG_UNRECOVERABLE_ERROR;
 extern const isc::log::MessageID DHCP6_CONFIG_UNSUPPORTED_OBJECT;
 extern const isc::log::MessageID DHCP6_CONFIG_UPDATE;
 extern const isc::log::MessageID DHCP6_DB_BACKEND_STARTED;
-extern const isc::log::MessageID DHCP6_DB_RECONNECT_ATTEMPT_FAILED;
-extern const isc::log::MessageID DHCP6_DB_RECONNECT_ATTEMPT_SCHEDULE;
 extern const isc::log::MessageID DHCP6_DB_RECONNECT_DISABLED;
+extern const isc::log::MessageID DHCP6_DB_RECONNECT_FAILED;
+extern const isc::log::MessageID DHCP6_DB_RECONNECT_LOST_CONNECTION;
 extern const isc::log::MessageID DHCP6_DB_RECONNECT_NO_DB_CTL;
-extern const isc::log::MessageID DHCP6_DB_RECONNECT_RETRIES_EXHAUSTED;
+extern const isc::log::MessageID DHCP6_DB_RECONNECT_SUCCEEDED;
 extern const isc::log::MessageID DHCP6_DDNS_CREATE_ADD_NAME_CHANGE_REQUEST;
 extern const isc::log::MessageID DHCP6_DDNS_FQDN_GENERATED;
 extern const isc::log::MessageID DHCP6_DDNS_GENERATED_FQDN_UPDATE_FAIL;
@@ -61,18 +62,21 @@ extern const isc::log::MessageID DHCP6_DHCP4O6_RECEIVING;
 extern const isc::log::MessageID DHCP6_DHCP4O6_SEND_FAIL;
 extern const isc::log::MessageID DHCP6_DYNAMIC_RECONFIGURATION;
 extern const isc::log::MessageID DHCP6_DYNAMIC_RECONFIGURATION_FAIL;
+extern const isc::log::MessageID DHCP6_DYNAMIC_RECONFIGURATION_SUCCESS;
 extern const isc::log::MessageID DHCP6_FLEX_ID;
 extern const isc::log::MessageID DHCP6_HANDLE_SIGNAL_EXCEPTION;
 extern const isc::log::MessageID DHCP6_HOOKS_LIBS_RELOAD_FAIL;
 extern const isc::log::MessageID DHCP6_HOOK_BUFFER_RCVD_DROP;
 extern const isc::log::MessageID DHCP6_HOOK_BUFFER_RCVD_SKIP;
 extern const isc::log::MessageID DHCP6_HOOK_BUFFER_SEND_SKIP;
+extern const isc::log::MessageID DHCP6_HOOK_DDNS_UPDATE;
 extern const isc::log::MessageID DHCP6_HOOK_DECLINE_DROP;
 extern const isc::log::MessageID DHCP6_HOOK_DECLINE_SKIP;
 extern const isc::log::MessageID DHCP6_HOOK_LEASE6_RELEASE_NA_SKIP;
 extern const isc::log::MessageID DHCP6_HOOK_LEASE6_RELEASE_PD_SKIP;
 extern const isc::log::MessageID DHCP6_HOOK_LEASES6_COMMITTED_DROP;
 extern const isc::log::MessageID DHCP6_HOOK_LEASES6_COMMITTED_PARK;
+extern const isc::log::MessageID DHCP6_HOOK_LEASES6_PARKING_LOT_FULL;
 extern const isc::log::MessageID DHCP6_HOOK_PACKET_RCVD_SKIP;
 extern const isc::log::MessageID DHCP6_HOOK_PACKET_SEND_DROP;
 extern const isc::log::MessageID DHCP6_HOOK_PACKET_SEND_SKIP;
@@ -87,15 +91,19 @@ extern const isc::log::MessageID DHCP6_LEASE_DATA;
 extern const isc::log::MessageID DHCP6_LEASE_NA_WITHOUT_DUID;
 extern const isc::log::MessageID DHCP6_LEASE_PD_WITHOUT_DUID;
 extern const isc::log::MessageID DHCP6_LEASE_RENEW;
+extern const isc::log::MessageID DHCP6_LEASE_REUSE;
 extern const isc::log::MessageID DHCP6_MULTI_THREADING_INFO;
-extern const isc::log::MessageID DHCP6_MULTI_THREADING_WARNING;
 extern const isc::log::MessageID DHCP6_NOT_RUNNING;
 extern const isc::log::MessageID DHCP6_NO_INTERFACES;
 extern const isc::log::MessageID DHCP6_NO_SOCKETS_OPEN;
 extern const isc::log::MessageID DHCP6_OPEN_SOCKET;
+extern const isc::log::MessageID DHCP6_OPEN_SOCKETS_FAILED;
+extern const isc::log::MessageID DHCP6_OPEN_SOCKETS_NO_RECONNECT_CTL;
 extern const isc::log::MessageID DHCP6_OPEN_SOCKET_FAIL;
 extern const isc::log::MessageID DHCP6_PACKET_DROP_DHCP_DISABLED;
 extern const isc::log::MessageID DHCP6_PACKET_DROP_DROP_CLASS;
+extern const isc::log::MessageID DHCP6_PACKET_DROP_DROP_CLASS2;
+extern const isc::log::MessageID DHCP6_PACKET_DROP_DROP_CLASS_EARLY;
 extern const isc::log::MessageID DHCP6_PACKET_DROP_DUPLICATE;
 extern const isc::log::MessageID DHCP6_PACKET_DROP_PARSE_FAIL;
 extern const isc::log::MessageID DHCP6_PACKET_DROP_SERVERID_MISMATCH;
@@ -119,6 +127,7 @@ extern const isc::log::MessageID DHCP6_PD_LEASE_ADVERT_FAIL;
 extern const isc::log::MessageID DHCP6_PD_LEASE_ALLOC;
 extern const isc::log::MessageID DHCP6_PD_LEASE_ALLOC_FAIL;
 extern const isc::log::MessageID DHCP6_PD_LEASE_RENEW;
+extern const isc::log::MessageID DHCP6_PD_LEASE_REUSE;
 extern const isc::log::MessageID DHCP6_PROCESS_IA_NA_EXTEND;
 extern const isc::log::MessageID DHCP6_PROCESS_IA_NA_RELEASE;
 extern const isc::log::MessageID DHCP6_PROCESS_IA_NA_REQUEST;
@@ -136,6 +145,7 @@ extern const isc::log::MessageID DHCP6_RELEASE_PD_FAIL;
 extern const isc::log::MessageID DHCP6_RELEASE_PD_FAIL_WRONG_DUID;
 extern const isc::log::MessageID DHCP6_RELEASE_PD_FAIL_WRONG_IAID;
 extern const isc::log::MessageID DHCP6_REQUIRED_OPTIONS_CHECK_FAIL;
+extern const isc::log::MessageID DHCP6_RESERVATIONS_LOOKUP_FIRST_ENABLED;
 extern const isc::log::MessageID DHCP6_RESPONSE_DATA;
 extern const isc::log::MessageID DHCP6_SERVER_FAILED;
 extern const isc::log::MessageID DHCP6_SHUTDOWN;
