@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,14 +6,15 @@
 
 #include <config.h>
 #include <gtest/gtest.h>
+#include <cc/command_interpreter.h>
 #include <dhcp/option6_status_code.h>
+#include <dhcp/tests/pkt_captures.h>
+#include <dhcpsrv/cfg_multi_threading.h>
 #include <dhcp6/tests/dhcp6_test_utils.h>
 #include <dhcp6/json_config_parser.h>
-#include <dhcp/tests/pkt_captures.h>
 #include <log/logger_support.h>
-#include <util/pointer_util.h>
-#include <cc/command_interpreter.h>
 #include <stats/stats_mgr.h>
+#include <util/pointer_util.h>
 #include <cstdio>
 #include <sstream>
 #include <string.h>
@@ -22,6 +23,7 @@ using namespace isc::data;
 using namespace isc::dhcp;
 using namespace isc::asiolink;
 using namespace isc::stats;
+using namespace isc::util;
 
 namespace isc {
 namespace dhcp {
@@ -42,7 +44,7 @@ BaseServerTest::~BaseServerTest() {
 
     // Remove default lease file.
     std::ostringstream s2;
-    s2 << CfgMgr::instance().getDataDir() << "/" << "kea-leases6.csv";
+    s2 << CfgMgr::instance().getDataDir() << "/kea-leases6.csv";
     static_cast<void>(::remove(s2.str().c_str()));
 
     // Revert to original data directory.
@@ -53,16 +55,14 @@ BaseServerTest::~BaseServerTest() {
 }
 
 Dhcpv6SrvTest::Dhcpv6SrvTest()
-    : NakedDhcpv6SrvTest(), srv_(0) {
-    subnet_ = isc::dhcp::Subnet6Ptr
-        (new isc::dhcp::Subnet6(isc::asiolink::IOAddress("2001:db8:1::"),
-                                48, 1000, 2000, 3000, 4000));
+    : NakedDhcpv6SrvTest(), srv_(0), multi_threading_(false) {
+    subnet_ = isc::dhcp::Subnet6Ptr(new isc::dhcp::Subnet6(isc::asiolink::IOAddress("2001:db8:1::"),
+                                                           48, 1000, 2000, 3000, 4000));
     subnet_->setIface("eth0");
 
-    pool_ = isc::dhcp::Pool6Ptr
-        (new isc::dhcp::Pool6(isc::dhcp::Lease::TYPE_NA,
-                              isc::asiolink::IOAddress("2001:db8:1:1::"),
-                              64));
+    pool_ = isc::dhcp::Pool6Ptr(new isc::dhcp::Pool6(isc::dhcp::Lease::TYPE_NA,
+                                isc::asiolink::IOAddress("2001:db8:1:1::"),
+                                64));
     subnet_->addPool(pool_);
 
     isc::dhcp::CfgMgr::instance().clear();
@@ -71,13 +71,22 @@ Dhcpv6SrvTest::Dhcpv6SrvTest()
     isc::dhcp::CfgMgr::instance().commit();
 
     // configure PD pool
-    pd_pool_ = isc::dhcp::Pool6Ptr
-        (new isc::dhcp::Pool6(isc::dhcp::Lease::TYPE_PD,
-                              isc::asiolink::IOAddress("2001:db8:1:2::"),
-                              64, 80));
+    pd_pool_ = isc::dhcp::Pool6Ptr(new isc::dhcp::Pool6(isc::dhcp::Lease::TYPE_PD,
+                                   isc::asiolink::IOAddress("2001:db8:1:2::"),
+                                   64, 80));
     subnet_->addPool(pd_pool_);
+
+    // Reset the thread pool.
+    MultiThreadingMgr::instance().apply(false, 0, 0);
 }
 
+Dhcpv6SrvTest::~Dhcpv6SrvTest() {
+    isc::dhcp::CfgMgr::instance().clear();
+
+    // Reset the thread pool.
+    MultiThreadingMgr::instance().apply(false, 0, 0);
+};
+
 // Checks that server response (ADVERTISE or REPLY) contains proper IA_NA option
 // It returns IAADDR option for each chaining with checkIAAddr method.
 boost::shared_ptr<Option6IAAddr>
@@ -96,9 +105,23 @@ Dhcpv6SrvTest::checkIA_NA(const Pkt6Ptr& rsp, uint32_t expected_iaid,
         return (boost::shared_ptr<Option6IAAddr>());
     }
 
-    EXPECT_EQ(expected_iaid, ia->getIAID());
-    EXPECT_EQ(expected_t1, ia->getT1());
-    EXPECT_EQ(expected_t2, ia->getT2());
+    if (expected_iaid != ia->getIAID()) {
+        ADD_FAILURE() << "ia->iaid: " << ia->getIAID()
+                      << " is not expected value: " << expected_iaid;
+        return (boost::shared_ptr<Option6IAAddr>());
+    }
+
+    if (expected_t1 != ia->getT1()) {
+        ADD_FAILURE() << "ia->t1: " << ia->getT1()
+                      << " is not expected value: " << expected_t1;
+        return (boost::shared_ptr<Option6IAAddr>());
+    }
+
+    if (expected_t2 != ia->getT2()) {
+        ADD_FAILURE() << "ia->t2: " << ia->getT2()
+                      << " is not expected value: " << expected_t2;
+        return (boost::shared_ptr<Option6IAAddr>());
+    }
 
     tmp = ia->getOption(D6O_IAADDR);
     boost::shared_ptr<Option6IAAddr> addr = boost::dynamic_pointer_cast<Option6IAAddr>(tmp);
@@ -188,7 +211,6 @@ Dhcpv6SrvTest::checkPdLease(const DuidPtr& duid, const OptionPtr& ia_pd,
     return (lease);
 }
 
-
 Pkt6Ptr
 Dhcpv6SrvTest::createMessage(uint8_t message_type, Lease::Type lease_type,
                              const IOAddress& addr, const uint8_t prefix_len,
@@ -196,6 +218,7 @@ Dhcpv6SrvTest::createMessage(uint8_t message_type, Lease::Type lease_type,
     Pkt6Ptr msg = Pkt6Ptr(new Pkt6(message_type, 1234));
     msg->setRemoteAddr(IOAddress("fe80::abcd"));
     msg->setIface("eth0");
+    msg->setIndex(ETH0_INDEX);
     msg->addOption(createIA(lease_type, addr, prefix_len, iaid));
     return (msg);
 }
@@ -228,15 +251,43 @@ Dhcpv6SrvTest::createIA(isc::dhcp::Lease::Type lease_type,
 }
 
 void
-Dhcpv6SrvTest::testRenewBasic(Lease::Type type, const std::string& existing_addr,
+Dhcpv6SrvTest::testRenewBasic(Lease::Type type,
+                              const std::string& existing_addr,
                               const std::string& renew_addr,
-                              const uint8_t prefix_len, bool insert_before_renew) {
+                              const uint8_t prefix_len,
+                              bool insert_before_renew,
+                              bool expire_before_renew,
+                              uint32_t hint_pref,
+                              uint32_t hint_valid,
+                              uint32_t expected_pref,
+                              uint32_t expected_valid) {
     NakedDhcpv6Srv srv(0);
 
     const IOAddress existing(existing_addr);
     const IOAddress renew(renew_addr);
     const uint32_t iaid = 234;
 
+    // To reuse an expired lease we need a subnet with a pool that
+    // consists of exactly one address. This address will get expired
+    // and then be reused.
+    if (expire_before_renew) {
+        CfgMgr::instance().clear();
+        subnet_.reset(new Subnet6(IOAddress("2001:db8:1:1::"),
+                                  48, 1000, 2000, 3000, 4000));
+        subnet_->setIface("eth0");
+        pool_.reset(new Pool6(Lease::TYPE_NA, existing, existing));
+        subnet_->addPool(pool_);
+        CfgMgr::instance().setFamily(AF_INET6);
+        CfgMgr::instance().getStagingCfg()->getCfgSubnets6()->add(subnet_);
+        CfgMgr::instance().commit();
+    }
+
+    // Use intervals for lifetimes for lifetime tests.
+    if (hint_pref != 300 || hint_valid != 500) {
+        subnet_->setPreferred(Triplet<uint32_t>(2000, 3000, 4000));
+        subnet_->setValid(Triplet<uint32_t>(3000, 4000, 5000));
+    }
+
     // Generate client-id also duid_
     OptionPtr clientid = generateClientId();
 
@@ -247,7 +298,7 @@ Dhcpv6SrvTest::testRenewBasic(Lease::Type type, const std::string& existing_addr
     if (insert_before_renew) {
         // Note that preferred, valid, T1 and T2 timers and CLTT are set to invalid
         // value on purpose. They should be updated during RENEW.
-        Lease6Ptr lease(new Lease6(type, existing, duid_, iaid, 501, 502, 503, 504,
+        Lease6Ptr lease(new Lease6(type, existing, duid_, iaid, 501, 502,
                                    subnet_->getID(), HWAddrPtr(), prefix_len));
         lease->cltt_ = 1234;
         ASSERT_TRUE(LeaseMgrFactory::instance().addLease(lease));
@@ -256,22 +307,83 @@ Dhcpv6SrvTest::testRenewBasic(Lease::Type type, const std::string& existing_addr
         l = LeaseMgrFactory::instance().getLease6(type, existing);
         ASSERT_TRUE(l);
 
-        // Check that T1, T2, preferred, valid and cltt really set and not using
+        // Check that preferred, valid and cltt really set and not using
         // previous (500, 501, etc.) values
-        EXPECT_NE(l->t1_, subnet_->getT1());
-        EXPECT_NE(l->t2_, subnet_->getT2());
         EXPECT_NE(l->preferred_lft_, subnet_->getPreferred());
         EXPECT_NE(l->valid_lft_, subnet_->getValid());
         EXPECT_NE(l->cltt_, time(NULL));
     }
 
-    Pkt6Ptr req = createMessage(DHCPV6_RENEW, type, IOAddress(renew_addr),
-                                prefix_len, iaid);
+    if (expire_before_renew) {
+        // The lease must exist.
+        ASSERT_TRUE(l);
+
+        // Change the subnet identifier to make the allocation engine to
+        // not treat the lease as being renewed by the same client,
+        // but to treat it as expired lease to be reused.
+        ++l->subnet_id_;
+
+        // Move the cllt back in time and make sure that the lease got expired.
+        l->cltt_ = time(NULL) - 10;
+        l->valid_lft_ = 5;
+        ASSERT_TRUE(l->expired());
+        // Update the lease in the lease database.
+        LeaseMgrFactory::instance().updateLease6(l);
+    }
+
+    Pkt6Ptr req;
+    uint8_t message_type = DHCPV6_RENEW;
+    // Use a request vs a renew for getting an expired lease without
+    // extending it. i.e. not call extendLease6 after reuseExpiredLease.
+    if (expire_before_renew) {
+        message_type = DHCPV6_REQUEST;
+    }
+
+    if (hint_pref == 300 && hint_valid == 500) {
+        req = createMessage(message_type, type, IOAddress(renew_addr),
+                            prefix_len, iaid);
+    } else {
+        // from createMessage
+        req.reset(new Pkt6(message_type, 1234));
+        req->setRemoteAddr(IOAddress("fe80::abcd"));
+        req->setIface("eth0");
+        req->setIndex(ETH0_INDEX);
+
+        // from createIA
+        uint16_t code;
+        OptionPtr subopt;
+        switch (type) {
+        case Lease::TYPE_NA:
+            code = D6O_IA_NA;
+            subopt.reset(new Option6IAAddr(D6O_IAADDR,
+                                           IOAddress(renew_addr),
+                                           hint_pref, hint_valid));
+            break;
+        case Lease::TYPE_PD:
+            code = D6O_IA_PD;
+            subopt.reset(new Option6IAPrefix(D6O_IAPREFIX,
+                                             IOAddress(renew_addr), prefix_len,
+                                             hint_pref, hint_valid));
+            break;
+        default:
+            isc_throw(BadValue, "Invalid lease type specified "
+                      << static_cast<int>(type));
+        }
+
+        Option6IAPtr ia = generateIA(code, iaid, 1500, 3000);
+        ia->addOption(subopt);
+        req->addOption(ia);
+    };
     req->addOption(clientid);
     req->addOption(srv.getServerID());
 
     // Pass it to the server and hope for a REPLY
-    Pkt6Ptr reply = srv.processRenew(req);
+    Pkt6Ptr reply;
+    if (!expire_before_renew) {
+        reply = srv.processRenew(req);
+    } else {
+        reply = srv.processRequest(req);
+    }
 
     // Check if we get response at all
     checkResponse(reply, DHCPV6_REPLY, 1234);
@@ -289,7 +401,8 @@ Dhcpv6SrvTest::testRenewBasic(Lease::Type type, const std::string& existing_addr
         ASSERT_TRUE(addr_opt);
 
         // Check that we've got the address we requested
-        checkIAAddr(addr_opt, renew, Lease::TYPE_NA);
+        checkIAAddr(addr_opt, renew, Lease::TYPE_NA,
+                    expected_pref, expected_valid);
 
         // Check that the lease is really in the database
         l = checkLease(duid_, reply->getOption(D6O_IA_NA), addr_opt);
@@ -305,7 +418,8 @@ Dhcpv6SrvTest::testRenewBasic(Lease::Type type, const std::string& existing_addr
         ASSERT_TRUE(prefix_opt);
 
         // Check that we've got the address we requested
-        checkIAAddr(prefix_opt, renew, Lease::TYPE_PD);
+        checkIAAddr(prefix_opt, renew, Lease::TYPE_PD,
+                    expected_pref, expected_valid);
         EXPECT_EQ(pd_pool_->getLength(), prefix_opt->getLength());
 
         // Check that the lease is really in the database
@@ -317,11 +431,11 @@ Dhcpv6SrvTest::testRenewBasic(Lease::Type type, const std::string& existing_addr
         isc_throw(BadValue, "Invalid lease type");
     }
 
-    // Check that T1, T2, preferred, valid and cltt were really updated
-    EXPECT_EQ(subnet_->getT1(), l->t1_);
-    EXPECT_EQ(subnet_->getT2(), l->t2_);
-    EXPECT_EQ(subnet_->getPreferred(), l->preferred_lft_);
-    EXPECT_EQ(subnet_->getValid(), l->valid_lft_);
+    // Check that preferred, valid and cltt were really updated
+    EXPECT_EQ(expected_pref ? expected_pref : subnet_->getPreferred().get(),
+              l->preferred_lft_);
+    EXPECT_EQ(expected_valid ? expected_valid : subnet_->getValid().get(),
+              l->valid_lft_);
 
     // Checking for CLTT is a bit tricky if we want to avoid off by 1 errors
     int32_t cltt = static_cast<int32_t>(l->cltt_);
@@ -329,7 +443,9 @@ Dhcpv6SrvTest::testRenewBasic(Lease::Type type, const std::string& existing_addr
     // equality or difference by 1 between cltt and expected is ok.
     EXPECT_GE(1, abs(cltt - expected));
 
-    EXPECT_TRUE(LeaseMgrFactory::instance().deleteLease(renew_addr));
+    Lease6Ptr lease = LeaseMgrFactory::instance().getLease6(type,
+                                                            IOAddress(renew_addr));
+    EXPECT_TRUE(LeaseMgrFactory::instance().deleteLease(lease));
 }
 
 void
@@ -356,7 +472,7 @@ Dhcpv6SrvTest::testRenewWrongIAID(Lease::Type type, const IOAddress& addr) {
     // Note that preferred, valid, T1 and T2 timers and CLTT are set to invalid
     // value on purpose. They should be updated during RENEW.
     Lease6Ptr lease(new Lease6(type, addr, duid_, valid_iaid,
-                               501, 502, 503, 504, subnet_->getID(),
+                               501, 502, subnet_->getID(),
                                HWAddrPtr(), prefix_len));
     ASSERT_TRUE(LeaseMgrFactory::instance().addLease(lease));
 
@@ -405,7 +521,7 @@ Dhcpv6SrvTest::testRenewSomeoneElsesLease(Lease::Type type, const IOAddress& add
 
     // Let's create a lease.
     Lease6Ptr lease(new Lease6(type, addr, duid_, valid_iaid,
-                               501, 502, 503, 504, subnet_->getID(),
+                               501, 502, subnet_->getID(),
                                HWAddrPtr(), prefix_len));
     ASSERT_TRUE(LeaseMgrFactory::instance().addLease(lease));
 
@@ -467,7 +583,7 @@ Dhcpv6SrvTest::testReleaseBasic(Lease::Type type, const IOAddress& existing,
 
     // Let's prepopulate the database
     Lease6Ptr lease(new Lease6(type, existing, duid_, iaid,
-                               501, 502, 503, 504, subnet_->getID(),
+                               501, 502, subnet_->getID(),
                                HWAddrPtr(), prefix_len));
     ASSERT_TRUE(LeaseMgrFactory::instance().addLease(lease));
 
@@ -595,8 +711,8 @@ Dhcpv6SrvTest::testReleaseReject(Lease::Type type, const IOAddress& addr) {
     // CASE 2: Lease is known and belongs to this client, but to a different IAID
     SCOPED_TRACE("CASE 2: Lease is known and belongs to this client, but to a different IAID");
 
-    Lease6Ptr lease(new Lease6(type, addr, duid_, valid_iaid, 501, 502, 503,
-                               504, subnet_->getID(), HWAddrPtr(), prefix_len));
+    Lease6Ptr lease(new Lease6(type, addr, duid_, valid_iaid, 501, 502,
+                               subnet_->getID(), HWAddrPtr(), prefix_len));
     ASSERT_TRUE(LeaseMgrFactory::instance().addLease(lease));
 
     // Let's create a different RELEASE, with a bogus iaid
@@ -651,7 +767,8 @@ Dhcpv6SrvTest::testReleaseReject(Lease::Type type, const IOAddress& addr) {
     EXPECT_EQ(1, stat->getInteger().first);
 
     // Finally, let's cleanup the database
-    EXPECT_TRUE(LeaseMgrFactory::instance().deleteLease(addr));
+    lease = LeaseMgrFactory::instance().getLease6(type, addr);
+    EXPECT_TRUE(LeaseMgrFactory::instance().deleteLease(lease));
 }
 
 void
@@ -666,11 +783,13 @@ Dhcpv6SrvTest::testReceiveStats(uint8_t pkt_type, const std::string& stat_name)
     // And pretend it's packet of a different type
     pkt->data_[0] = pkt_type;
 
-    // Check that those statistics are not set before the test
+    // Check that the tested statistics is initially set to 0
     ObservationPtr pkt6_rcvd = mgr.getObservation("pkt6-received");
     ObservationPtr tested_stat = mgr.getObservation(stat_name);
-    EXPECT_FALSE(pkt6_rcvd);
-    EXPECT_FALSE(tested_stat);
+    ASSERT_TRUE(pkt6_rcvd);
+    ASSERT_TRUE(tested_stat);
+    EXPECT_EQ(0, pkt6_rcvd->getInteger().first);
+    EXPECT_EQ(0, tested_stat->getInteger().first);
 
     // Simulate that we have received that traffic
     srv.fakeReceive(pkt);
@@ -692,18 +811,31 @@ Dhcpv6SrvTest::testReceiveStats(uint8_t pkt_type, const std::string& stat_name)
 }
 
 void
-Dhcpv6SrvTest::configure(const std::string& config) {
-    configure(config, srv_);
+Dhcpv6SrvTest::configure(const std::string& config,
+                         const bool commit,
+                         const bool open_sockets,
+                         const bool create_managers,
+                         const bool test) {
+    configure(config, srv_, commit, open_sockets, create_managers, test);
 }
 
 void
-Dhcpv6SrvTest::configure(const std::string& config, NakedDhcpv6Srv& srv) {
+Dhcpv6SrvTest::configure(const std::string& config,
+                         NakedDhcpv6Srv& srv,
+                         const bool commit,
+                         const bool open_sockets,
+                         const bool create_managers,
+                         const bool test) {
+    setenv("KEA_LFC_EXECUTABLE", KEA_LFC_EXECUTABLE, 1);
+    MultiThreadingCriticalSection cs;
     ConstElementPtr json;
     try {
         json = parseJSON(config);
     } catch (const std::exception& ex) {
         // Fatal failure on parsing error
-        FAIL() << "config parsing failed, test is broken: " << ex.what();
+        FAIL() << "parsing failure:"
+               << "config:" << config << std::endl
+               << "error: " << ex.what();
     }
 
     ConstElementPtr status;
@@ -711,24 +843,49 @@ Dhcpv6SrvTest::configure(const std::string& config, NakedDhcpv6Srv& srv) {
     // Disable the re-detect flag
     disableIfacesReDetect(json);
 
+    // Set up multi-threading
+    configureMultiThreading(multi_threading_, json);
+
     // Configure the server and make sure the config is accepted
-    EXPECT_NO_THROW(status = configureDhcp6Server(srv, json));
+    EXPECT_NO_THROW(status = configureDhcp6Server(srv, json, test));
     ASSERT_TRUE(status);
     int rcode;
     ConstElementPtr comment = isc::config::parseAnswer(rcode, status);
     ASSERT_EQ(0, rcode) << "configuration failed, test is broken: "
         << comment->str();
 
-    CfgMgr::instance().commit();
+    // Use specified lease database backend.
+    if (create_managers) {
+        ASSERT_NO_THROW( {
+            CfgDbAccessPtr cfg_db = CfgMgr::instance().getStagingCfg()->getCfgDbAccess();
+            cfg_db->setAppendedParameters("universe=6");
+            cfg_db->createManagers();
+        } );
+    }
+
+    try {
+        CfgMultiThreading::apply(CfgMgr::instance().getStagingCfg()->getDHCPMultiThreading());
+    } catch (const std::exception& ex) {
+        ADD_FAILURE() << "Error applying multi threading settings: "
+            << ex.what();
+    }
+
+    if (commit) {
+        CfgMgr::instance().commit();
+    }
+
+    // Opening sockets.
+    if (open_sockets) {
+        IfaceMgr::instance().openSockets6();
+    }
 }
 
 NakedDhcpv6SrvTest::NakedDhcpv6SrvTest()
-: rcode_(-1) {
+    : rcode_(-1) {
     // it's ok if that fails. There should not be such a file anyway
     static_cast<void>(remove(DUID_FILE));
 
-    const isc::dhcp::IfaceMgr::IfaceCollection& ifaces =
-        isc::dhcp::IfaceMgr::instance().getIfaces();
+    const IfaceCollection& ifaces = IfaceMgr::instance().getIfaces();
 
     // There must be some interface detected
     if (ifaces.empty()) {
@@ -737,9 +894,10 @@ NakedDhcpv6SrvTest::NakedDhcpv6SrvTest()
     }
 
     valid_iface_ = (*ifaces.begin())->getName();
+    valid_ifindex_ = (*ifaces.begin())->getIndex();
 
     // Let's wipe all existing statistics.
-    isc::stats::StatsMgr::instance().removeAll();
+    StatsMgr::instance().removeAll();
 }
 
 NakedDhcpv6SrvTest::~NakedDhcpv6SrvTest() {
@@ -858,6 +1016,6 @@ NakedDhcpv6SrvTest::checkIA_NAStatusCode(
     }
 }
 
-}; // end of isc::dhcp::test namespace
-}; // end of isc::dhcp namespace
-}; // end of isc namespace
+}  // namespace test
+}  // namespace dhcp
+}  // namespace isc
