@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2017 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -316,17 +316,21 @@ TEST_F(InfRequestTest, infRequestStats) {
         getCfgSubnets6()->getAll();
     ASSERT_EQ(1, subnets->size());
 
-    // Ok, let's check the statistics. None should be present.
+    // Check that the tested statistics is initially set to 0
     using namespace isc::stats;
     StatsMgr& mgr = StatsMgr::instance();
     ObservationPtr pkt6_rcvd = mgr.getObservation("pkt6-received");
     ObservationPtr pkt6_infreq_rcvd = mgr.getObservation("pkt6-infrequest-received");
     ObservationPtr pkt6_reply_sent = mgr.getObservation("pkt6-reply-sent");
     ObservationPtr pkt6_sent = mgr.getObservation("pkt6-sent");
-    EXPECT_FALSE(pkt6_rcvd);
-    EXPECT_FALSE(pkt6_infreq_rcvd);
-    EXPECT_FALSE(pkt6_reply_sent);
-    EXPECT_FALSE(pkt6_sent);
+    ASSERT_TRUE(pkt6_rcvd);
+    ASSERT_TRUE(pkt6_infreq_rcvd);
+    ASSERT_TRUE(pkt6_reply_sent);
+    ASSERT_TRUE(pkt6_sent);
+    EXPECT_EQ(0, pkt6_rcvd->getInteger().first);
+    EXPECT_EQ(0, pkt6_infreq_rcvd->getInteger().first);
+    EXPECT_EQ(0, pkt6_reply_sent->getInteger().first);
+    EXPECT_EQ(0, pkt6_sent->getInteger().first);
 
     // Perform 2-way exchange (Inf-request/reply)
     client.requestOption(D6O_NAME_SERVERS);
