@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -472,7 +472,10 @@ TEST_F(RebindTest, directClientLostLease) {
     Lease6 lease_client = client.getLease(0);
     // The lease has been acquired. Now, let's explicitly remove it from the
     // lease database.
-    LeaseMgrFactory::instance().deleteLease(lease_client.addr_);
+    Lease6Ptr lease = LeaseMgrFactory::instance().getLease6(Lease::TYPE_NA,
+                                                            lease_client.addr_);
+    EXPECT_TRUE(LeaseMgrFactory::instance().deleteLease(lease));
+
     // Send Rebind.
     ASSERT_NO_THROW(client.doRebind());
 
@@ -604,7 +607,9 @@ TEST_F(RebindTest, relayedClientLostLease) {
     Lease6 lease_client = client.getLease(0);
     // The lease has been acquired. Now, let's explicitly remove it from the
     // lease database.
-    LeaseMgrFactory::instance().deleteLease(lease_client.addr_);
+    Lease6Ptr lease = LeaseMgrFactory::instance().getLease6(Lease::TYPE_NA,
+                                                            lease_client.addr_);
+    EXPECT_TRUE(LeaseMgrFactory::instance().deleteLease(lease));
 
     // Send Rebind.
     ASSERT_NO_THROW(client.doRebind());
@@ -793,7 +798,7 @@ TEST_F(RebindTest, directClientPDChangingPrefix) {
     // but the prefix will not match. According to the RFC 8415, section 18.3.5
     // the server may return delegated prefix with lifetime of 0 when it
     // finds that the lease entry for the particular IAID but the prefix
-    // is not appropriate. This constitues explicit notification to the
+    // is not appropriate. This constitutes explicit notification to the
     // client to not use this prefix.
     ASSERT_NO_THROW(client.doRebind());
     // Make sure that the server has discarded client's message. In such case,
@@ -940,7 +945,7 @@ TEST_F(RebindTest, requestPrefixInRebind) {
     EXPECT_EQ(STATUS_Success, client.getStatusCode(1234));
 
     // The lease should have been rebound.
-    EXPECT_EQ(1000, leases_client_na_rebound[0].cltt_ - leases_client_na[0].cltt_);
+    EXPECT_GE(leases_client_na_rebound[0].cltt_ - leases_client_na[0].cltt_, 1000);
 
     // The client should now also acquire a PD lease.
     leases_client_pd = client.getLeasesByType(Lease::TYPE_PD);
@@ -1146,5 +1151,4 @@ TEST_F(RebindTest, optionsInheritance) {
     ASSERT_TRUE(client.hasOptionWithAddress(D6O_SNTP_SERVERS, "3000:2::2"));
 }
 
-
-} // end of anonymous namespace
+}  // namespace
