@@ -7,10 +7,11 @@
 #include <config.h>
 
 #include <cc/data.h>
-#include <netconf/parser_context.h>
 #include <cc/dhcp_config_error.h>
+#include <netconf/parser_context.h>
 #include <testutils/gtest_utils.h>
 #include <testutils/io_utils.h>
+#include <testutils/log_utils.h>
 #include <testutils/user_context_utils.h>
 
 #include <gtest/gtest.h>
@@ -18,6 +19,8 @@
 #include <fstream>
 #include <set>
 
+#include <boost/algorithm/string.hpp>
+
 using namespace isc::data;
 using namespace isc::test;
 using namespace std;
@@ -51,8 +54,9 @@ void compareJSON(ConstElementPtr a, ConstElementPtr b) {
 /// @param compare whether to compare the output with legacy JSON parser
 void testParser(const std::string& txt, ParserContext::ParserType parser_type,
     bool compare = true) {
-    ConstElementPtr test_json;
+    SCOPED_TRACE("\n=== tested config ===\n" + txt + "=====================");
 
+    ConstElementPtr test_json;
     ASSERT_NO_THROW_LOG({
             try {
                 ParserContext ctx;
@@ -396,8 +400,9 @@ TEST(ParserTest, file) {
 /// @param msg expected content of the exception
 void testError(const std::string& txt,
                ParserContext::ParserType parser_type,
-               const std::string& msg)
-{
+               const std::string& msg) {
+    SCOPED_TRACE("\n=== tested config ===\n" + txt + "=====================");
+
     try {
         ParserContext ctx;
         ConstElementPtr parsed = ctx.parseString(txt, parser_type);
@@ -644,10 +649,6 @@ TEST(ParserTest, errors) {
               ParserContext::PARSER_JSON,
               "<string>:1.3: syntax error, unexpected \",\", "
               "expecting }");
-    testError("{ \"foo\":true, }\n",
-              ParserContext::PARSER_JSON,
-              "<string>:1.15: syntax error, unexpected }, "
-              "expecting constant string");
 
     // bad type
     testError("{ \"Netconf\":{\n"
@@ -703,7 +704,7 @@ TEST(ParserTest, errors) {
               "  \"Netconf\":{\n"
               "  \"comment\": \"second\" }}\n",
               ParserContext::PARSER_NETCONF,
-              "<string>:2.23: syntax error, unexpected \",\", expecting }");
+              "<string>:3.3-11: syntax error, unexpected Netconf, expecting \",\" or }");
 
     // duplicate of not string entries
     testError("{ \"Netconf\":{\n"
@@ -917,6 +918,71 @@ TEST(ParserTest, duplicateMapEntries) {
     cout << "checked " << cnt << " duplicated map entries\n";
 }
 
+/// @brief Test fixture for trailing commas.
+class TrailingCommasTest : public isc::dhcp::test::LogContentTest {
+public:
+    /// @brief Add a log entry.
+    ///
+    /// @param loc Location of the trailing comma.
+    void addLog(const string& loc) {
+        string log = "NETCONF_CONFIG_SYNTAX_WARNING Netconf ";
+        log += "configuration syntax warning: " + loc;
+        log += ": Extraneous comma. ";
+        log += "A piece of configuration may have been omitted.";
+        addString(log);
+    }
+};
+
+// Test that trailing commas are allowed.
+TEST_F(TrailingCommasTest, tests) {
+    string txt(R"({
+  "Netconf": {
+    "boot-update": true,
+    "loggers": [
+      {
+        "name": "kea-netconf",
+        "severity": "DEBUG",
+        "debuglevel": 99,
+      },
+    ],
+    "managed-servers": {
+      "dhcp4": {
+        "control-socket": {
+          "socket-name": "/tmp/kea-dhcp4-ctrl.sock",
+          "socket-type": "unix",
+        },
+        "model": "kea-dhcp4-server",
+      },
+      "dhcp6": {
+        "control-socket": {
+          "socket-name": "/tmp/kea-dhcp6-ctrl.sock",
+          "socket-type": "unix",
+        },
+        "model": "kea-dhcp6-server",
+      },
+    },
+    "subscribe-changes": true,
+    "validate-changes": true,
+  },
+})");
+    testParser(txt, ParserContext::PARSER_NETCONF, false);
+
+    addLog("<string>:8.25");
+    addLog("<string>:9.8");
+    addLog("<string>:15.32");
+    addLog("<string>:17.36");
+    addLog("<string>:22.32");
+    addLog("<string>:24.36");
+    addLog("<string>:25.8");
+    addLog("<string>:28.29");
+    addLog("<string>:29.4");
+    EXPECT_TRUE(checkFile());
+
+    // Test with many consecutive commas.
+    boost::replace_all(txt, ",", ",,,,");
+    testParser(txt, ParserContext::PARSER_NETCONF, false);
 }
-}
-}
+
+}  // namespace test
+}  // namespace netconf
+}  // namespace isc
