@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -18,8 +18,12 @@
 #include <dhcp/pkt6.h>
 #include <http/date_time.h>
 #include <util/boost_time_utils.h>
-#include <boost/bind.hpp>
+#include <util/multi_threading_mgr.h>
+
 #include <boost/pointer_cast.hpp>
+
+#include <functional>
+#include <limits>
 #include <sstream>
 #include <utility>
 
@@ -28,7 +32,10 @@ using namespace isc::data;
 using namespace isc::dhcp;
 using namespace isc::http;
 using namespace isc::log;
+using namespace isc::util;
+
 using namespace boost::posix_time;
+using namespace std;
 
 namespace {
 
@@ -51,8 +58,10 @@ CommunicationState::CommunicationState(const IOServicePtr& io_service,
     : io_service_(io_service), config_(config), timer_(), interval_(0),
       poke_time_(boost::posix_time::microsec_clock::universal_time()),
       heartbeat_impl_(0), partner_state_(-1), partner_scopes_(),
-      clock_skew_(0, 0, 0, 0), last_clock_skew_warn_(), my_time_at_skew_(),
-      partner_time_at_skew_(), analyzed_messages_count_(0) {
+      clock_skew_(0, 0, 0, 0), last_clock_skew_warn_(),
+      my_time_at_skew_(), partner_time_at_skew_(),
+      analyzed_messages_count_(0), unsent_update_count_(0),
+      partner_unsent_update_count_{0, 0}, mutex_(new mutex()) {
 }
 
 CommunicationState::~CommunicationState() {
@@ -60,31 +69,67 @@ CommunicationState::~CommunicationState() {
 }
 
 void
-CommunicationState::setPartnerState(const std::string& state) {
-    if (state == "hot-standby") {
-        partner_state_ = HA_HOT_STANDBY_ST;
-    } else if (state == "load-balancing") {
-        partner_state_ = HA_LOAD_BALANCING_ST;
-    } else if (state == "partner-down") {
-        partner_state_ = HA_PARTNER_DOWN_ST;
-    } else if (state == "ready") {
-        partner_state_ = HA_READY_ST;
-    } else if (state == "syncing") {
-        partner_state_ = HA_SYNCING_ST;
-    } else if (state == "terminated") {
-        partner_state_ = HA_TERMINATED_ST;
-    } else if (state == "waiting") {
-        partner_state_ = HA_WAITING_ST;
-    } else if (state == "unavailable") {
-        partner_state_ = HA_UNAVAILABLE_ST;
+CommunicationState::modifyPokeTime(const long secs) {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        poke_time_ += boost::posix_time::seconds(secs);
     } else {
+        poke_time_ += boost::posix_time::seconds(secs);
+    }
+}
+
+int
+CommunicationState::getPartnerState() const {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        return (partner_state_);
+    } else {
+        return (partner_state_);
+    }
+}
+
+void
+CommunicationState::setPartnerState(const std::string& state) {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        setPartnerStateInternal(state);
+    } else {
+        setPartnerStateInternal(state);
+    }
+}
+
+void
+CommunicationState::setPartnerStateInternal(const std::string& state) {
+    try {
+        partner_state_ = stringToState(state);
+    } catch (...) {
         isc_throw(BadValue, "unsupported HA partner state returned "
                   << state);
     }
 }
 
+std::set<std::string>
+CommunicationState::getPartnerScopes() const {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        return (partner_scopes_);
+    } else {
+        return (partner_scopes_);
+    }
+}
+
 void
 CommunicationState::setPartnerScopes(ConstElementPtr new_scopes) {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        setPartnerScopesInternal(new_scopes);
+    } else {
+        setPartnerScopesInternal(new_scopes);
+    }
+}
+
+void
+CommunicationState::setPartnerScopesInternal(ConstElementPtr new_scopes) {
     if (!new_scopes || (new_scopes->getType() != Element::list)) {
         isc_throw(BadValue, "unable to record partner's HA scopes because"
                   " the received value is not a valid JSON list");
@@ -107,13 +152,18 @@ CommunicationState::setPartnerScopes(ConstElementPtr new_scopes) {
 
 void
 CommunicationState::startHeartbeat(const long interval,
-                                   const boost::function<void()>& heartbeat_impl) {
-    startHeartbeatInternal(interval, heartbeat_impl);
+                                   const std::function<void()>& heartbeat_impl) {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        startHeartbeatInternal(interval, heartbeat_impl);
+    } else {
+        startHeartbeatInternal(interval, heartbeat_impl);
+    }
 }
 
 void
 CommunicationState::startHeartbeatInternal(const long interval,
-                                           const boost::function<void()>& heartbeat_impl) {
+                                           const std::function<void()>& heartbeat_impl) {
     bool settings_modified = false;
 
     // If we're setting the heartbeat for the first time, it should
@@ -154,6 +204,16 @@ CommunicationState::startHeartbeatInternal(const long interval,
 
 void
 CommunicationState::stopHeartbeat() {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        stopHeartbeatInternal();
+    } else {
+        stopHeartbeatInternal();
+    }
+}
+
+void
+CommunicationState::stopHeartbeatInternal() {
     if (timer_) {
         timer_->cancel();
         timer_.reset();
@@ -162,12 +222,49 @@ CommunicationState::stopHeartbeat() {
     }
 }
 
-void
-CommunicationState::poke() {
+bool
+CommunicationState::isHeartbeatRunning() const {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        return (static_cast<bool>(timer_));
+    } else {
+        return (static_cast<bool>(timer_));
+    }
+}
+
+boost::posix_time::time_duration
+CommunicationState::updatePokeTime() {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        return (updatePokeTimeInternal());
+    } else {
+        return (updatePokeTimeInternal());
+    }
+}
+
+boost::posix_time::time_duration
+CommunicationState::updatePokeTimeInternal() {
     // Remember previous poke time.
     boost::posix_time::ptime prev_poke_time = poke_time_;
     // Set poke time to the current time.
     poke_time_ = boost::posix_time::microsec_clock::universal_time();
+    return (poke_time_ - prev_poke_time);
+}
+
+void
+CommunicationState::poke() {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        pokeInternal();
+    } else {
+        pokeInternal();
+    }
+}
+
+void
+CommunicationState::pokeInternal() {
+    // Update poke time and compute duration.
+    boost::posix_time::time_duration duration_since_poke = updatePokeTimeInternal();
 
     // If we have been tracking the DHCP messages directed to the partner,
     // we need to clear any gathered information because the connection
@@ -177,16 +274,13 @@ CommunicationState::poke() {
 
     if (timer_) {
         // Check the duration since last poke. If it is less than a second, we don't
-        // want to reschedule the timer. The only case when the poke time duration is
-        // lower than 1s is when we're performing lease updates. In order to avoid the
-        // overhead of re-scheduling the timer too frequently we reschedule it only if the
+        // want to reschedule the timer. In order to avoid the overhead of
+        // re-scheduling the timer too frequently we reschedule it only if the
         // duration is 1s or more. This matches the time resolution for heartbeats.
-        boost::posix_time::time_duration duration_since_poke = poke_time_ - prev_poke_time;
         if (duration_since_poke.total_seconds() > 0) {
             // A poke causes the timer to be re-scheduled to prevent it
             // from triggering a heartbeat shortly after confirming the
-            // connection is ok, based on the lease update or another
-            // command.
+            // connection is ok.
             startHeartbeatInternal();
         }
     }
@@ -194,6 +288,16 @@ CommunicationState::poke() {
 
 int64_t
 CommunicationState::getDurationInMillisecs() const {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        return (getDurationInMillisecsInternal());
+    } else {
+        return (getDurationInMillisecsInternal());
+    }
+}
+
+int64_t
+CommunicationState::getDurationInMillisecsInternal() const {
     ptime now = boost::posix_time::microsec_clock::universal_time();
     time_duration duration = now - poke_time_;
     return (duration.total_milliseconds());
@@ -211,6 +315,16 @@ CommunicationState::getAnalyzedMessagesCount() const {
 
 bool
 CommunicationState::clockSkewShouldWarn() {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        return (clockSkewShouldWarnInternal());
+    } else {
+        return (clockSkewShouldWarnInternal());
+    }
+}
+
+bool
+CommunicationState::clockSkewShouldWarnInternal() {
     // First check if the clock skew is beyond the threshold.
     if (isClockSkewGreater(WARN_CLOCK_SKEW)) {
 
@@ -228,6 +342,8 @@ CommunicationState::clockSkewShouldWarn() {
         if (last_clock_skew_warn_.is_not_a_date_time() ||
             (since_warn_duration.total_seconds() > MIN_TIME_SINCE_CLOCK_SKEW_WARN)) {
             last_clock_skew_warn_ = now;
+            LOG_WARN(ha_logger, HA_HIGH_CLOCK_SKEW)
+                     .arg(logFormatClockSkewInternal());
             return (true);
         }
     }
@@ -238,8 +354,24 @@ CommunicationState::clockSkewShouldWarn() {
 
 bool
 CommunicationState::clockSkewShouldTerminate() const {
-    // Issue a warning if the clock skew is greater than 60s.
-    return (isClockSkewGreater(TERM_CLOCK_SKEW));
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        // Issue a warning if the clock skew is greater than 60s.
+        return (clockSkewShouldTerminateInternal());
+    } else {
+        return (clockSkewShouldTerminateInternal());
+    }
+}
+
+bool
+CommunicationState::clockSkewShouldTerminateInternal() const {
+    if (isClockSkewGreater(TERM_CLOCK_SKEW)) {
+        LOG_ERROR(ha_logger, HA_HIGH_CLOCK_SKEW_CAUSES_TERMINATION)
+                  .arg(logFormatClockSkewInternal());
+        return (true);
+    }
+
+    return (false);
 }
 
 bool
@@ -250,6 +382,16 @@ CommunicationState::isClockSkewGreater(const long seconds) const {
 
 void
 CommunicationState::setPartnerTime(const std::string& time_text) {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        setPartnerTimeInternal(time_text);
+    } else {
+        setPartnerTimeInternal(time_text);
+    }
+}
+
+void
+CommunicationState::setPartnerTimeInternal(const std::string& time_text) {
     partner_time_at_skew_ = HttpDateTime().fromRfc1123(time_text).getPtime();
     my_time_at_skew_ = HttpDateTime().getPtime();
     clock_skew_ = partner_time_at_skew_ - my_time_at_skew_;
@@ -257,6 +399,16 @@ CommunicationState::setPartnerTime(const std::string& time_text) {
 
 std::string
 CommunicationState::logFormatClockSkew() const {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        return (logFormatClockSkewInternal());
+    } else {
+        return (logFormatClockSkewInternal());
+    }
+}
+
+std::string
+CommunicationState::logFormatClockSkewInternal() const {
     std::ostringstream os;
 
     if ((my_time_at_skew_.is_not_a_date_time()) ||
@@ -311,9 +463,9 @@ CommunicationState::getReport() const {
     report->set("unacked-clients", Element::create(static_cast<long long>(getUnackedClientsCount())));
 
     long long unacked_clients_left = 0;
-    if (isCommunicationInterrupted() && (config_->getMaxUnackedClients() > getUnackedClientsCount())) {
+    if (isCommunicationInterrupted() && (config_->getMaxUnackedClients() >= getUnackedClientsCount())) {
         unacked_clients_left = static_cast<long long>(config_->getMaxUnackedClients() -
-                                                      getUnackedClientsCount());
+                                                      getUnackedClientsCount() + 1);
     }
     report->set("unacked-clients-left", Element::create(unacked_clients_left));
     report->set("analyzed-packets", Element::create(static_cast<long long>(getAnalyzedMessagesCount())));
@@ -321,6 +473,69 @@ CommunicationState::getReport() const {
     return (report);
 }
 
+uint64_t
+CommunicationState::getUnsentUpdateCount() const {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        return (unsent_update_count_);
+    } else {
+        return (unsent_update_count_);
+    }
+}
+
+void
+CommunicationState::increaseUnsentUpdateCount() {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        increaseUnsentUpdateCountInternal();
+    } else {
+        increaseUnsentUpdateCountInternal();
+    }
+}
+
+void
+CommunicationState::increaseUnsentUpdateCountInternal() {
+    // Protect against setting the incremented value to zero.
+    // The zero value is reserved for a server startup.
+    if (unsent_update_count_ < std::numeric_limits<uint64_t>::max()) {
+        ++unsent_update_count_;
+    } else {
+        unsent_update_count_ = 1;
+    }
+}
+
+bool
+CommunicationState::hasPartnerNewUnsentUpdates() const {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        return (hasPartnerNewUnsentUpdatesInternal());
+    } else {
+        return (hasPartnerNewUnsentUpdatesInternal());
+    }
+}
+
+bool
+CommunicationState::hasPartnerNewUnsentUpdatesInternal() const {
+    return (partner_unsent_update_count_.second > 0 &&
+            (partner_unsent_update_count_.first != partner_unsent_update_count_.second));
+}
+
+void
+CommunicationState::setPartnerUnsentUpdateCount(uint64_t unsent_update_count) {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        setPartnerUnsentUpdateCountInternal(unsent_update_count);
+    } else {
+        setPartnerUnsentUpdateCountInternal(unsent_update_count);
+    }
+}
+
+void
+CommunicationState::setPartnerUnsentUpdateCountInternal(uint64_t unsent_update_count) {
+    partner_unsent_update_count_.first = partner_unsent_update_count_.second;
+    partner_unsent_update_count_.second = unsent_update_count;
+}
+
 CommunicationState4::CommunicationState4(const IOServicePtr& io_service,
                                          const HAConfigPtr& config)
     : CommunicationState(io_service, config), connecting_clients_() {
@@ -328,6 +543,16 @@ CommunicationState4::CommunicationState4(const IOServicePtr& io_service,
 
 void
 CommunicationState4::analyzeMessage(const boost::shared_ptr<dhcp::Pkt>& message) {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        analyzeMessageInternal(message);
+    } else {
+        analyzeMessageInternal(message);
+    }
+}
+
+void
+CommunicationState4::analyzeMessageInternal(const boost::shared_ptr<dhcp::Pkt>& message) {
     // The DHCP message must successfully cast to a Pkt4 object.
     Pkt4Ptr msg = boost::dynamic_pointer_cast<Pkt4>(message);
     if (!msg) {
@@ -396,30 +621,52 @@ CommunicationState4::analyzeMessage(const boost::shared_ptr<dhcp::Pkt>& message)
     // Only log the first time we detect a client is unacked.
     if (log_unacked) {
         unsigned unacked_left = 0;
-        if (config_->getMaxUnackedClients() > getUnackedClientsCount()) {
-            unacked_left = config_->getMaxUnackedClients() > getUnackedClientsCount();
+        unsigned unacked_total = connecting_clients_.get<1>().count(true);
+        if (config_->getMaxUnackedClients() >= unacked_total) {
+            unacked_left = config_->getMaxUnackedClients() - unacked_total + 1;
         }
         LOG_INFO(ha_logger, HA_COMMUNICATION_INTERRUPTED_CLIENT4_UNACKED)
             .arg(message->getLabel())
-            .arg(getUnackedClientsCount())
+            .arg(unacked_total)
             .arg(unacked_left);
     }
 }
 
 bool
 CommunicationState4::failureDetected() const {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        return (failureDetectedInternal());
+    } else {
+        return (failureDetectedInternal());
+    }
+}
+
+bool
+CommunicationState4::failureDetectedInternal() const {
     return ((config_->getMaxUnackedClients() == 0) ||
-            (getUnackedClientsCount() > config_->getMaxUnackedClients()));
+            (connecting_clients_.get<1>().count(true) >
+             config_->getMaxUnackedClients()));
 }
 
 size_t
 CommunicationState4::getConnectingClientsCount() const {
-    return (connecting_clients_.size());
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        return (connecting_clients_.size());
+    } else {
+        return (connecting_clients_.size());
+    }
 }
 
 size_t
 CommunicationState4::getUnackedClientsCount() const {
-    return (connecting_clients_.get<1>().count(true));
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        return (connecting_clients_.get<1>().count(true));
+    } else {
+        return (connecting_clients_.get<1>().count(true));
+    }
 }
 
 void
@@ -434,6 +681,16 @@ CommunicationState6::CommunicationState6(const IOServicePtr& io_service,
 
 void
 CommunicationState6::analyzeMessage(const boost::shared_ptr<dhcp::Pkt>& message) {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        analyzeMessageInternal(message);
+    } else {
+        analyzeMessageInternal(message);
+    }
+}
+
+void
+CommunicationState6::analyzeMessageInternal(const boost::shared_ptr<dhcp::Pkt>& message) {
     // The DHCP message must successfully cast to a Pkt6 object.
     Pkt6Ptr msg = boost::dynamic_pointer_cast<Pkt6>(message);
     if (!msg) {
@@ -490,30 +747,52 @@ CommunicationState6::analyzeMessage(const boost::shared_ptr<dhcp::Pkt>& message)
     // Only log the first time we detect a client is unacked.
     if (log_unacked) {
         unsigned unacked_left = 0;
-        if (config_->getMaxUnackedClients() > getUnackedClientsCount()) {
-            unacked_left = config_->getMaxUnackedClients() > getUnackedClientsCount();
+        unsigned unacked_total = connecting_clients_.get<1>().count(true);
+        if (config_->getMaxUnackedClients() >= unacked_total) {
+            unacked_left = config_->getMaxUnackedClients() - unacked_total + 1;
         }
         LOG_INFO(ha_logger, HA_COMMUNICATION_INTERRUPTED_CLIENT6_UNACKED)
             .arg(message->getLabel())
-            .arg(getUnackedClientsCount())
+            .arg(unacked_total)
             .arg(unacked_left);
     }
 }
 
 bool
 CommunicationState6::failureDetected() const {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        return (failureDetectedInternal());
+    } else {
+        return (failureDetectedInternal());
+    }
+}
+
+bool
+CommunicationState6::failureDetectedInternal() const {
     return ((config_->getMaxUnackedClients() == 0) ||
-            (getUnackedClientsCount() > config_->getMaxUnackedClients()));
+            (connecting_clients_.get<1>().count(true) >
+             config_->getMaxUnackedClients()));
 }
 
 size_t
 CommunicationState6::getConnectingClientsCount() const {
-    return (connecting_clients_.size());
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        return (connecting_clients_.size());
+    } else {
+        return (connecting_clients_.size());
+    }
 }
 
 size_t
 CommunicationState6::getUnackedClientsCount() const {
-    return (connecting_clients_.get<1>().count(true));
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lk(*mutex_);
+        return (connecting_clients_.get<1>().count(true));
+    } else {
+        return (connecting_clients_.get<1>().count(true));
+    }
 }
 
 void
