@@ -51,37 +51,14 @@ public:
             ADD_FAILURE() << "expected ConfigError exception, thrown no exception";
 
         } catch (const ConfigError& ex) {
-            EXPECT_EQ(expected_error, std::string(ex.what()));
-
+            // Expect the error to be contained in the exception message.
+            std::string const exception(ex.what());
+            EXPECT_NE(exception.find(expected_error), std::string::npos);
         } catch (...) {
             ADD_FAILURE() << "expected ConfigError exception, thrown different"
                 " exception type";
         }
     }
-
-    /// @brief Replace a pattern in a configuration.
-    ///
-    /// @param config Configuration to patch.
-    /// @param from String to replace.
-    /// @param repl String which replaces all occurrences of from.
-    /// @result A copy of config where all occurrences of from were replaced
-    /// by repl.
-    std::string replaceInConfig(const std::string& config,
-                                const std::string& from,
-                                const std::string& repl) {
-        std::string result(config);
-        if (from.empty()) {
-            return (result);
-        }
-        for (;;) {
-            size_t where = result.find(from);
-            if (where == std::string::npos) {
-                return (result);
-            }
-            result.replace(where, from.size(), repl);
-        }
-        return (result);
-    }
 };
 
 // Verifies that load balancing configuration is parsed correctly.
@@ -373,8 +350,9 @@ TEST_F(HAConfigTest, configurePassiveBackup) {
         "            {"
         "                \"name\": \"server3\","
         "                \"url\": \"http://127.0.0.1:8082/\","
-        "                \"basic-auth-user\": \"test\","
-        "                \"basic-auth-password\": \"123\\u00a3\","
+        "                \"basic-auth-user\": \"keatest\","
+        "                \"basic-auth-password-file\": \""
+                             TEST_HTTP_DIR "/hiddenp\","
         "                \"role\": \"backup\""
         "            }"
         "        ]"
@@ -408,7 +386,7 @@ TEST_F(HAConfigTest, configurePassiveBackup) {
     EXPECT_EQ("http://127.0.0.1:8082/", cfg->getUrl().toText());
     EXPECT_EQ(HAConfig::PeerConfig::BACKUP, cfg->getRole());
     ASSERT_TRUE(cfg->getBasicAuth());
-    EXPECT_EQ("dGVzdDoxMjPCow==", cfg->getBasicAuth()->getCredential());
+    EXPECT_EQ("a2VhdGVzdDpLZWFUZXN0", cfg->getBasicAuth()->getCredential());
 
     // Verify multi-threading default values.
     EXPECT_FALSE(impl->getConfig()->getEnableMultiThreading());
@@ -1354,6 +1332,8 @@ TEST_F(HAConfigTest, tlsParameterInheritance) {
         "        \"trust-anchor\": \"!CA!/kea-ca.crt\","
         "        \"cert-file\": \"!CA!/kea-client.crt\","
         "        \"key-file\": \"!CA!/kea-client.key\","
+        "        \"require-client-certs\": false,"
+        "        \"restrict-commands\": true,"
         "        \"peers\": ["
         "            {"
         "                \"name\": \"my-server\","
@@ -1401,6 +1381,8 @@ TEST_F(HAConfigTest, tlsParameterInheritance) {
     expected = TEST_CA_DIR;
     expected += "/kea-client.key";
     EXPECT_EQ(expected, impl->getConfig()->getKeyFile().get());
+    EXPECT_FALSE(impl->getConfig()->getRequireClientCerts());
+    EXPECT_TRUE(impl->getConfig()->getRestrictCommands());
 
     // Check the first peer parameters: it inherits them from the global level.
     HAConfig::PeerConfigPtr cfg = impl->getConfig()->getThisServerConfig();
@@ -1446,6 +1428,7 @@ TEST_F(HAConfigTest, missingTrustAnchor) {
         "        \"trust-anchor\": \"!CA!/kea-ca.crt\","
         "        \"cert-file\": \"!CA!/kea-client.crt\","
         "        \"key-file\": \"!CA!/kea-client.key\","
+        "        \"require-client-certs\": false,"
         "        \"peers\": ["
         "            {"
         "                \"name\": \"server1\","
@@ -1481,6 +1464,7 @@ TEST_F(HAConfigTest, missingCertFile) {
         "        \"trust-anchor\": \"!CA!/kea-ca.crt\","
         "        \"cert-file\": \"!CA!/kea-client.crt\","
         "        \"key-file\": \"!CA!/kea-client.key\","
+        "        \"require-client-certs\": false,"
         "        \"peers\": ["
         "            {"
         "                \"name\": \"server1\","
@@ -1516,6 +1500,7 @@ TEST_F(HAConfigTest, missingKeyFile) {
         "        \"trust-anchor\": \"!CA!/kea-ca.crt\","
         "        \"cert-file\": \"!CA!/kea-client.crt\","
         "        \"key-file\": \"!CA!/kea-client.key\","
+        "        \"require-client-certs\": false,"
         "        \"peers\": ["
         "            {"
         "                \"name\": \"server1\","
@@ -1551,6 +1536,7 @@ TEST_F(HAConfigTest, badTrustAnchor) {
         "        \"trust-anchor\": \"/this-file-does-not-exist\","
         "        \"cert-file\": \"!CA!/kea-client.crt\","
         "        \"key-file\": \"!CA!/kea-client.key\","
+        "        \"require-client-certs\": false,"
         "        \"peers\": ["
         "            {"
         "                \"name\": \"server1\","
@@ -1590,6 +1576,7 @@ TEST_F(HAConfigTest, badCertFile) {
         "        \"trust-anchor\": \"!CA!/kea-ca.crt\","
         "        \"cert-file\": \"/this-file-does-not-exist\","
         "        \"key-file\": \"!CA!/kea-client.key\","
+        "        \"require-client-certs\": false,"
         "        \"peers\": ["
         "            {"
         "                \"name\": \"server1\","
@@ -1629,6 +1616,7 @@ TEST_F(HAConfigTest, badKeyFile) {
         "        \"trust-anchor\": \"!CA!/kea-ca.crt\","
         "        \"cert-file\": \"!CA!/kea-client.crt\","
         "        \"key-file\": \"/this-file-does-not-exist\","
+        "        \"require-client-certs\": false,"
         "        \"peers\": ["
         "            {"
         "                \"name\": \"server1\","
@@ -1717,7 +1705,6 @@ TEST_F(HAConfigTest, pausingToString) {
               HAConfig::StateConfig::pausingToString(STATE_PAUSE_NEVER));
     EXPECT_EQ("once",
               HAConfig::StateConfig::pausingToString(STATE_PAUSE_ONCE));
-
 }
 
 // Verifies permutations of HA+MT configuration.
@@ -1883,6 +1870,6 @@ TEST_F(HAConfigTest, ipv6Url) {
 
     // Check the URL.
     EXPECT_EQ(impl->getConfig()->getThisServerConfig()->getUrl().toText(), "http://[2001:db8::1]:8080/");
-};
+}
 
 } // end of anonymous namespace
