@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -20,6 +20,7 @@
 #include <dhcpsrv/network.h>
 #include <dhcpsrv/subnet_id.h>
 #include <exceptions/exceptions.h>
+#include <util/triplet.h>
 #include <mysql/mysql_binding.h>
 #include <mysql/mysql_connection.h>
 #include <set>
@@ -135,7 +136,7 @@ public:
     /// @return Pointer to a null binding if the triplet is "unspecified" or
     /// a pointer to a binding representing 32-bit unsigned integer value
     /// otherwise.
-    static db::MySqlBindingPtr createBinding(const Triplet<uint32_t>& triplet);
+    static db::MySqlBindingPtr createBinding(const util::Triplet<uint32_t>& triplet);
 
     /// @brief Creates MySQL binding from a @c Triplet max value.
     ///
@@ -144,7 +145,7 @@ public:
     /// the max value is the same as the default value, or a pointer to
     /// a binding representing 32-bit unsigned integer value from the max
     /// value otherwise.
-    static db::MySqlBindingPtr createMaxBinding(const Triplet<uint32_t>& triplet);
+    static db::MySqlBindingPtr createMaxBinding(const util::Triplet<uint32_t>& triplet);
 
     /// @brief Creates MySQL binding from a @c Triplet min value.
     ///
@@ -153,7 +154,7 @@ public:
     /// the min value is the same as the default value, or a pointer to
     /// a binding representing 32-bit unsigned integer value from the min
     /// value otherwise.
-    static db::MySqlBindingPtr createMinBinding(const Triplet<uint32_t>& triplet);
+    static db::MySqlBindingPtr createMinBinding(const util::Triplet<uint32_t>& triplet);
 
     /// @brief Creates @c Triplet object from MySQL binding.
     ///
@@ -164,7 +165,7 @@ public:
     /// @throw isc::Unexpected if the provided binding pointer is NULL.
     /// @throw isc::InvalidOperation if the binding does not represent
     /// a 32-bit unsigned integer.
-    static Triplet<uint32_t> createTriplet(const db::MySqlBindingPtr& binding);
+    static util::Triplet<uint32_t> createTriplet(const db::MySqlBindingPtr& binding);
 
     /// @brief Creates @c Triplet object from MySQL bindings.
     ///
@@ -178,9 +179,9 @@ public:
     /// @throw isc::Unexpected if the provided binding pointer is NULL.
     /// @throw isc::InvalidOperation if the binding does not represent
     /// a 32-bit unsigned integer.
-    static Triplet<uint32_t> createTriplet(const db::MySqlBindingPtr& def_binding,
-                                           const db::MySqlBindingPtr& min_binding,
-                                           const db::MySqlBindingPtr& max_binding);
+    static util::Triplet<uint32_t> createTriplet(const db::MySqlBindingPtr& def_binding,
+                                                 const db::MySqlBindingPtr& min_binding,
+                                                 const db::MySqlBindingPtr& max_binding);
 
     /// @brief Returns server tag associated with the particular selector.
     ///
@@ -195,7 +196,7 @@ public:
     /// is more than one server tag associated with the selector.
     std::string getServerTag(const db::ServerSelector& server_selector,
                              const std::string& operation) const {
-        auto tags = server_selector.getTags();
+        auto const& tags = server_selector.getTags();
         if (tags.size() != 1) {
             isc_throw(InvalidOperation, "expected exactly one server tag to be specified"
                       " while " << operation << ". Got: "
@@ -211,8 +212,8 @@ public:
     /// This method is useful for logging purposes.
     std::string getServerTagsAsText(const db::ServerSelector& server_selector) const {
         std::ostringstream s;
-        auto server_tags = server_selector.getTags();
-        for (auto tag : server_tags) {
+        auto const& server_tags = server_selector.getTags();
+        for (auto const& tag : server_tags) {
             if (s.tellp() != 0) {
                 s << ", ";
             }
@@ -446,10 +447,11 @@ public:
     ///
     /// @return Pointer to the returned option or NULL if such option
     /// doesn't exist.
-    OptionDescriptorPtr
-    getOption(const int index, const Option::Universe& universe,
-              const db::ServerSelector& server_selector, const uint16_t code,
-              const std::string& space);
+    OptionDescriptorPtr getOption(const int index,
+                                  const Option::Universe& universe,
+                                  const db::ServerSelector& server_selector,
+                                  const uint16_t code,
+                                  const std::string& space);
 
     /// @brief Sends query to retrieve all global options.
     ///
@@ -457,9 +459,9 @@ public:
     /// @param universe Option universe, i.e. V4 or V6.
     /// @param server_selector Server selector.
     /// @return Container holding returned options.
-    OptionContainer
-    getAllOptions(const int index, const Option::Universe& universe,
-                  const db::ServerSelector& server_selector);
+    OptionContainer getAllOptions(const int index,
+                                  const Option::Universe& universe,
+                                  const db::ServerSelector& server_selector);
 
     /// @brief Sends query to retrieve global options with modification
     /// time later than specified timestamp.
@@ -468,10 +470,10 @@ public:
     /// @param universe Option universe, i.e. V4 or V6.
     /// @param server_selector Server selector.
     /// @return Container holding returned options.
-    OptionContainer
-    getModifiedOptions(const int index, const Option::Universe& universe,
-                       const db::ServerSelector& server_selector,
-                       const boost::posix_time::ptime& modification_time);
+    OptionContainer getModifiedOptions(const int index,
+                                       const Option::Universe& universe,
+                                       const db::ServerSelector& server_selector,
+                                       const boost::posix_time::ptime& modification_time);
 
     /// @brief Sends query to retrieve single option by code and option space
     /// for a given subnet id.
@@ -573,9 +575,8 @@ public:
     /// @param universe V4 or V6.
     /// @param first_binding Iterator of the output binding containing
     /// option_id.
-    OptionDescriptorPtr
-    processOptionRow(const Option::Universe& universe,
-                     db::MySqlBindingCollection::iterator first_binding);
+    OptionDescriptorPtr processOptionRow(const Option::Universe& universe,
+                                         db::MySqlBindingCollection::iterator first_binding);
 
     /// @brief Returns DHCP option definition instance from output bindings.
     ///
@@ -595,8 +596,7 @@ public:
     /// @param first_binding Iterator of the output binding containing
     /// option definition id.
     /// @return Pointer to the option definition.
-    OptionDefinitionPtr
-    processOptionDefRow(db::MySqlBindingCollection::iterator first_binding);
+    OptionDefinitionPtr processOptionDefRow(db::MySqlBindingCollection::iterator first_binding);
 
     /// @brief Associates a configuration element with multiple servers.
     ///
@@ -609,7 +609,7 @@ public:
     void attachElementToServers(const int index,
                                 const db::ServerSelector& server_selector,
                                 const db::MySqlBindingPtr& first_binding,
-                                const db::MySqlBindingPtr& in_bindings...);
+                                const db::MySqlBindingPtr& in_bindings);
 
     /// @brief Creates input binding for relay addresses.
     ///
@@ -644,7 +644,7 @@ public:
 
     /// @brief Creates input binding for user context parameter.
     ///
-    /// @param T Type of the configuration element to which context belongs.
+    /// @tparam T Type of the configuration element to which context belongs.
     /// @param network Pointer to a shared network, subnet or other configuration
     /// element for which binding should be created.
     /// @return Pointer to the binding (possibly null binding if context is
@@ -758,7 +758,7 @@ public:
         }
 
         // Go over the collection of elements.
-        for (auto elem = index.begin(); elem != index.end(); ) {
+        for (auto elem = index.begin(); elem != index.end();) {
 
             // If we're asking for shared networks matching all servers,
             // we have to make sure that the fetched element has "all"
@@ -782,9 +782,9 @@ public:
                 // Server selector contains explicit server tags, so
                 // let's see if the returned elements includes any of
                 // them.
-                auto tags = server_selector.getTags();
+                auto const& tags = server_selector.getTags();
                 bool tag_found = false;
-                for (auto tag : tags) {
+                for (auto const& tag : tags) {
                     if ((*elem)->hasServerTag(tag) ||
                         (*elem)->hasAllServerTag()) {
                         tag_found = true;
@@ -835,7 +835,7 @@ public:
         return (parameters_);
     }
 
-    /// @brief Sets IO service to be used by the MySql config backend.
+    /// @brief Sets IO service to be used by the MySQL config backend.
     ///
     /// @param IOService object, used for all ASIO operations.
     static void setIOService(const isc::asiolink::IOServicePtr& io_service) {
@@ -863,7 +863,7 @@ private:
     /// @brief Connection parameters
     isc::db::DatabaseConnection::ParameterMap parameters_;
 
-    /// The IOService object, used for all ASIO operations.
+    /// @brief The IOService object, used for all ASIO operations.
     static isc::asiolink::IOServicePtr io_service_;
 };
 
