@@ -1,11 +1,10 @@
-// Copyright (C) 2018-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 //
 
-
 #include <config.h>
 
 #include <exceptions/exceptions.h>
@@ -16,8 +15,8 @@
 #include <dhcpsrv/cfgmgr.h>
 #include <cc/command_interpreter.h>
 #include <cc/data.h>
-#include <cc/data.h>
 #include <stats/stats_mgr.h>
+#include <testutils/gtest_utils.h>
 
 #include <boost/date_time/posix_time/posix_time.hpp>
 #include <gtest/gtest.h>
@@ -136,7 +135,7 @@ public:
         // If we expect nothing and nothing is what we got, we're done.
         if (exp_args->size() == 0) {
             ASSERT_TRUE(actual_args->size() == 0)
-                << "Acutal args should be empty:" << toJSON(actual_args);
+                << "Actual args should be empty:" << toJSON(actual_args);
             return;
         }
 
@@ -346,7 +345,7 @@ public:
         lmptr_ = 0;
     }
 
-    /// @brief Initializes lease manager for v6 operation
+    /// @brief Initializes lease manager for v4 operation
     ///
     /// Creates a lease manager (memfile, trimmed down to keep everything in memory
     /// only) and adds five subnets to the configuration.
@@ -789,7 +788,7 @@ TEST_F(StatCmdsTest, StatLease4GetBadParams) {
 // These test scenarios are all valid, and not expected to throw.
 TEST_F(StatCmdsTest, statLease4GetValid) {
 
-    // Initialize lease manager (false = v4, false = don't add leases)
+    // Initialize lease manager.
     initLeaseMgr4();
 
     // Note timestamp actual values are not important but are included
@@ -860,7 +859,7 @@ TEST_F(StatCmdsTest, statLease4GetValid) {
         "        \"assigned-addresses\", \"declined-addresses\"\n"
         "   ],\n"
         "   \"rows\": [\n"
-        "       [ 20, 16, 10, 3, 0 ],\n"
+        "       [ 20, 16, 10, 3, 0 ]\n"
         "   ],\n"
         "   \"timestamp\": \"2018-05-04 15:03:37.000000\" }\n"
         "}\n"
@@ -887,7 +886,7 @@ TEST_F(StatCmdsTest, statLease4GetValid) {
         "   \"rows\": [\n"
         "       [ 10, 256, 300, 2, 3 ],\n"
         "       [ 20, 16, 10, 3, 0 ],\n"
-        "       [ 30, 256, 0, 0, 0 ],\n"
+        "       [ 30, 256, 0, 0, 0 ]\n"
         "   ],\n"
         "   \"timestamp\": \"2018-05-04 15:03:37.000000\" }\n"
         "}\n"
@@ -914,7 +913,7 @@ TEST_F(StatCmdsTest, statLease4GetValid) {
         "   \"rows\": [\n"
         "       [ 20, 16, 10, 3, 0 ],\n"
         "       [ 30, 256, 0, 0, 0 ],\n"
-        "       [ 40, 16, 4, 4, 0 ],\n"
+        "       [ 40, 16, 4, 4, 0 ]\n"
         "   ],\n"
         "   \"timestamp\": \"2018-05-04 15:03:37.000000\" }\n"
         "}\n"
@@ -941,7 +940,7 @@ TEST_F(StatCmdsTest, statLease4GetValid) {
         "   \"rows\": [\n"
         "       [ 30, 256, 0, 0, 0 ],\n"
         "       [ 40, 16, 4, 4, 0 ],\n"
-        "       [ 50, 256, 2, 1, 1 ],\n"
+        "       [ 50, 256, 2, 1, 1 ]\n"
         "   ],\n"
         "   \"timestamp\": \"2018-05-04 15:03:37.000000\" }\n"
         "}\n"
@@ -967,7 +966,7 @@ TEST_F(StatCmdsTest, statLease4GetValid) {
         "   ],\n"
         "   \"rows\": [\n"
         "       [ 30, 256, 0, 0, 0 ],\n"
-        "       [ 40, 16, 4, 4, 0 ],\n"
+        "       [ 40, 16, 4, 4, 0 ]\n"
         "   ],\n"
         "   \"timestamp\": \"2018-05-04 15:03:37.000000\" }\n"
         "}\n"
@@ -988,7 +987,7 @@ TEST_F(StatCmdsTest, statLease4GetValid) {
 // result in no matching subnets.
 TEST_F(StatCmdsTest, statLease4GetSubnetsNotFound) {
 
-    // Initialize lease manager (false = v4, false = don't add leases)
+    // Initialize lease manager.
     initLeaseMgr4();
 
     // Note timestamp actual values are not important but are included
@@ -1247,7 +1246,7 @@ TEST_F(StatCmdsTest, statLease6GetValid) {
         "}\n"
         },
         {
-        "ALL-Subnets6 arugments omitted",
+        "ALL-Subnets6 arguments omitted",
         "{\n"
         "    \"command\": \"stat-lease6-get\"\n"
         "}",
@@ -1316,7 +1315,7 @@ TEST_F(StatCmdsTest, statLease6GetValid) {
         "   \"rows\": [\n"
         "       [ 10, 65536, 10000, 2, 3, 0, 0, 0 ],\n"
         "       [ 20, 16777216, 10, 3, 0, 0, 0, 0 ],\n"
-        "       [ 30, 16, 2, 1, 1, 65536, 4, 3 ],\n"
+        "       [ 30, 16, 2, 1, 1, 65536, 4, 3 ]\n"
         "   ],\n"
         "   \"timestamp\": \"2018-05-04 15:03:37.000000\" }\n"
         "}\n"
@@ -1474,10 +1473,110 @@ TEST_F(StatCmdsTest, statLease6GetSubnetsNotFound) {
                     (*test).exp_response_, (*test).exp_result_json);
         }
     }
-
 }
 
+// Verifies that statistics for v4 subnets which no longer
+// exist are dropped from the result sets.
+TEST_F(StatCmdsTest, statLease4OrphanedStats) {
 
+    // Initialize lease manager.
+    initLeaseMgr4();
+
+    // Now remove subnets 10, 30, and 50 thereby orphaning their leases.
+    CfgMgr& cfg_mgr = CfgMgr::instance();
+    CfgSubnets4Ptr subnets = cfg_mgr.getCurrentCfg()->getCfgSubnets4();
+    ASSERT_NO_THROW_LOG(subnets->del(10));
+    ASSERT_NO_THROW_LOG(subnets->del(30));
+    ASSERT_NO_THROW_LOG(subnets->del(50));
+    cfg_mgr.commit();
+
+    // Note timestamp actual values are not important but are included
+    // for clarity.
+    std::vector<TestScenario> tests = {
+        {
+        "ALL-Subnets",
+        "{\n"
+        "    \"command\": \"stat-lease4-get\",\n"
+        "    \"arguments\": {"
+        "    }\n"
+        "}",
+        "stat-lease4-get[all subnets]: 2 rows found",
+        "{\n"
+        "\"result-set\": {\n"
+        "   \"columns\": [\n"
+        "        \"subnet-id\", \"total-addresses\",\n"
+        "        \"cumulative-assigned-addresses\",\n"
+        "        \"assigned-addresses\", \"declined-addresses\"\n"
+        "   ],\n"
+        "   \"rows\": [\n"
+        "       [ 20, 16, 0, 3, 0 ],\n"
+        "       [ 40, 16, 0, 4, 0 ]\n"
+        "   ],\n"
+        "   \"timestamp\": \"2018-05-04 15:03:37.000000\" }\n"
+        "}\n"
+        }
+    };
+
+    for (auto test = tests.begin(); test != tests.end(); ++test) {
+        {
+        SCOPED_TRACE((*test).description_);
+        testCommand((*test).command_txt_, CONTROL_RESULT_SUCCESS,
+                    (*test).exp_response_, (*test).exp_result_json);
+        }
+    }
+}
+
+// Verifies that statistics for v6 subnets which no longer
+// exist are dropped from the result sets.
+TEST_F(StatCmdsTest, statLease6OrphanedStats) {
+
+    // Initialize lease manager.
+    initLeaseMgr6();
+
+    // Now remove subnets 10, 30, and 50 thereby orphaning their leases.
+    CfgMgr& cfg_mgr = CfgMgr::instance();
+    CfgSubnets6Ptr subnets = cfg_mgr.getCurrentCfg()->getCfgSubnets6();
+    ASSERT_NO_THROW_LOG(subnets->del(10));
+    ASSERT_NO_THROW_LOG(subnets->del(30));
+    ASSERT_NO_THROW_LOG(subnets->del(50));
+    cfg_mgr.commit();
+
+    // Note timestamp actual values are not important but are included
+    // for clarity.
+    std::vector<TestScenario> tests = {
+        {
+        "ALL-Subnets",
+        "{\n"
+        "    \"command\": \"stat-lease6-get\",\n"
+        "    \"arguments\": {"
+        "    }\n"
+        "}",
+        "stat-lease6-get[all subnets]: 2 rows found",
+        "{\n"
+        "\"result-set\": {\n"
+        "   \"columns\": [\n"
+        "        \"subnet-id\", \"total-nas\",\n"
+        "        \"cumulative-assigned-nas\", \"assigned-nas\",\n"
+        "        \"declined-nas\", \"total-pds\",\n"
+        "        \"cumulative-assigned-pds\", \"assigned-pds\"\n"
+        "   ],\n"
+        "   \"rows\": [\n"
+        "       [ 20, 16777216, 0, 3, 0, 0, 0, 0 ],\n"
+        "       [ 40, 16777216, 0, 0, 0, 0, 0, 0 ]\n"
+        "   ],\n"
+        "   \"timestamp\": \"2018-05-04 15:03:37.000000\" }\n"
+        "}\n"
+        }
+    };
+
+    for (auto test = tests.begin(); test != tests.end(); ++test) {
+        {
+        SCOPED_TRACE((*test).description_);
+        testCommand((*test).command_txt_, CONTROL_RESULT_SUCCESS,
+                    (*test).exp_response_, (*test).exp_result_json);
+        }
+    }
+}
 
 } // end of anonymous namespace
 
