@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -16,6 +16,7 @@
 
 using namespace isc::data;
 using namespace isc::db;
+using namespace isc::util;
 namespace ph = std::placeholders;
 
 /// @brief Test fixture for exercising DbLostCallback invocation
@@ -525,11 +526,7 @@ TEST(DatabaseConnection, toElementDbAccessStringValid) {
     const char* configs[] = {
         "{\n"
         "\"connect-timeout\" : 200, \n"
-        "\"contact-points\": \"contact_str\", \n"
-        "\"consistency\": \"quorum\", \n"
         "\"on-fail\": \"stop-retry-exit\", \n"
-        "\"serial-consistency\": \"serial\", \n"
-        "\"keyspace\": \"keyspace_str\", \n"
         "\"lfc-interval\" : 100, \n"
         "\"host\": \"whatevah\", \n"
         "\"max-reconnect-tries\": 5, \n"
@@ -539,12 +536,13 @@ TEST(DatabaseConnection, toElementDbAccessStringValid) {
         "\"port\" : 300, \n"
         "\"readonly\" : false, \n"
         "\"reconnect-wait-time\": 99, \n"
-        "\"request-timeout\" : 100, \n"
-        "\"tcp-keepalive\": 101, \n"
-        "\"tcp-nodelay\": false, \n"
         "\"type\": \"memfile\", \n"
         "\"user\": \"user_str\", \n"
-        "\"max-row-errors\": 50 \n"
+        "\"max-row-errors\": 50, \n"
+        "\"trust-anchor\": \"my-ca\", \n"
+        "\"cert-file\": \"my-cert.crt\", \n"
+        "\"key-file\": \"my-key.key\", \n"
+        "\"cipher-list\": \"AES\" \n"
         "}\n"
     };
 
