@@ -1,9 +1,11 @@
-// Copyright (C) 2013-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+#include <config.h>
+
 #include <dhcp/dhcp6.h>
 #include <dhcp/option6_client_fqdn.h>
 #include <dns/labelsequence.h>
@@ -40,16 +42,16 @@ public:
     /// @param domain_name A domain name carried by the option given in the
     /// textual format.
     /// @param name_type A value which indicates whether domain-name
-    /// is partial of fully qualified.
+    /// is partial or fully qualified.
     Option6ClientFqdnImpl(const uint8_t flags,
                           const std::string& domain_name,
                           const Option6ClientFqdn::DomainNameType name_type);
 
     /// @brief Constructor, from wire data.
     ///
-    /// @param first An iterator pointing to the begining of the option data
+    /// @param first An iterator pointing to the beginning of the option data
     /// in the wire format.
-    /// @param last An iterator poiting to the end of the option data in the
+    /// @param last An iterator pointing to the end of the option data in the
     /// wire format.
     Option6ClientFqdnImpl(OptionBufferConstIter first,
                           OptionBufferConstIter last);
@@ -87,9 +89,9 @@ public:
 
     /// @brief Parse the Option provided in the wire format.
     ///
-    /// @param first An iterator pointing to the begining of the option data
+    /// @param first An iterator pointing to the beginning of the option data
     /// in the wire format.
-    /// @param last An iterator poiting to the end of the option data in the
+    /// @param last An iterator pointing to the end of the option data in the
     /// wire format.
     void parseWireData(OptionBufferConstIter first,
                        OptionBufferConstIter last);
@@ -401,7 +403,7 @@ Option6ClientFqdn::getDomainNameType() const {
 }
 
 void
-Option6ClientFqdn::pack(isc::util::OutputBuffer& buf) const {
+Option6ClientFqdn::pack(isc::util::OutputBuffer& buf, bool) const {
     // Header = option code and length.
     packHeader(buf);
     // Flags field.
@@ -425,7 +427,7 @@ std::string
 Option6ClientFqdn::toText(int indent) const {
     std::ostringstream stream;
     std::string in(indent, ' '); // base indentation
-    stream << in  << "type=" << type_ << "(CLIENT_FQDN)" << ", "
+    stream << in  << "type=" << type_ << "(CLIENT_FQDN), "
            << "flags: ("
            << "N=" << (getFlag(FLAG_N) ? "1" : "0") << ", "
            << "O=" << (getFlag(FLAG_O) ? "1" : "0") << ", "
