@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -17,32 +17,17 @@ namespace dhcp {
 Pkt::Pkt(uint32_t transid, const isc::asiolink::IOAddress& local_addr,
          const isc::asiolink::IOAddress& remote_addr, uint16_t local_port,
          uint16_t remote_port)
-    :transid_(transid),
-     iface_(""),
-     ifindex_(-1),
-     local_addr_(local_addr),
-     remote_addr_(remote_addr),
-     local_port_(local_port),
-     remote_port_(remote_port),
-     buffer_out_(0),
-     copy_retrieved_options_(false)
-{
+    : transid_(transid), iface_(""), ifindex_(-1), local_addr_(local_addr),
+      remote_addr_(remote_addr), local_port_(local_port),
+      remote_port_(remote_port), buffer_out_(0), copy_retrieved_options_(false) {
 }
 
 Pkt::Pkt(const uint8_t* buf, uint32_t len, const isc::asiolink::IOAddress& local_addr,
          const isc::asiolink::IOAddress& remote_addr, uint16_t local_port,
          uint16_t remote_port)
-    :transid_(0),
-     iface_(""),
-     ifindex_(-1),
-     local_addr_(local_addr),
-     remote_addr_(remote_addr),
-     local_port_(local_port),
-     remote_port_(remote_port),
-     buffer_out_(0),
-     copy_retrieved_options_(false)
-{
-
+    : transid_(0), iface_(""), ifindex_(-1), local_addr_(local_addr),
+      remote_addr_(remote_addr), local_port_(local_port),
+      remote_port_(remote_port), buffer_out_(0), copy_retrieved_options_(false) {
     if (len != 0) {
         if (buf == NULL) {
             isc_throw(InvalidParameter, "data buffer passed to Pkt is NULL");
@@ -59,7 +44,7 @@ Pkt::addOption(const OptionPtr& opt) {
 
 OptionPtr
 Pkt::getNonCopiedOption(const uint16_t type) const {
-    OptionCollection::const_iterator x = options_.find(type);
+    const auto& x = options_.find(type);
     if (x != options_.end()) {
         return (x->second);
     }
@@ -68,7 +53,7 @@ Pkt::getNonCopiedOption(const uint16_t type) const {
 
 OptionPtr
 Pkt::getOption(const uint16_t type) {
-    OptionCollection::iterator x = options_.find(type);
+    const auto& x = options_.find(type);
     if (x != options_.end()) {
         if (copy_retrieved_options_) {
             OptionPtr option_copy = x->second->clone();
@@ -81,9 +66,8 @@ Pkt::getOption(const uint16_t type) {
 
 bool
 Pkt::delOption(uint16_t type) {
-
-    isc::dhcp::OptionCollection::iterator x = options_.find(type);
-    if (x!=options_.end()) {
+    const auto& x = options_.find(type);
+    if (x != options_.end()) {
         options_.erase(x);
         return (true); // delete successful
     } else {
@@ -138,7 +122,6 @@ void
 Pkt::setHWAddrMember(const uint8_t htype, const uint8_t,
                       const std::vector<uint8_t>& hw_addr,
                       HWAddrPtr& storage) {
-
     storage.reset(new HWAddr(hw_addr, htype));
 }
 
@@ -282,5 +265,5 @@ Pkt::getMACFromIPv6(const isc::asiolink::IOAddress& addr) {
     return (mac);
 }
 
-};
-};
+} // end of namespace isc::dhcp
+} // end of namespace isc
