@@ -1,13 +1,16 @@
-// Copyright (C) 2014-2017 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #include <config.h>
+
 #include <dhcp/opaque_data_tuple.h>
 #include <util/buffer.h>
+
 #include <gtest/gtest.h>
+
 #include <algorithm>
 #include <sstream>
 #include <vector>
@@ -18,6 +21,23 @@ using namespace isc::util;
 
 namespace {
 
+struct OpaqueDataTupleLenientParsing : ::testing::Test {
+    void SetUp() final override {
+        // Retain the current setting for future restoration.
+        previous_ = Option::lenient_parsing_;
+
+        // Enable lenient parsing.
+        Option::lenient_parsing_ = true;
+    }
+
+    void TearDown() final override {
+        // Restore.
+        Option::lenient_parsing_ = previous_;
+    }
+
+    bool previous_;
+};
+
 // This test checks that when the default constructor is called, the data buffer
 // is empty.
 TEST(OpaqueDataTuple, constructor) {
@@ -31,35 +51,33 @@ TEST(OpaqueDataTuple, constructor) {
 // Test that the constructor which takes the buffer as argument parses the
 // wire data.
 TEST(OpaqueDataTuple, constructorParse1Byte) {
-    const char wire_data[] = {
+    OpaqueDataTuple::Buffer wire_data = {
         0x0B,                               // Length is 11
         0x48, 0x65, 0x6C, 0x6C, 0x6F, 0x20, // Hello<space>
         0x77, 0x6F, 0x72, 0x6C, 0x64        // world
     };
 
-    OpaqueDataTuple tuple(OpaqueDataTuple::LENGTH_1_BYTE, wire_data,
-                          wire_data + sizeof(wire_data));
+    OpaqueDataTuple tuple(OpaqueDataTuple::LENGTH_1_BYTE, wire_data.begin(),
+                          wire_data.end());
 
     EXPECT_EQ(11, tuple.getLength());
     EXPECT_EQ("Hello world", tuple.getText());
-
 }
 
 // Test that the constructor which takes the buffer as argument parses the
 // wire data.
 TEST(OpaqueDataTuple, constructorParse2Bytes) {
-    const char wire_data[] = {
+    OpaqueDataTuple::Buffer wire_data = {
         0x00, 0x0B,                         // Length is 11
         0x48, 0x65, 0x6C, 0x6C, 0x6F, 0x20, // Hello<space>
         0x77, 0x6F, 0x72, 0x6C, 0x64        // world
     };
 
-    OpaqueDataTuple tuple(OpaqueDataTuple::LENGTH_2_BYTES, wire_data,
-                          wire_data + sizeof(wire_data));
+    OpaqueDataTuple tuple(OpaqueDataTuple::LENGTH_2_BYTES, wire_data.begin(),
+                          wire_data.end());
 
     EXPECT_EQ(11, tuple.getLength());
     EXPECT_EQ("Hello world", tuple.getText());
-
 }
 
 
@@ -70,24 +88,24 @@ TEST(OpaqueDataTuple, assignData) {
     OpaqueDataTuple::Buffer buf = tuple.getData();
     ASSERT_TRUE(buf.empty());
     // Prepare some input data and assign to the tuple.
-    const uint8_t data1[] = {
+    OpaqueDataTuple::Buffer data1 = {
         0xCA, 0xFE, 0xBE, 0xEF
     };
-    tuple.assign(data1, sizeof(data1));
+    tuple.assign(data1.begin(), data1.size());
     // Tuple should now hold the data we assigned.
-    ASSERT_EQ(sizeof(data1), tuple.getLength());
+    ASSERT_EQ(data1.size(), tuple.getLength());
     buf = tuple.getData();
-    EXPECT_TRUE(std::equal(buf.begin(), buf.end(), data1));
+    EXPECT_EQ(buf, data1);
 
     // Prepare the other set of data and assign to the tuple.
-    const uint8_t data2[] = {
+    OpaqueDataTuple::Buffer data2 = {
         1, 2, 3, 4, 5, 6
     };
-    tuple.assign(data2, sizeof(data2));
+    tuple.assign(data2.begin(), data2.size());
     // The new data should have replaced the old data.
-    ASSERT_EQ(sizeof(data2), tuple.getLength());
+    ASSERT_EQ(data2.size(), tuple.getLength());
     buf = tuple.getData();
-    EXPECT_TRUE(std::equal(buf.begin(), buf.end(), data2));
+    EXPECT_EQ(buf, data2);
 }
 
 // This test checks that it is possible to append the data to the tuple using
@@ -98,27 +116,27 @@ TEST(OpaqueDataTuple, appendData) {
     OpaqueDataTuple::Buffer buf = tuple.getData();
     ASSERT_TRUE(buf.empty());
     // Prepare some input data and append to the empty tuple.
-    const uint8_t data1[] = {
+    OpaqueDataTuple::Buffer data1 = {
         0xCA, 0xFE, 0xBE, 0xEF
     };
-    tuple.append(data1, sizeof(data1));
+    tuple.append(data1.begin(), data1.size());
     // The tuple should now hold only the data we appended.
-    ASSERT_EQ(sizeof(data1), tuple.getLength());
+    ASSERT_EQ(data1.size(), tuple.getLength());
     buf = tuple.getData();
-    EXPECT_TRUE(std::equal(buf.begin(), buf.end(), data1));
+    EXPECT_EQ(buf, data1);
     // Prepare the new set of data and append.
-    const uint8_t data2[] = {
+    OpaqueDataTuple::Buffer data2 = {
         1, 2, 3, 4, 5, 6
     };
-    tuple.append(data2, sizeof(data2));
+    tuple.append(data2.begin(), data2.size());
     // We expect that the tuple now has both sets of data we appended. In order
     // to verify that, we have to concatenate the input data1 and data2.
-    std::vector<uint8_t> data12(data1, data1 + sizeof(data1));
-    data12.insert(data12.end(), data2, data2 + sizeof(data2));
+    OpaqueDataTuple::Buffer data12(data1.begin(), data1.end());
+    data12.insert(data12.end(), data2.begin(), data2.end());
     // The size of the tuple should be a sum of data1 and data2 lengths.
-    ASSERT_EQ(sizeof(data1) + sizeof(data2), tuple.getLength());
+    ASSERT_EQ(data1.size() + data2.size(), tuple.getLength());
     buf = tuple.getData();
-    EXPECT_TRUE(std::equal(buf.begin(), buf.end(), data12.begin()));
+    EXPECT_EQ(buf, data12);
 }
 
 // This test checks that it is possible to assign the string to the tuple.
@@ -225,7 +243,6 @@ TEST(OpaqueDataTuple, operatorOutputStream) {
     EXPECT_NO_THROW(tuple = " and some other text");
     EXPECT_NO_THROW(s << tuple);
     EXPECT_EQ(s.str(), "Some text and some other text");
-
 }
 
 // This test verifies that the value of the tuple can be initialized from the
@@ -253,17 +270,22 @@ TEST(OpaqueDataTuple, pack1Byte) {
     OpaqueDataTuple tuple(OpaqueDataTuple::LENGTH_1_BYTE);
     // Initially, the tuple should be empty.
     ASSERT_EQ(0, tuple.getLength());
-    // The empty data doesn't make much sense, so the pack() should not
-    // allow it.
+    // It turns out that Option 124 can be sent with 0 length Opaque Data
+    // See #2021 for more details
     OutputBuffer out_buf(10);
-    EXPECT_THROW(tuple.pack(out_buf), OpaqueDataTupleError);
+    ASSERT_NO_THROW(tuple.pack(out_buf));
+    ASSERT_EQ(1, out_buf.getLength());
+    const uint8_t* zero_len = static_cast<const uint8_t*>(out_buf.getData());
+    ASSERT_EQ(0, *zero_len);
+    // Reset the output buffer for another test.
+    out_buf.clear();
     // Set the data for tuple.
     std::vector<uint8_t> data;
     for (uint8_t i = 0; i < 100; ++i) {
         data.push_back(i);
     }
     tuple.assign(data.begin(), data.size());
-    // The pack should now succeed.
+    // Packing the data should succeed.
     ASSERT_NO_THROW(tuple.pack(out_buf));
     // The rendered buffer should be 101 bytes long - 1 byte for length,
     // 100 bytes for the actual data.
@@ -312,17 +334,22 @@ TEST(OpaqueDataTuple, pack2Bytes) {
     OpaqueDataTuple tuple(OpaqueDataTuple::LENGTH_2_BYTES);
     // Initially, the tuple should be empty.
     ASSERT_EQ(0, tuple.getLength());
-    // The empty data doesn't make much sense, so the pack() should not
-    // allow it.
+    // It turns out that Option 124 can be sent with 0 length Opaque Data
+    // See #2021 for more details
     OutputBuffer out_buf(10);
-    EXPECT_THROW(tuple.pack(out_buf), OpaqueDataTupleError);
+    ASSERT_NO_THROW(tuple.pack(out_buf));
+    ASSERT_EQ(2, out_buf.getLength());
+    const uint16_t* zero_len = static_cast<const uint16_t*>(out_buf.getData());
+    ASSERT_EQ(0, *zero_len);
+    // Reset the output buffer for another test.
+    out_buf.clear();
     // Set the data for tuple.
     std::vector<uint8_t> data;
     for (unsigned i = 0; i < 512; ++i) {
         data.push_back(i & 0xff);
     }
     tuple.assign(data.begin(), data.size());
-    // The pack should now succeed.
+    // Packing the data should succeed.
     ASSERT_NO_THROW(tuple.pack(out_buf));
     // The rendered buffer should be 514 bytes long - 2 bytes for length,
     // 512 bytes for the actual data.
@@ -360,13 +387,13 @@ TEST(OpaqueDataTuple, pack2Bytes) {
 // This test verifies that the tuple is decoded from the wire format.
 TEST(OpaqueDataTuple, unpack1Byte) {
     OpaqueDataTuple tuple(OpaqueDataTuple::LENGTH_1_BYTE);
-    const char wire_data[] = {
+    OpaqueDataTuple::Buffer wire_data = {
         0x0B,                               // Length is 11
         0x48, 0x65, 0x6C, 0x6C, 0x6F, 0x20, // Hello<space>
         0x77, 0x6F, 0x72, 0x6C, 0x64        // world
     };
 
-    ASSERT_NO_THROW(tuple.unpack(wire_data, wire_data + sizeof(wire_data)));
+    ASSERT_NO_THROW(tuple.unpack(wire_data.begin(), wire_data.end()));
     EXPECT_EQ(11, tuple.getLength());
     EXPECT_EQ("Hello world", tuple.getText());
 }
@@ -378,38 +405,53 @@ TEST(OpaqueDataTuple, unpack1ByteZeroLength) {
     EXPECT_NO_THROW(tuple = "Hello world");
     ASSERT_NE(tuple.getLength(), 0);
 
-    const char wire_data[] = {
+    OpaqueDataTuple::Buffer wire_data = {
         0
     };
-    ASSERT_NO_THROW(tuple.unpack(wire_data, wire_data + sizeof(wire_data)));
+    ASSERT_NO_THROW(tuple.unpack(wire_data.begin(), wire_data.end()));
 
     EXPECT_EQ(0, tuple.getLength());
 }
 
+// This test verifies that the tuple having a length of 0, followed by no
+// data, is decoded from the wire format.
+TEST(OpaqueDataTuple, unpack1ByteZeroLengthNoData) {
+    OpaqueDataTuple tuple(OpaqueDataTuple::LENGTH_1_BYTE);
+    OpaqueDataTuple::Buffer wire_data = {0};
+    ASSERT_NO_THROW(tuple.unpack(wire_data.begin(), wire_data.end()));
+}
+
+// This test verifies that the tuple having a length of 0, followed by no
+// data, is decoded from the wire format.
+TEST(OpaqueDataTuple, unpack2ByteZeroLengthNoData) {
+    OpaqueDataTuple tuple(OpaqueDataTuple::LENGTH_2_BYTES);
+    OpaqueDataTuple::Buffer wire_data = {0, 0};
+    ASSERT_NO_THROW(tuple.unpack(wire_data.begin(), wire_data.end()));
+}
+
 // This test verifies that exception is thrown if the empty buffer is being
 // parsed.
 TEST(OpaqueDataTuple, unpack1ByteEmptyBuffer) {
     OpaqueDataTuple tuple(OpaqueDataTuple::LENGTH_1_BYTE);
-    const char wire_data[] = {
-        1, 2, 3
-    };
-    EXPECT_THROW(tuple.unpack(wire_data, wire_data), OpaqueDataTupleError);
+    OpaqueDataTuple::Buffer wire_data = {};
+    EXPECT_THROW(tuple.unpack(wire_data.begin(), wire_data.end()),
+                 OpaqueDataTupleError);
 }
 
 // This test verifies that exception is thrown when parsing truncated buffer.
 TEST(OpaqueDataTuple, unpack1ByteTruncatedBuffer) {
    OpaqueDataTuple tuple(OpaqueDataTuple::LENGTH_1_BYTE);
-    const char wire_data[] = {
+    OpaqueDataTuple::Buffer wire_data = {
         10, 2, 3
     };
-    EXPECT_THROW(tuple.unpack(wire_data, wire_data + sizeof(wire_data)),
+    EXPECT_THROW(tuple.unpack(wire_data.begin(), wire_data.end()),
                  OpaqueDataTupleError);
 }
 
 // This test verifies that the tuple is decoded from the wire format.
 TEST(OpaqueDataTuple, unpack2Byte) {
     OpaqueDataTuple tuple(OpaqueDataTuple::LENGTH_2_BYTES);
-    std::vector<uint8_t> wire_data;
+    OpaqueDataTuple::Buffer wire_data;
     // Set tuple length to 400 (0x190).
     wire_data.push_back(1);
     wire_data.push_back(0x90);
@@ -434,11 +476,11 @@ TEST(OpaqueDataTuple, unpack2ByteZeroLength) {
     EXPECT_NO_THROW(tuple = "Hello world");
     ASSERT_NE(tuple.getLength(), 0);
     // The buffer holds just a length field with the value of 0.
-    const char wire_data[] = {
+    OpaqueDataTuple::Buffer wire_data = {
         0, 0
     };
     // The empty tuple should be successfully decoded.
-    ASSERT_NO_THROW(tuple.unpack(wire_data, wire_data + sizeof(wire_data)));
+    ASSERT_NO_THROW(tuple.unpack(wire_data.begin(), wire_data.end()));
     // The data should be replaced with an empty buffer.
     EXPECT_EQ(0, tuple.getLength());
 }
@@ -447,28 +489,35 @@ TEST(OpaqueDataTuple, unpack2ByteZeroLength) {
 // parsed.
 TEST(OpaqueDataTuple, unpack2ByteEmptyBuffer) {
     OpaqueDataTuple tuple(OpaqueDataTuple::LENGTH_2_BYTES);
-    //  Initialize the input buffer with some data, just to avoid initializing
-    // empty array.
-    const char wire_data[] = {
-        1, 2, 3
-    };
+    OpaqueDataTuple::Buffer wire_data = {};
     // Pass empty buffer (first iterator equal to second iterator).
     // This should not be accepted.
-    EXPECT_THROW(tuple.unpack(wire_data, wire_data), OpaqueDataTupleError);
+    EXPECT_THROW(tuple.unpack(wire_data.begin(), wire_data.end()),
+                 OpaqueDataTupleError);
 }
 
-// This test verifies that exception if thrown when parsing truncated buffer.
+// This test verifies that exception is thrown when parsing truncated buffer.
 TEST(OpaqueDataTuple, unpack2ByteTruncatedBuffer) {
    OpaqueDataTuple tuple(OpaqueDataTuple::LENGTH_2_BYTES);
    // Specify the data with the length of 10, but limit the buffer size to
    // 2 bytes.
-   const char wire_data[] = {
+   OpaqueDataTuple::Buffer wire_data = {
        0, 10, 2, 3
    };
    // This should fail because the buffer is truncated.
-   EXPECT_THROW(tuple.unpack(wire_data, wire_data + sizeof(wire_data)),
+   EXPECT_THROW(tuple.unpack(wire_data.begin(), wire_data.end()),
                 OpaqueDataTupleError);
 }
 
+// Test that an exception is not thrown when parsing in lenient mode.
+TEST_F(OpaqueDataTupleLenientParsing, unpack) {
+    OpaqueDataTuple tuple(OpaqueDataTuple::LENGTH_2_BYTES);
+    // Specify the data with the length of 10, but limit the buffer size to 2.
+   OpaqueDataTuple::Buffer wire_data = {
+        0, 10, 2, 3
+    };
+    EXPECT_NO_THROW(tuple.unpack(wire_data.begin(), wire_data.end()));
+    EXPECT_EQ(tuple.getData(), OpaqueDataTuple::Buffer({2, 3}));
+}
 
 } // anonymous namespace
