@@ -1,4 +1,4 @@
-// Copyright (C) 2011-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -1545,10 +1545,10 @@ TEST_F(Pkt6Test, getMACFromRemoteIdRelayOption) {
     pkt.setIface(iface->getName());
     pkt.setIndex(iface->getIndex());
 
-    // Generate option data with randomly picked enterprise number and MAC address
+    // Generate option data with randomly picked enterprise number and remote-id
     const uint8_t opt_data[] = {
         1, 2, 3, 4,  // enterprise-number
-        0xa, 0xb, 0xc, 0xd, 0xe, 0xf // MAC
+        0xa, 0xb, 0xc, 0xd, 0xe, 0xf // remote-id can be used as a standard MAC
     };
 
     // Create option with number 37 (remote-id relay agent option)
@@ -1579,6 +1579,64 @@ TEST_F(Pkt6Test, getMACFromRemoteIdRelayOption) {
     EXPECT_EQ(HWAddr::HWADDR_SOURCE_REMOTE_ID, mac->source_);
 }
 
+// Test checks whether getMACFromRemoteIdRelayOption() returns the hardware (MAC)
+// address properly from a relayed message (even if the remote-id is longer than
+// 20 bytes).
+TEST_F(Pkt6Test, getMACFromRemoteIdRelayOptionExtendedValue) {
+
+    // Create a solicit message.
+    Pkt6 pkt(DHCPV6_SOLICIT, 1234);
+
+    // This should fail as the message is't relayed yet.
+    EXPECT_FALSE(pkt.getMAC(HWAddr::HWADDR_SOURCE_REMOTE_ID));
+
+    // Let's get the first interface
+    IfacePtr iface = IfaceMgr::instance().getIface(1);
+    ASSERT_TRUE(iface);
+
+    // and set source interface data properly. getMACFromIPv6LinkLocal attempts
+    // to use source interface to obtain hardware type
+    pkt.setIface(iface->getName());
+    pkt.setIndex(iface->getIndex());
+
+    // Generate option data with randomly picked enterprise number and remote-id
+    const uint8_t opt_data[] = {
+        1, 2, 3, 4,  // enterprise-number
+        0xa, 0xb, 0xc, 0xd, 0xe, 0xf, // remote-id can be longer than 20 bytes,
+        0xa, 0xb, 0xc, 0xd, 0xe, 0xf, // truncate it so that is can be used as
+        0xa, 0xb, 0xc, 0xd, 0xe, 0xf, // a standard MAC
+        0xa, 0xb, 0xc, 0xd, 0xe, 0xf
+    };
+
+    // Create option with number 37 (remote-id relay agent option)
+    OptionPtr relay_opt(new Option(Option::V6, D6O_REMOTE_ID,
+                        OptionBuffer(opt_data, opt_data + sizeof(opt_data))));
+
+    // First simulate relaying message without adding remote-id option
+    Pkt6::RelayInfo info;
+    pkt.addRelayInfo(info);
+    ASSERT_EQ(1, pkt.relay_info_.size());
+
+    // This should fail as the remote-id option isn't there
+    EXPECT_FALSE(pkt.getMAC(HWAddr::HWADDR_SOURCE_REMOTE_ID));
+
+    // Now add this option to the relayed message
+    info.options_.insert(make_pair(relay_opt->getType(), relay_opt));
+    pkt.addRelayInfo(info);
+    ASSERT_EQ(2, pkt.relay_info_.size());
+
+    // This should work now
+    HWAddrPtr mac = pkt.getMAC(HWAddr::HWADDR_SOURCE_REMOTE_ID);
+    ASSERT_TRUE(mac);
+
+    stringstream tmp;
+    tmp << "hwtype=" << (int)iface->getHWType()
+        << " 0a:0b:0c:0d:0e:0f:0a:0b:0c:0d:0e:0f:0a:0b:0c:0d:0e:0f:0a:0b";
+
+    EXPECT_EQ(tmp.str(), mac->toText(true));
+    EXPECT_EQ(HWAddr::HWADDR_SOURCE_REMOTE_ID, mac->source_);
+}
+
 // This test verifies that a solicit that passed through two relays is parsed
 // properly. In particular the second relay (outer encapsulation) included RSOO
 // (Relay Supplied Options option). This test checks whether it was parsed
@@ -2057,4 +2115,4 @@ TEST_F(Pkt6Test, relayDataOption) {
     EXPECT_EQ(orig_data, clone_data);
 }
 
-}
+}  // namespace
