@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -99,7 +99,32 @@ D2Dhcid::toStr() const {
 void
 D2Dhcid::fromClientId(const std::vector<uint8_t>& clientid_data,
                       const std::vector<uint8_t>& wire_fqdn) {
-    createDigest(DHCID_ID_CLIENTID, clientid_data, wire_fqdn);
+    // IPv4 Client ID containing a DUID looks like this in RFC4361
+    //  Type  IAID                DUID
+    // +-----+----+----+----+----+----+----+---
+    // | 255 | i1 | i2 | i3 | i4 | d1 | d2 |...
+    // +-----+----+----+----+----+----+----+---
+    if (!clientid_data.empty() && clientid_data[0] == 255) {
+        if (clientid_data.size() <= 5) {
+            isc_throw(isc::dhcp_ddns::DhcidRdataComputeError,
+                      "unable to compute DHCID from client identifier, embedded DUID "
+                      "length of: " << clientid_data.size() << ", is too short");
+        }
+        // RFC3315 states that the DUID is a type code of 2 octets followed
+        // by no more then 128 octets. So add the 5 from above and make sure
+        // the length is not too long.
+        if (clientid_data.size() > 135) {
+            isc_throw(isc::dhcp_ddns::DhcidRdataComputeError,
+                      "unable to compute DHCID from client identifier, embedded DUID "
+                      "length of: " << clientid_data.size() << ", is too long");
+        }
+        std::vector<uint8_t>::const_iterator start = clientid_data.begin() + 5;
+        std::vector<uint8_t>::const_iterator end = clientid_data.end();
+        std::vector<uint8_t> duid(start, end);
+        createDigest(DHCID_ID_DUID, duid, wire_fqdn);
+    } else {
+        createDigest(DHCID_ID_CLIENTID, clientid_data, wire_fqdn);
+    }
 }
 
 void
@@ -200,7 +225,8 @@ operator<<(std::ostream& os, const D2Dhcid& dhcid) {
 NameChangeRequest::NameChangeRequest()
     : change_type_(CHG_ADD), forward_change_(false),
     reverse_change_(false), fqdn_(""), ip_io_address_("0.0.0.0"),
-    dhcid_(), lease_expires_on_(), lease_length_(0), status_(ST_NEW) {
+    dhcid_(), lease_expires_on_(), lease_length_(0), conflict_resolution_(true),
+    status_(ST_NEW) {
 }
 
 NameChangeRequest::NameChangeRequest(const NameChangeType change_type,
@@ -208,11 +234,13 @@ NameChangeRequest::NameChangeRequest(const NameChangeType change_type,
             const std::string& fqdn, const std::string& ip_address,
             const D2Dhcid& dhcid,
             const uint64_t lease_expires_on,
-            const uint32_t lease_length)
+            const uint32_t lease_length,
+            const bool conflict_resolution)
     : change_type_(change_type), forward_change_(forward_change),
     reverse_change_(reverse_change), fqdn_(fqdn), ip_io_address_("0.0.0.0"),
     dhcid_(dhcid), lease_expires_on_(lease_expires_on),
-    lease_length_(lease_length), status_(ST_NEW) {
+    lease_length_(lease_length), conflict_resolution_(conflict_resolution),
+    status_(ST_NEW) {
 
     // User setter to validate fqdn.
     setFqdn(fqdn);
@@ -248,7 +276,7 @@ NameChangeRequest::fromFormat(const NameChangeFormat format,
             // NameChangeRequest instance.  Note the factory may throw
             // NcrMessageError.
             ncr = NameChangeRequest::fromJSON(string_data);
-        } catch (isc::util::InvalidBufferPosition& ex) {
+        } catch (const isc::util::InvalidBufferPosition& ex) {
             // Read error accessing data in InputBuffer.
             isc_throw(NcrMessageError, "fromFormat: buffer read error: "
                       << ex.what());
@@ -299,7 +327,7 @@ NameChangeRequest::fromJSON(const std::string& json) {
     isc::data::ElementPtr elements;
     try {
         elements = isc::data::Element::fromJSON(json);
-    } catch (isc::data::JSONError& ex) {
+    } catch (const isc::data::JSONError& ex) {
         isc_throw(NcrMessageError,
                   "Malformed NameChangeRequest JSON: " << ex.what());
     }
@@ -316,7 +344,7 @@ NameChangeRequest::fromJSON(const std::string& json) {
     // NcrMessageError if the given Element is the wrong type or its data
     // content is lexically invalid.   If the element is NOT found in the
     // map, getElement will throw NcrMessageError indicating the missing
-    // member. Currently there are no optional values.
+    // member.
     element = ncr->getElement("change-type", element_map);
     ncr->setChangeType(element);
 
@@ -341,6 +369,15 @@ NameChangeRequest::fromJSON(const std::string& json) {
     element = ncr->getElement("lease-length", element_map);
     ncr->setLeaseLength(element);
 
+    // For backward compatibility  use-conflict-resolution is optional
+    // and defaults to true.
+    auto found = element_map.find("use-conflict-resolution");
+    if (found != element_map.end()) {
+        ncr->setConflictResolution(found->second);
+    } else {
+        ncr->setConflictResolution(true);
+    }
+
     // All members were in the Element set and were correct lexically. Now
     // validate the overall content semantically.  This will throw an
     // NcrMessageError if anything is amiss.
@@ -366,7 +403,9 @@ NameChangeRequest::toJSON() const  {
         << "\"ip-address\":\"" << getIpAddress() << "\","
         << "\"dhcid\":\"" << getDhcid().toStr() << "\","
         << "\"lease-expires-on\":\""  << getLeaseExpiresOnStr() << "\","
-        << "\"lease-length\":" << getLeaseLength() << "}";
+        << "\"lease-length\":" << getLeaseLength() << ","
+        << "\"use-conflict-resolution\":"
+        << (useConflictResolution() ? "true" : "false") << "}";
 
     return (stream.str());
 }
@@ -420,7 +459,7 @@ NameChangeRequest::setChangeType(isc::data::ConstElementPtr element) {
     try {
         // Get the element's integer value.
         raw_value = element->intValue();
-    } catch (isc::data::TypeError& ex) {
+    } catch (const isc::data::TypeError& ex) {
         // We expect a integer Element type, don't have one.
         isc_throw(NcrMessageError,
                   "Wrong data type for change_type: " << ex.what());
@@ -447,7 +486,7 @@ NameChangeRequest::setForwardChange(isc::data::ConstElementPtr element) {
     try {
         // Get the element's boolean value.
         value = element->boolValue();
-    } catch (isc::data::TypeError& ex) {
+    } catch (const isc::data::TypeError& ex) {
         // We expect a boolean Element type, don't have one.
         isc_throw(NcrMessageError,
                   "Wrong data type for forward-change: " << ex.what());
@@ -468,7 +507,7 @@ NameChangeRequest::setReverseChange(isc::data::ConstElementPtr element) {
     try {
         // Get the element's boolean value.
         value = element->boolValue();
-    } catch (isc::data::TypeError& ex) {
+    } catch (const isc::data::TypeError& ex) {
         // We expect a boolean Element type, don't have one.
         isc_throw(NcrMessageError,
                   "Wrong data type for reverse_change: " << ex.what());
@@ -532,7 +571,7 @@ void
 NameChangeRequest::setLeaseExpiresOn(const std::string&  value) {
     try {
         lease_expires_on_ = isc::util::timeFromText64(value);
-    } catch(...) {
+    } catch (...) {
         // We were given an invalid string, so throw.
         isc_throw(NcrMessageError,
             "Invalid date-time string: [" << value << "]");
@@ -556,7 +595,7 @@ NameChangeRequest::setLeaseLength(isc::data::ConstElementPtr element) {
     try {
         // Get the element's integer value.
         value = element->intValue();
-    } catch (isc::data::TypeError& ex) {
+    } catch (const isc::data::TypeError& ex) {
         // We expect a integer Element type, don't have one.
         isc_throw(NcrMessageError,
                   "Wrong data type for lease_length: " << ex.what());
@@ -576,6 +615,27 @@ NameChangeRequest::setLeaseLength(isc::data::ConstElementPtr element) {
     setLeaseLength(static_cast<uint32_t>(value));
 }
 
+void
+NameChangeRequest::setConflictResolution(const bool value) {
+    conflict_resolution_ = value;
+}
+
+void
+NameChangeRequest::setConflictResolution(isc::data::ConstElementPtr element) {
+    bool value;
+    try {
+        // Get the element's boolean value.
+        value = element->boolValue();
+    } catch (const isc::data::TypeError& ex) {
+        // We expect a boolean Element type, don't have one.
+        isc_throw(NcrMessageError,
+                  "Wrong data type for use-conflict-resolution: " << ex.what());
+    }
+
+    // Good to go, make the assignment.
+    setConflictResolution(value);
+}
+
 void
 NameChangeRequest::setStatus(const NameChangeStatus value) {
     status_ = value;
@@ -606,7 +666,9 @@ NameChangeRequest::toText() const {
            << "IP Address: [" << ip_io_address_ << "]" << std::endl
            << "DHCID: [" << dhcid_.toStr() << "]" << std::endl
            << "Lease Expires On: " << getLeaseExpiresOnStr() << std::endl
-           << "Lease Length: " << lease_length_ << std::endl;
+           << "Lease Length: " << lease_length_ << std::endl
+           << "Conflict Resolution: " << (conflict_resolution_ ? "yes" : "no")
+           << std::endl;
 
     return (stream.str());
 }
@@ -620,7 +682,8 @@ NameChangeRequest::operator == (const NameChangeRequest& other) {
             (ip_io_address_ == other.ip_io_address_) &&
             (dhcid_ == other.dhcid_) &&
             (lease_expires_on_ == other.lease_expires_on_) &&
-            (lease_length_ == other.lease_length_));
+            (lease_length_ == other.lease_length_) &&
+            (conflict_resolution_ == other.conflict_resolution_));
 }
 
 bool
