@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -93,13 +93,11 @@ CfgSubnets4::merge(CfgOptionDefPtr cfg_def, CfgSharedNetworks4Ptr networks,
     // Iterate over the subnets to be merged. They will replace the existing
     // subnets with the same id. All new subnets will be inserted into the
     // configuration into which we're merging.
-    auto other_subnets = other.getAll();
-    for (auto other_subnet = other_subnets->begin();
-         other_subnet != other_subnets->end();
-         ++other_subnet) {
+    auto const& other_subnets = other.getAll();
+    for (auto const& other_subnet : (*other_subnets)) {
 
         // Check if there is a subnet with the same ID.
-        auto subnet_id_it = index_id.find((*other_subnet)->getID());
+        auto subnet_id_it = index_id.find(other_subnet->getID());
         if (subnet_id_it != index_id.end()) {
 
             // Subnet found.
@@ -107,7 +105,7 @@ CfgSubnets4::merge(CfgOptionDefPtr cfg_def, CfgSharedNetworks4Ptr networks,
 
             // If the existing subnet and other subnet
             // are the same instance skip it.
-            if (existing_subnet == *other_subnet) {
+            if (existing_subnet == other_subnet) {
                 continue;
             }
 
@@ -129,7 +127,7 @@ CfgSubnets4::merge(CfgOptionDefPtr cfg_def, CfgSharedNetworks4Ptr networks,
         }
 
         // Check if there is a subnet with the same prefix.
-        auto subnet_prefix_it = index_prefix.find((*other_subnet)->toText());
+        auto subnet_prefix_it = index_prefix.find(other_subnet->toText());
         if (subnet_prefix_it != index_prefix.end()) {
 
             // Subnet found.
@@ -153,26 +151,28 @@ CfgSubnets4::merge(CfgOptionDefPtr cfg_def, CfgSharedNetworks4Ptr networks,
         }
 
         // Create the subnet's options based on the given definitions.
-        (*other_subnet)->getCfgOption()->createOptions(cfg_def);
-        for (auto pool : (*other_subnet)->getPoolsWritable(Lease::TYPE_V4)) {
+        other_subnet->getCfgOption()->createOptions(cfg_def);
+
+        // Create the options for pool based on the given definitions.
+        for (auto const& pool : other_subnet->getPoolsWritable(Lease::TYPE_V4)) {
             pool->getCfgOption()->createOptions(cfg_def);
         }
 
         // Add the "other" subnet to the our collection of subnets.
-        static_cast<void>(subnets_.insert(*other_subnet));
+        static_cast<void>(subnets_.insert(other_subnet));
 
         // If it belongs to a shared network, find the network and
         // add the subnet to it
-        std::string network_name = (*other_subnet)->getSharedNetworkName();
+        std::string network_name = other_subnet->getSharedNetworkName();
         if (!network_name.empty()) {
             SharedNetwork4Ptr network = networks->getByName(network_name);
             if (network) {
-                network->add(*other_subnet);
+                network->add(other_subnet);
             } else {
                 // This implies the shared-network collection we were given
                 // is out of sync with the subnets we were given.
                 isc_throw(InvalidOperation, "Cannot assign subnet ID of "
-                          << (*other_subnet)->getID()
+                          << other_subnet->getID()
                           << " to shared network: " << network_name
                           << ", network does not exist");
             }
@@ -231,30 +231,29 @@ CfgSubnets4::initSelector(const Pkt4Ptr& query) {
                 if (link_select_buf.size() == sizeof(uint32_t)) {
                     selector.option_select_ =
                         IOAddress::fromBytes(AF_INET, &link_select_buf[0]);
+                    return (selector);
                 }
             }
         }
-    } else {
-        // Or Subnet Selection option
-        OptionPtr sbnsel = query->getOption(DHO_SUBNET_SELECTION);
-        if (sbnsel) {
-            OptionCustomPtr oc =
-                boost::dynamic_pointer_cast<OptionCustom>(sbnsel);
-            if (oc) {
-                selector.option_select_ = oc->readAddress();
-            }
+    }
+    // The query does not include a RAI option or that option does
+    // not contain the link-selection sub-option. Try subnet-selection
+    // option.
+    OptionPtr sbnsel = query->getOption(DHO_SUBNET_SELECTION);
+    if (sbnsel) {
+        OptionCustomPtr oc =
+            boost::dynamic_pointer_cast<OptionCustom>(sbnsel);
+        if (oc) {
+            selector.option_select_ = oc->readAddress();
         }
     }
-
     return (selector);
 }
 
 Subnet4Ptr
 CfgSubnets4::selectSubnet4o6(const SubnetSelector& selector) const {
-
-    for (Subnet4Collection::const_iterator subnet = subnets_.begin();
-         subnet != subnets_.end(); ++subnet) {
-        Cfg4o6& cfg4o6 = (*subnet)->get4o6();
+    for (auto const& subnet : subnets_) {
+        Cfg4o6& cfg4o6 = subnet->get4o6();
 
         // Is this an 4o6 subnet at all?
         if (!cfg4o6.enabled()) {
@@ -270,34 +269,38 @@ CfgSubnets4::selectSubnet4o6(const SubnetSelector& selector) const {
             IOAddress last = lastAddrInPrefix(pref.first, pref.second);
             if ((first <= selector.remote_address_) &&
                 (selector.remote_address_ <= last)) {
-                return (*subnet);
+                return (subnet);
             }
         }
 
         // Second match criteria: check if the interface-id matches
         if (cfg4o6.getInterfaceId() && selector.interface_id_ &&
             cfg4o6.getInterfaceId()->equals(selector.interface_id_)) {
-            return (*subnet);
+            return (subnet);
         }
 
         // Third match criteria: check if the interface name matches
         if (!cfg4o6.getIface4o6().empty() && !selector.iface_name_.empty()
             && cfg4o6.getIface4o6() == selector.iface_name_) {
-            return (*subnet);
+            return (subnet);
         }
     }
 
+    LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE, DHCPSRV_SUBNET4O6_SELECT_FAILED);
+
     // Ok, wasn't able to find any matching subnet.
     return (Subnet4Ptr());
 }
 
 Subnet4Ptr
 CfgSubnets4::selectSubnet(const SubnetSelector& selector) const {
-
     // First use RAI link select sub-option or subnet select option
     if (!selector.option_select_.isV4Zero()) {
         return (selectSubnet(selector.option_select_,
                              selector.client_classes_));
+    } else {
+        LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE,
+                  DHCPSRV_SUBNET4_SELECT_NO_RAI_OPTIONS);
     }
 
     // If relayed message has been received, try to match the giaddr with the
@@ -306,30 +309,39 @@ CfgSubnets4::selectSubnet(const SubnetSelector& selector) const {
     // addresses across all subnets, but we need to verify that for all subnets
     // before we can try to use the giaddr to match with the subnet prefix.
     if (!selector.giaddr_.isV4Zero()) {
-        for (Subnet4Collection::const_iterator subnet = subnets_.begin();
-             subnet != subnets_.end(); ++subnet) {
+        for (auto const& subnet : subnets_) {
 
             // If relay information is specified for this subnet, it must match.
             // Otherwise, we ignore this subnet.
-            if ((*subnet)->hasRelays()) {
-                if (!(*subnet)->hasRelayAddress(selector.giaddr_)) {
+            if (subnet->hasRelays()) {
+                if (!subnet->hasRelayAddress(selector.giaddr_)) {
                     continue;
                 }
             } else {
                 // Relay information is not specified on the subnet level,
                 // so let's try matching on the shared network level.
                 SharedNetwork4Ptr network;
-                (*subnet)->getSharedNetwork(network);
+                subnet->getSharedNetwork(network);
                 if (!network || !(network->hasRelayAddress(selector.giaddr_))) {
                     continue;
                 }
             }
 
             // If a subnet meets the client class criteria return it.
-            if ((*subnet)->clientSupported(selector.client_classes_)) {
-                return (*subnet);
+            if (subnet->clientSupported(selector.client_classes_)) {
+                LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE,
+                          DHCPSRV_CFGMGR_SUBNET4_RELAY)
+                    .arg(subnet->toText())
+                    .arg(selector.giaddr_.toText());
+                return (subnet);
             }
         }
+        LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE,
+                  DHCPSRV_SUBNET4_SELECT_BY_RELAY_ADDRESS_NO_MATCH)
+            .arg(selector.giaddr_.toText());
+    } else {
+        LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE,
+                  DHCPSRV_SUBNET4_SELECT_NO_RELAY_ADDRESS);
     }
 
     // If we got to this point it means that we were not able to match the
@@ -381,6 +393,9 @@ CfgSubnets4::selectSubnet(const SubnetSelector& selector) const {
 
     // Unable to find a suitable address to use for subnet selection.
     if (address.isV4Zero()) {
+        LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE,
+                  DHCPSRV_SUBNET4_SELECT_NO_USABLE_ADDRESS);
+
         return (Subnet4Ptr());
     }
 
@@ -392,15 +407,13 @@ CfgSubnets4::selectSubnet(const SubnetSelector& selector) const {
 Subnet4Ptr
 CfgSubnets4::selectSubnet(const std::string& iface,
                           const ClientClasses& client_classes) const {
-    for (Subnet4Collection::const_iterator subnet = subnets_.begin();
-         subnet != subnets_.end(); ++subnet) {
-
+    for (auto const& subnet : subnets_) {
         Subnet4Ptr subnet_selected;
 
         // First, try subnet specific interface name.
-        if (!(*subnet)->getIface().empty()) {
-            if ((*subnet)->getIface() == iface) {
-                subnet_selected = (*subnet);
+        if (!subnet->getIface(Network4::Inheritance::NONE).empty()) {
+            if (subnet->getIface(Network4::Inheritance::NONE) == iface) {
+                subnet_selected = subnet;
             }
 
         } else {
@@ -408,38 +421,40 @@ CfgSubnets4::selectSubnet(const std::string& iface,
             // we can match with shared network specific setting of
             // the interface.
             SharedNetwork4Ptr network;
-            (*subnet)->getSharedNetwork(network);
-            if (network && !network->getIface().empty() &&
-                (network->getIface() == iface)) {
-                subnet_selected = (*subnet);
+            subnet->getSharedNetwork(network);
+            if (network &&
+                (network->getIface(Network4::Inheritance::NONE) == iface)) {
+                subnet_selected = subnet;
             }
         }
 
         if (subnet_selected) {
-
             // If a subnet meets the client class criteria return it.
             if (subnet_selected->clientSupported(client_classes)) {
                 LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE,
                           DHCPSRV_CFGMGR_SUBNET4_IFACE)
-                    .arg((*subnet)->toText())
+                    .arg(subnet->toText())
                     .arg(iface);
                 return (subnet_selected);
             }
         }
     }
 
+    LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE,
+              DHCPSRV_SUBNET4_SELECT_BY_INTERFACE_NO_MATCH)
+        .arg(iface);
+
     // Failed to find a subnet.
     return (Subnet4Ptr());
 }
 
 Subnet4Ptr
 CfgSubnets4::getSubnet(const SubnetID id) const {
-
     /// @todo: Once this code is migrated to multi-index container, use
     /// an index rather than full scan.
-    for (auto subnet = subnets_.begin(); subnet != subnets_.end(); ++subnet) {
-        if ((*subnet)->getID() == id) {
-            return (*subnet);
+    for (auto const& subnet : subnets_) {
+        if (subnet->getID() == id) {
+            return (subnet);
         }
     }
     return (Subnet4Ptr());
@@ -447,24 +462,27 @@ CfgSubnets4::getSubnet(const SubnetID id) const {
 
 Subnet4Ptr
 CfgSubnets4::selectSubnet(const IOAddress& address,
-                 const ClientClasses& client_classes) const {
-    for (Subnet4Collection::const_iterator subnet = subnets_.begin();
-         subnet != subnets_.end(); ++subnet) {
+                          const ClientClasses& client_classes) const {
+    for (auto const& subnet : subnets_) {
 
         // Address is in range for the subnet prefix, so return it.
-        if (!(*subnet)->inRange(address)) {
+        if (!subnet->inRange(address)) {
             continue;
         }
 
         // If a subnet meets the client class criteria return it.
-        if ((*subnet)->clientSupported(client_classes)) {
+        if (subnet->clientSupported(client_classes)) {
             LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE, DHCPSRV_CFGMGR_SUBNET4_ADDR)
-                .arg((*subnet)->toText())
+                .arg(subnet->toText())
                 .arg(address.toText());
-            return (*subnet);
+            return (subnet);
         }
     }
 
+    LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE,
+              DHCPSRV_SUBNET4_SELECT_BY_ADDRESS_NO_MATCH)
+        .arg(address.toText());
+
     // Failed to find a subnet.
     return (Subnet4Ptr());
 }
@@ -475,9 +493,8 @@ CfgSubnets4::removeStatistics() {
 
     // For each v4 subnet currently configured, remove the statistic.
     StatsMgr& stats_mgr = StatsMgr::instance();
-    for (Subnet4Collection::const_iterator subnet4 = subnets_.begin();
-         subnet4 != subnets_.end(); ++subnet4) {
-        SubnetID subnet_id = (*subnet4)->getID();
+    for (auto const& subnet4 : subnets_) {
+        SubnetID subnet_id = subnet4->getID();
         stats_mgr.del(StatsMgr::generateName("subnet", subnet_id,
                                              "total-addresses"));
 
@@ -503,20 +520,23 @@ CfgSubnets4::updateStatistics() {
     using namespace isc::stats;
 
     StatsMgr& stats_mgr = StatsMgr::instance();
-    for (Subnet4Collection::const_iterator subnet4 = subnets_.begin();
-         subnet4 != subnets_.end(); ++subnet4) {
-        SubnetID subnet_id = (*subnet4)->getID();
+    for (auto const& subnet4 : subnets_) {
+        SubnetID subnet_id = subnet4->getID();
 
         stats_mgr.setValue(StatsMgr::
                            generateName("subnet", subnet_id, "total-addresses"),
                                         static_cast<int64_t>
-                                        ((*subnet4)->getPoolCapacity(Lease::
-                                                                     TYPE_V4)));
-        const std::string& name =
+                                        (subnet4->getPoolCapacity(Lease::TYPE_V4)));
+        std::string name =
             StatsMgr::generateName("subnet", subnet_id, "cumulative-assigned-addresses");
         if (!stats_mgr.getObservation(name)) {
             stats_mgr.setValue(name, static_cast<int64_t>(0));
         }
+
+        name = StatsMgr::generateName("subnet", subnet_id, "v4-reservation-conflicts");
+        if (!stats_mgr.getObservation(name)) {
+            stats_mgr.setValue(name, static_cast<int64_t>(0));
+        }
     }
 
     // Only recount the stats if we have subnets.
@@ -529,9 +549,8 @@ ElementPtr
 CfgSubnets4::toElement() const {
     ElementPtr result = Element::createList();
     // Iterate subnets
-    for (Subnet4Collection::const_iterator subnet = subnets_.cbegin();
-         subnet != subnets_.cend(); ++subnet) {
-        result->add((*subnet)->toElement());
+    for (auto const& subnet : subnets_) {
+        result->add(subnet->toElement());
     }
     return (result);
 }
