@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -8,9 +8,14 @@
 #define CFG_SUBNETS4_H
 
 #include <asiolink/io_address.h>
+#include <cc/cfg_to_element.h>
+#include <dhcp/pkt4.h>
+#include <dhcpsrv/cfg_shared_networks.h>
 #include <dhcpsrv/subnet.h>
+#include <dhcpsrv/subnet_id.h>
 #include <dhcpsrv/subnet_selector.h>
 #include <boost/shared_ptr.hpp>
+#include <string>
 
 namespace isc {
 namespace dhcp {
@@ -25,7 +30,7 @@ namespace dhcp {
 ///
 /// See @c CfgSubnets4::selectSubnet documentation for more details on how the
 /// subnet is selected for the client.
-class CfgSubnets4 {
+class CfgSubnets4 : public isc::data::CfgToElement {
 public:
 
     /// @brief Adds new subnet to the configuration.
@@ -36,6 +41,74 @@ public:
     /// duplicates id of an existing subnet.
     void add(const Subnet4Ptr& subnet);
 
+    /// @brief Replaces subnet in the configuration.
+    ///
+    /// This method replaces a subnet by another subnet with the same ID.
+    /// The prefix should be the same too.
+    ///
+    /// @param subnet Pointer to the subnet being updated.
+    /// @throw BadValue if the subnet to update does not exit.
+    /// @return Pointer to the replaced subnet or NULL if it failed.
+    Subnet4Ptr replace(const Subnet4Ptr& subnet);
+
+    /// @brief Removes subnet from the configuration.
+    ///
+    /// @param subnet Pointer to the subnet to be removed.
+    ///
+    /// @throw isc::BadValue if such subnet doesn't exist.
+    void del(const ConstSubnet4Ptr& subnet);
+
+    /// @brief Removes subnet from the configuration.
+    ///
+    /// @param subnet_id Identifier of the subnet to be removed.
+    ///
+    /// @throw isc::BadValue if such subnet doesn't exist.
+    void del(const SubnetID& subnet_id);
+
+    /// @brief Merges specified subnet configuration into this configuration.
+    ///
+    /// This method merges subnets from the @c other configuration into this
+    /// configuration. The general rule is that existing subnets are replaced
+    /// by the subnets from @c other. If there is no corresponding subnet in
+    /// this configuration the subnet from @c other configuration is inserted.
+    ///
+    /// The complexity of the merge process stems from the associations between
+    /// the subnets and shared networks.  It is assumed that subnets in @c other
+    /// are the authority on their shared network assignments. It is also
+    /// assumed that @ networks is the list of shared networks that should be
+    /// used in making assignments.  The general concept is that the overarching
+    /// merge process will first merge shared networks and then pass that list
+    /// of networks into this method. Subnets from @c other are then merged
+    /// into this configuration as follows:
+    ///
+    /// For each subnet in @c other:
+    ///
+    /// - If a subnet of the same ID already exists in this configuration:
+    ///    -# If it belongs to a shared network, remove it from that network
+    ///    -# Remove the subnet from this configuration and discard it
+    ///
+    /// - Create the subnet's option instance, as well as any options
+    ///   that belong to any of the subnet's pools.
+    /// - Add the subnet from @c other to this configuration.
+    /// - If that subnet is associated to shared network, find that network
+    ///   in @ networks and add that subnet to it.
+    ///
+    /// @warning The merge operation affects the @c other configuration.
+    /// Therefore, the caller must not rely on the data held in the @c other
+    /// object after the call to @c merge. Also, the data held in @c other must
+    /// not be modified after the call to @c merge because it may affect the
+    /// merged configuration.
+    ///
+    /// @param cfg_def set of of user-defined option definitions to use
+    /// when creating option instances.
+    /// @param networks collection of shared networks that to which assignments
+    /// should be added. In other words, the list of shared networks that belong
+    /// to the same SrvConfig instance we are merging into.
+    /// @param other the subnet configuration to be merged into this
+    /// configuration.
+    void merge(CfgOptionDefPtr cfg_def, CfgSharedNetworks4Ptr networks,
+               CfgSubnets4& other);
+
     /// @brief Returns pointer to the collection of all IPv4 subnets.
     ///
     /// This is used in a hook (subnet4_select), where the hook is able
@@ -47,6 +120,56 @@ public:
         return (&subnets_);
     }
 
+    /// @brief Returns const pointer to a subnet identified by the specified
+    /// subnet identifier.
+    ///
+    /// The const pointer is returned by this method to prevent a caller from
+    /// modifying the subnet configuration. Modifications to subnet configuration
+    /// is dangerous and must be done carefully. The subnets' configuration is
+    /// held in the multi index container and any modifications to the subnet
+    /// id or subnet prefix must trigger re-indexing of multi index container.
+    /// There is no possibility to enforce this when the non-const pointer is
+    /// returned.
+    ///
+    /// @param subnet_id Subnet identifier.
+    ///
+    /// @return Pointer to the @c Subnet4 object or null pointer if such
+    /// subnet doesn't exist.
+    ConstSubnet4Ptr getBySubnetId(const SubnetID& subnet_id) const;
+
+    /// @brief Returns const pointer to a subnet which matches the specified
+    /// prefix in the canonical form.
+    ///
+    /// The const pointer is returned by this method to prevent a caller from
+    /// modifying the subnet configuration. Modifications to subnet configuration
+    /// is dangerous and must be done carefully. The subnets' configuration is
+    /// held in the multi index container and any modifications to the subnet
+    /// id or subnet prefix must trigger re-indexing of multi index container.
+    /// There is no possibility to enforce this when the non-const pointer is
+    /// returned.
+    ///
+    /// @param subnet_prefix Subnet prefix, e.g. 10.2.3.0/24
+    ///
+    /// @return Pointer to the @c Subnet4 object or null pointer if such
+    /// subnet doesn't exist.
+    ConstSubnet4Ptr getByPrefix(const std::string& subnet_prefix) const;
+
+    /// @brief Checks if specified server identifier has been specified for
+    /// any subnet.
+    ///
+    /// @param server_id Server identifier.
+    ///
+    /// @return true if there is a subnet with a specified server identifier.
+    bool hasSubnetWithServerId(const asiolink::IOAddress& server_id) const;
+
+    /// @brief Build selector from a client's message.
+    ///
+    /// @note: code moved from server.
+    ///
+    /// @param query client's message.
+    /// @return filled selector.
+    static SubnetSelector initSelector(const Pkt4Ptr& query);
+
     /// @brief Returns a pointer to the selected subnet.
     ///
     /// This method tries to retrieve the subnet for the client using various
@@ -81,7 +204,7 @@ public:
     ///
     /// @todo This method requires performance improvement! It currently
     /// iterates over all existing subnets (possibly a couple of times)
-    /// to find the one which fulfils the search criteria. The subnet storage
+    /// to find the one which fulfills the search criteria. The subnet storage
     /// is implemented as a simple STL vector which precludes fast searches
     /// using specific keys. Hence, full scan is required. To improve the
     /// search performance a different container type is required, e.g.
@@ -96,6 +219,15 @@ public:
     /// or they are insufficient to select a subnet.
     Subnet4Ptr selectSubnet(const SubnetSelector& selector) const;
 
+    /// @brief Returns subnet with specified subnet-id value
+    ///
+    /// Warning: this method uses full scan. Its use is not recommended for
+    /// packet processing.
+    /// Please use @ref getBySubnetId instead when possible.
+    ///
+    /// @return Subnet (or NULL)
+    Subnet4Ptr getSubnet(const SubnetID id) const;
+
     /// @brief Returns a pointer to a subnet if provided address is in its range.
     ///
     /// This method returns a pointer to the subnet if the address passed in
@@ -104,7 +236,7 @@ public:
     /// @c selectSubnet(SubnetSelector).
     ///
     /// @todo This method requires performance improvement! It currently
-    /// iterates over all existing subnets to find the one which fulfils
+    /// iterates over all existing subnets to find the one which fulfills
     /// the search criteria. The subnet storage is implemented as a simple
     /// STL vector which precludes fast searches using specific keys.
     /// Hence, full scan is required. To improve the search performance a
@@ -129,7 +261,7 @@ public:
     /// @c selectSubnet(SubnetSelector).
     ///
     /// @todo This method requires performance improvement! It currently
-    /// iterates over all existing subnets to find the one which fulfils
+    /// iterates over all existing subnets to find the one which fulfills
     /// the search criteria. The subnet storage is implemented as a simple
     /// STL vector which precludes fast searches using specific keys.
     /// Hence, full scan is required. To improve the search performance a
@@ -157,7 +289,7 @@ public:
     ///   received over.
     ///
     /// @todo: Add additional selection criteria. See
-    ///  http://kea.isc.org/wiki/ISC-DHCP4o6-Design for details.
+    ///  https://gitlab.isc.org/isc-projects/kea/wikis/designs/dhcpv4o6-design for details.
     ///
     /// @param selector Const reference to the selector structure which holds
     /// various information extracted from the client's packet which are used
@@ -171,7 +303,7 @@ public:
     /// This method updates statistics that are affected by the newly committed
     /// configuration. In particular, it updates the number of available addresses
     /// in each subnet. Other statistics may be added in the future. In general,
-    /// these are statistics that are dependant only on configuration, so they are
+    /// these are statistics that are dependent only on configuration, so they are
     /// not expected to change until the next reconfiguration event.
     void updateStatistics();
 
@@ -183,15 +315,12 @@ public:
     /// configuration and also subnet-ids may change.
     void removeStatistics();
 
-private:
+    /// @brief Unparse a configuration object
+    ///
+    /// @return a pointer to unparsed configuration
+    virtual isc::data::ElementPtr toElement() const;
 
-    /// @brief Checks that the IPv4 subnet with the given id already exists.
-    ///
-    /// @param subnet Subnet for which this function will check if the other
-    /// subnet with equal id already exists.
-    ///
-    /// @return true if the duplicate subnet exists.
-    bool isDuplicate(const Subnet4& subnet) const;
+private:
 
     /// @brief A container for IPv4 subnets.
     Subnet4Collection subnets_;
