@@ -49,45 +49,6 @@ extern const isc::log::MessageID DHCPSRV_CFGMGR_UPDATE_SUBNET6;
 extern const isc::log::MessageID DHCPSRV_CFGMGR_USE_ADDRESS;
 extern const isc::log::MessageID DHCPSRV_CFGMGR_USE_UNICAST;
 extern const isc::log::MessageID DHCPSRV_CLOSE_DB;
-extern const isc::log::MessageID DHCPSRV_CQL_ADD_ADDR4;
-extern const isc::log::MessageID DHCPSRV_CQL_ADD_ADDR6;
-extern const isc::log::MessageID DHCPSRV_CQL_COMMIT;
-extern const isc::log::MessageID DHCPSRV_CQL_CONNECTION_BEGIN_TRANSACTION;
-extern const isc::log::MessageID DHCPSRV_CQL_CONNECTION_COMMIT;
-extern const isc::log::MessageID DHCPSRV_CQL_CONNECTION_ROLLBACK;
-extern const isc::log::MessageID DHCPSRV_CQL_DB;
-extern const isc::log::MessageID DHCPSRV_CQL_DEALLOC_ERROR;
-extern const isc::log::MessageID DHCPSRV_CQL_DELETE_ADDR;
-extern const isc::log::MessageID DHCPSRV_CQL_DELETE_EXPIRED_RECLAIMED4;
-extern const isc::log::MessageID DHCPSRV_CQL_DELETE_EXPIRED_RECLAIMED6;
-extern const isc::log::MessageID DHCPSRV_CQL_GET4;
-extern const isc::log::MessageID DHCPSRV_CQL_GET_ADDR4;
-extern const isc::log::MessageID DHCPSRV_CQL_GET_ADDR6;
-extern const isc::log::MessageID DHCPSRV_CQL_GET_CLIENTID;
-extern const isc::log::MessageID DHCPSRV_CQL_GET_CLIENTID_HWADDR_SUBID;
-extern const isc::log::MessageID DHCPSRV_CQL_GET_EXPIRED4;
-extern const isc::log::MessageID DHCPSRV_CQL_GET_EXPIRED6;
-extern const isc::log::MessageID DHCPSRV_CQL_GET_HOSTNAME4;
-extern const isc::log::MessageID DHCPSRV_CQL_GET_HOSTNAME6;
-extern const isc::log::MessageID DHCPSRV_CQL_GET_HWADDR;
-extern const isc::log::MessageID DHCPSRV_CQL_GET_IAID_DUID;
-extern const isc::log::MessageID DHCPSRV_CQL_GET_IAID_SUBID_DUID;
-extern const isc::log::MessageID DHCPSRV_CQL_GET_PAGE4;
-extern const isc::log::MessageID DHCPSRV_CQL_GET_PAGE6;
-extern const isc::log::MessageID DHCPSRV_CQL_GET_SUBID4;
-extern const isc::log::MessageID DHCPSRV_CQL_GET_SUBID_CLIENTID;
-extern const isc::log::MessageID DHCPSRV_CQL_GET_SUBID_HWADDR;
-extern const isc::log::MessageID DHCPSRV_CQL_GET_VERSION;
-extern const isc::log::MessageID DHCPSRV_CQL_HOST_ADD;
-extern const isc::log::MessageID DHCPSRV_CQL_HOST_DB;
-extern const isc::log::MessageID DHCPSRV_CQL_HOST_DB_GET_VERSION;
-extern const isc::log::MessageID DHCPSRV_CQL_HOST_GET4;
-extern const isc::log::MessageID DHCPSRV_CQL_HOST_GET6;
-extern const isc::log::MessageID DHCPSRV_CQL_HOST_GET_ALL;
-extern const isc::log::MessageID DHCPSRV_CQL_LEASE_EXCEPTION_THROWN;
-extern const isc::log::MessageID DHCPSRV_CQL_ROLLBACK;
-extern const isc::log::MessageID DHCPSRV_CQL_UPDATE_ADDR4;
-extern const isc::log::MessageID DHCPSRV_CQL_UPDATE_ADDR6;
 extern const isc::log::MessageID DHCPSRV_DEPRECATED;
 extern const isc::log::MessageID DHCPSRV_DHCP4O6_RECEIVED_BAD_PACKET;
 extern const isc::log::MessageID DHCPSRV_DHCP_DDNS_ERROR_EXCEPTION;
@@ -199,8 +160,10 @@ extern const isc::log::MessageID DHCPSRV_MYSQL_LEASE_DB_RECONNECT_ATTEMPT_FAILED
 extern const isc::log::MessageID DHCPSRV_MYSQL_LEASE_DB_RECONNECT_ATTEMPT_SCHEDULE;
 extern const isc::log::MessageID DHCPSRV_MYSQL_LEASE_DB_RECONNECT_FAILED;
 extern const isc::log::MessageID DHCPSRV_MYSQL_NEGATIVE_LEASES_STAT;
+extern const isc::log::MessageID DHCPSRV_MYSQL_NO_TLS;
 extern const isc::log::MessageID DHCPSRV_MYSQL_ROLLBACK;
 extern const isc::log::MessageID DHCPSRV_MYSQL_START_TRANSACTION;
+extern const isc::log::MessageID DHCPSRV_MYSQL_TLS_CIPHER;
 extern const isc::log::MessageID DHCPSRV_MYSQL_UPDATE_ADDR4;
 extern const isc::log::MessageID DHCPSRV_MYSQL_UPDATE_ADDR6;
 extern const isc::log::MessageID DHCPSRV_NOTYPE_DB;
@@ -246,13 +209,25 @@ extern const isc::log::MessageID DHCPSRV_PGSQL_LEASE_DB_RECONNECT_ATTEMPT_FAILED
 extern const isc::log::MessageID DHCPSRV_PGSQL_LEASE_DB_RECONNECT_ATTEMPT_SCHEDULE;
 extern const isc::log::MessageID DHCPSRV_PGSQL_LEASE_DB_RECONNECT_FAILED;
 extern const isc::log::MessageID DHCPSRV_PGSQL_NEGATIVE_LEASES_STAT;
+extern const isc::log::MessageID DHCPSRV_PGSQL_NO_TLS_SUPPORT;
 extern const isc::log::MessageID DHCPSRV_PGSQL_ROLLBACK;
 extern const isc::log::MessageID DHCPSRV_PGSQL_START_TRANSACTION;
+extern const isc::log::MessageID DHCPSRV_PGSQL_TLS_SUPPORT;
 extern const isc::log::MessageID DHCPSRV_PGSQL_UPDATE_ADDR4;
 extern const isc::log::MessageID DHCPSRV_PGSQL_UPDATE_ADDR6;
 extern const isc::log::MessageID DHCPSRV_QUEUE_NCR;
 extern const isc::log::MessageID DHCPSRV_QUEUE_NCR_FAILED;
 extern const isc::log::MessageID DHCPSRV_QUEUE_NCR_SKIP;
+extern const isc::log::MessageID DHCPSRV_SUBNET4O6_SELECT_FAILED;
+extern const isc::log::MessageID DHCPSRV_SUBNET4_SELECT_BY_ADDRESS_NO_MATCH;
+extern const isc::log::MessageID DHCPSRV_SUBNET4_SELECT_BY_INTERFACE_NO_MATCH;
+extern const isc::log::MessageID DHCPSRV_SUBNET4_SELECT_BY_RELAY_ADDRESS_NO_MATCH;
+extern const isc::log::MessageID DHCPSRV_SUBNET4_SELECT_NO_RAI_OPTIONS;
+extern const isc::log::MessageID DHCPSRV_SUBNET4_SELECT_NO_RELAY_ADDRESS;
+extern const isc::log::MessageID DHCPSRV_SUBNET4_SELECT_NO_USABLE_ADDRESS;
+extern const isc::log::MessageID DHCPSRV_SUBNET6_SELECT_BY_ADDRESS_NO_MATCH;
+extern const isc::log::MessageID DHCPSRV_SUBNET6_SELECT_BY_INTERFACE_ID_NO_MATCH;
+extern const isc::log::MessageID DHCPSRV_SUBNET6_SELECT_BY_INTERFACE_NO_MATCH;
 extern const isc::log::MessageID DHCPSRV_TIMERMGR_CALLBACK_FAILED;
 extern const isc::log::MessageID DHCPSRV_TIMERMGR_REGISTER_TIMER;
 extern const isc::log::MessageID DHCPSRV_TIMERMGR_RUN_TIMER_OPERATION;
