@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -14,8 +14,10 @@
 #include <dhcpsrv/memfile_lease_mgr.h>
 #include <dhcpsrv/timer_mgr.h>
 #include <exceptions/exceptions.h>
+#include <stats/stats_mgr.h>
 #include <util/multi_threading_mgr.h>
 #include <util/pid_file.h>
+
 #include <cstdio>
 #include <cstring>
 #include <errno.h>
@@ -37,8 +39,10 @@ const char* KEA_LFC_EXECUTABLE_ENV_NAME = "KEA_LFC_EXECUTABLE";
 }  // namespace
 
 using namespace isc::asiolink;
+using namespace isc::data;
 using namespace isc::db;
 using namespace isc::util;
+using namespace isc::stats;
 
 namespace isc {
 namespace dhcp {
@@ -625,8 +629,10 @@ private:
 
 // Explicit definition of class static constants.  Values are given in the
 // declaration so they're not needed here.
-const int Memfile_LeaseMgr::MAJOR_VERSION;
-const int Memfile_LeaseMgr::MINOR_VERSION;
+const int Memfile_LeaseMgr::MAJOR_VERSION_V4;
+const int Memfile_LeaseMgr::MINOR_VERSION_V4;
+const int Memfile_LeaseMgr::MAJOR_VERSION_V6;
+const int Memfile_LeaseMgr::MINOR_VERSION_V6;
 
 Memfile_LeaseMgr::Memfile_LeaseMgr(const DatabaseConnection::ParameterMap& parameters)
     : LeaseMgr(), lfc_setup_(), conn_(parameters), mutex_(new std::mutex) {
@@ -660,8 +666,9 @@ Memfile_LeaseMgr::Memfile_LeaseMgr(const DatabaseConnection::ParameterMap& param
         LOG_WARN(dhcpsrv_logger, DHCPSRV_MEMFILE_NO_STORAGE);
     } else  {
         if (conversion_needed) {
+            auto const& version(getVersion());
             LOG_WARN(dhcpsrv_logger, DHCPSRV_MEMFILE_CONVERTING_LEASE_FILES)
-                    .arg(MAJOR_VERSION).arg(MINOR_VERSION);
+                    .arg(version.first).arg(version.second);
         }
         lfcSetup(conversion_needed);
     }
@@ -679,11 +686,15 @@ Memfile_LeaseMgr::~Memfile_LeaseMgr() {
 }
 
 std::string
-Memfile_LeaseMgr::getDBVersion() {
+Memfile_LeaseMgr::getDBVersion(Universe const& u) {
     std::stringstream tmp;
-    tmp << "Memfile backend " << MAJOR_VERSION;
-    tmp << "." << MINOR_VERSION;
-    return (tmp.str());
+    tmp << "Memfile backend ";
+    if (u == V4) {
+        tmp << MAJOR_VERSION_V4 << "." << MINOR_VERSION_V4;
+    } else if (u == V6) {
+        tmp << MAJOR_VERSION_V6 << "." << MINOR_VERSION_V6;
+    }
+    return tmp.str();
 }
 
 bool
@@ -706,6 +717,9 @@ Memfile_LeaseMgr::addLeaseInternal(const Lease4Ptr& lease) {
     // of the Lease up to the point of insertion in the database).
     lease->updateCurrentExpirationTime();
 
+    // Increment class lease counters.
+    class_lease_counter_.addLease(lease);
+
     return (true);
 }
 
@@ -742,6 +756,9 @@ Memfile_LeaseMgr::addLeaseInternal(const Lease6Ptr& lease) {
     // of the Lease up to the point of insertion in the database).
     lease->updateCurrentExpirationTime();
 
+    // Increment class lease counters.
+    class_lease_counter_.addLease(lease);
+
     return (true);
 }
 
@@ -1403,8 +1420,15 @@ Memfile_LeaseMgr::updateLease4Internal(const Lease4Ptr& lease) {
     // Update lease current expiration time.
     lease->updateCurrentExpirationTime();
 
+    // Save a copy of the old lease as lease_it will point to the new
+    // one after the replacement.
+    Lease4Ptr old_lease = *lease_it;
+
     // Use replace() to re-index leases.
     index.replace(lease_it, Lease4Ptr(new Lease4(*lease)));
+
+    // Adjust class lease counters.
+    class_lease_counter_.updateLease(lease, old_lease);
 }
 
 void
@@ -1450,8 +1474,15 @@ Memfile_LeaseMgr::updateLease6Internal(const Lease6Ptr& lease) {
     // Update lease current expiration time.
     lease->updateCurrentExpirationTime();
 
+    // Save a copy of the old lease as lease_it will point to the new
+    // one after the replacement.
+    Lease6Ptr old_lease = *lease_it;
+
     // Use replace() to re-index leases.
     index.replace(lease_it, Lease6Ptr(new Lease6(*lease)));
+
+    // Adjust class lease counters.
+    class_lease_counter_.updateLease(lease, old_lease);
 }
 
 void
@@ -1491,7 +1522,12 @@ Memfile_LeaseMgr::deleteLeaseInternal(const Lease4Ptr& lease) {
                 return false;
             }
         }
+
         storage4_.erase(l);
+
+        // Decrement class lease counters.
+        class_lease_counter_.removeLease(lease);
+
         return (true);
     }
 }
@@ -1533,7 +1569,12 @@ Memfile_LeaseMgr::deleteLeaseInternal(const Lease6Ptr& lease) {
                 return false;
             }
         }
+
         storage6_.erase(l);
+
+        // Decrement class lease counters.
+        class_lease_counter_.removeLease(lease);
+
         return (true);
     }
 }
@@ -1655,6 +1696,17 @@ Memfile_LeaseMgr::getDescription() const {
     return (std::string("In memory database with leases stored in a CSV file."));
 }
 
+std::pair<uint32_t, uint32_t>
+Memfile_LeaseMgr::getVersion() const {
+    std::string const& universe(conn_.getParameter("universe"));
+    if (universe == "4") {
+        return std::make_pair(MAJOR_VERSION_V4, MINOR_VERSION_V4);
+    } else if (universe == "6") {
+        return std::make_pair(MAJOR_VERSION_V6, MINOR_VERSION_V6);
+    }
+    isc_throw(BadValue, "cannot determine version for universe " << universe);
+}
+
 void
 Memfile_LeaseMgr::commit() {
     LOG_DEBUG(dhcpsrv_logger, DHCPSRV_DBG_TRACE_DETAIL, DHCPSRV_MEMFILE_COMMIT);
@@ -2039,6 +2091,267 @@ Memfile_LeaseMgr::wipeLeases6(const SubnetID& subnet_id) {
     return (num);
 }
 
+void
+Memfile_LeaseMgr::recountClassLeases4() {
+    class_lease_counter_.clear();
+    for (auto lease = storage4_.begin(); lease != storage4_.end(); ++lease) {
+        // Bump the appropriate accumulator
+        if ((*lease)->state_ == Lease::STATE_DEFAULT) {
+            class_lease_counter_.addLease(*lease);
+        }
+    }
+}
+
+void
+Memfile_LeaseMgr::recountClassLeases6() {
+    class_lease_counter_.clear();
+    for (auto lease = storage6_.begin(); lease != storage6_.end(); ++lease) {
+        // Bump the appropriate accumulator
+        if ((*lease)->state_ == Lease::STATE_DEFAULT) {
+            class_lease_counter_.addLease(*lease);
+        }
+    }
+}
+
+size_t
+Memfile_LeaseMgr::getClassLeaseCount(const ClientClass& client_class,
+                                     const Lease::Type& ltype /* = Lease::TYPE_V4*/) const {
+    if (MultiThreadingMgr::instance().getMode()) {
+        std::lock_guard<std::mutex> lock(*mutex_);
+        return(class_lease_counter_.getClassCount(client_class, ltype));
+    } else {
+        return(class_lease_counter_.getClassCount(client_class, ltype));
+    }
+}
+
+void
+Memfile_LeaseMgr::clearClassLeaseCounts() {
+    return(class_lease_counter_.clear());
+}
+
+std::string
+Memfile_LeaseMgr::checkLimits4(isc::data::ConstElementPtr const& user_context) const {
+    if (!user_context) {
+        return ("");
+    }
+
+    ConstElementPtr limits = user_context->find("ISC/limits");
+    if (!limits) {
+        return ("");
+    }
+
+    // Iterate of the 'client-classes' list in 'limits'. For each class that specifies
+    // an "address-limit", check its value against the class's lease count.
+    ConstElementPtr classes = limits->get("client-classes");
+    if (classes) {
+        for (int i = 0; i < classes->size(); ++i) {
+            ConstElementPtr class_elem = classes->get(i);
+            // Get class name.
+            ConstElementPtr name_elem = class_elem->get("name");
+            if (!name_elem) {
+                isc_throw(BadValue, "checkLimits4 - client-class.name is missing: "
+                          << prettyPrint(limits));
+            }
+
+            std::string name = name_elem->stringValue();
+
+            // Now look for an address-limit
+            size_t limit;
+            if (!getLeaseLimit(class_elem, Lease::TYPE_V4, limit)) {
+                // No limit, go to the next class.
+                continue;
+            }
+
+            // If the limit is > 0 look up the class lease count.  Limit of 0 always
+            // denies the lease.
+            size_t lease_count = 0;
+            if (limit) {
+                lease_count = getClassLeaseCount(name);
+            }
+
+            // If we're over the limit, return the error, no need to evaluate any others.
+            if (lease_count >= limit) {
+                std::ostringstream ss;
+                ss << "address limit " << limit << " for client class \""
+                   << name << "\", current lease count " << lease_count;
+                return (ss.str());
+            }
+        }
+    }
+
+    // If there were class limits we passed them, now look for a subnet limit.
+    ConstElementPtr subnet_elem = limits->get("subnet");
+    if (subnet_elem) {
+        // Get the subnet id.
+        ConstElementPtr id_elem = subnet_elem->get("id");
+        if (!id_elem) {
+            isc_throw(BadValue, "checkLimits4 - subnet.id is missing: "
+                      << prettyPrint(limits));
+        }
+
+        SubnetID subnet_id = id_elem->intValue();
+
+        // Now look for an address-limit.
+        size_t limit;
+        if (getLeaseLimit(subnet_elem, Lease::TYPE_V4, limit)) {
+            // If the limit is > 0 look up the subnet lease count. Limit of 0 always
+            // denies the lease.
+            int64_t lease_count = 0;
+            if (limit) {
+                lease_count = getSubnetStat(subnet_id, "assigned-addresses");
+            }
+
+            // If we're over the limit, return the error.
+            if (lease_count >= limit) {
+                std::ostringstream ss;
+                ss << "address limit " << limit << " for subnet ID " << subnet_id
+                   << ", current lease count " << lease_count;
+                return (ss.str());
+            }
+        }
+    }
+
+    // No limits exceeded!
+    return ("");
+}
+
+std::string
+Memfile_LeaseMgr::checkLimits6(isc::data::ConstElementPtr const& user_context) const {
+    if (!user_context) {
+        return ("");
+    }
+
+    ConstElementPtr limits = user_context->find("ISC/limits");
+    if (!limits) {
+        return ("");
+    }
+
+    // Iterate over the 'client-classes' list in 'limits'. For each class that specifies
+    // limit (either "address-limit" or "prefix-limit", check its value against the appropriate
+    // class lease count.
+    ConstElementPtr classes = limits->get("client-classes");
+    if (classes) {
+        for (int i = 0; i < classes->size(); ++i) {
+            ConstElementPtr class_elem = classes->get(i);
+            // Get class name.
+            ConstElementPtr name_elem = class_elem->get("name");
+            if (!name_elem) {
+                isc_throw(BadValue, "checkLimits6 - client-class.name is missing: "
+                          << prettyPrint(limits));
+            }
+
+            std::string name = name_elem->stringValue();
+
+            // Now look for either address-limit or a prefix=limit.
+            size_t limit = 0;
+            Lease::Type ltype = Lease::TYPE_NA;
+            if (!getLeaseLimit(class_elem, ltype, limit)) {
+                ltype = Lease::TYPE_PD;
+                if (!getLeaseLimit(class_elem, ltype, limit)) {
+                    // No limits for this class, skip to the next.
+                    continue;
+                }
+            }
+
+            // If the limit is > 0 look up the class lease count.  Limit of 0 always
+            // denies the lease.
+            size_t lease_count = 0;
+            if (limit) {
+                lease_count = getClassLeaseCount(name, ltype);
+            }
+
+            // If we're over the limit, return the error, no need to evaluate any others.
+            if (lease_count >= limit) {
+                std::ostringstream ss;
+                ss << (ltype == Lease::TYPE_NA ? "address" : "prefix")
+                   << " limit " << limit << " for client class \""
+                   << name << "\", current lease count " << lease_count;
+                return (ss.str());
+            }
+        }
+    }
+
+    // If there were class limits we passed them, now look for a subnet limit.
+    ConstElementPtr subnet_elem = limits->get("subnet");
+    if (subnet_elem) {
+        // Get the subnet id.
+        ConstElementPtr id_elem = subnet_elem->get("id");
+        if (!id_elem) {
+            isc_throw(BadValue, "checkLimits6 - subnet.id is missing: "
+                      << prettyPrint(limits));
+        }
+
+        SubnetID subnet_id = id_elem->intValue();
+
+        // Now look for either address-limit or a prefix=limit.
+        size_t limit = 0;
+        Lease::Type ltype = Lease::TYPE_NA;
+        if (!getLeaseLimit(subnet_elem, ltype, limit)) {
+            ltype = Lease::TYPE_PD;
+            if (!getLeaseLimit(subnet_elem, ltype, limit)) {
+                // No limits for the subnet so none exceeded!
+                return ("");
+            }
+        }
+
+        // If the limit is > 0 look up the class lease count.  Limit of 0 always
+        // denies the lease.
+        int64_t lease_count = 0;
+        if (limit) {
+            lease_count = getSubnetStat(subnet_id, (ltype == Lease::TYPE_NA ?
+                                                    "assigned-nas" : "assigned-pds"));
+        }
+
+        // If we're over the limit, return the error.
+        if (lease_count >= limit) {
+            std::ostringstream ss;
+            ss << (ltype == Lease::TYPE_NA ? "address" : "prefix")
+               << " limit " << limit << " for subnet ID " << subnet_id
+               << ", current lease count " << lease_count;
+            return (ss.str());
+        }
+    }
+
+    // No limits exceeded!
+    return ("");
+}
+
+bool
+Memfile_LeaseMgr::isJsonSupported() const {
+    return true;
+}
+
+int64_t
+Memfile_LeaseMgr::getSubnetStat(const SubnetID& subnet_id, const std::string& stat_label) const {
+    /// @todo This could be simplified if StatsMgr provided a mechanism to
+    /// return the most recent sample as an InterSample.
+    std::string stat_name = StatsMgr::generateName("subnet", subnet_id, stat_label);
+    ConstElementPtr stat = StatsMgr::instance().get(stat_name);
+    ConstElementPtr samples = stat->get(stat_name);
+    if (samples && samples->size()) {
+        auto sample = samples->get(0);
+        if (sample->size()) {
+            auto count_elem = sample->get(0);
+            return (count_elem->intValue());
+        }
+    }
+
+    return (0);
+}
+
+bool
+Memfile_LeaseMgr::getLeaseLimit(ConstElementPtr parent, Lease::Type ltype, size_t& limit) const {
+    ConstElementPtr limit_elem = parent->get(ltype == Lease::TYPE_PD ?
+                                             "prefix-limit" : "address-limit");
+    if (limit_elem) {
+        limit = limit_elem->intValue();
+        return (true);
+    }
+
+    return (false);
+}
+
+
+
 }  // namespace dhcp
 }  // namespace isc
-
