@@ -1,4 +1,4 @@
-// Copyright (C) 2016-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -36,68 +36,70 @@ namespace dhcp {
 /// list and map types for entries.
 /// Order follows global_param rule in bison grammar.
 const SimpleKeywords SimpleParser6::GLOBAL6_PARAMETERS = {
-    { "data-directory",                 Element::string },
-    { "preferred-lifetime",             Element::integer },
-    { "min-preferred-lifetime",         Element::integer },
-    { "max-preferred-lifetime",         Element::integer },
-    { "valid-lifetime",                 Element::integer },
-    { "min-valid-lifetime",             Element::integer },
-    { "max-valid-lifetime",             Element::integer },
-    { "renew-timer",                    Element::integer },
-    { "rebind-timer",                   Element::integer },
-    { "decline-probation-period",       Element::integer },
-    { "subnet6",                        Element::list },
-    { "shared-networks",                Element::list },
-    { "interfaces-config",              Element::map },
-    { "lease-database",                 Element::map },
-    { "hosts-database",                 Element::map },
-    { "hosts-databases",                Element::list },
-    { "mac-sources",                    Element::list },
-    { "relay-supplied-options",         Element::list },
-    { "host-reservation-identifiers",   Element::list },
-    { "client-classes",                 Element::list },
-    { "option-def",                     Element::list },
-    { "option-data",                    Element::list },
-    { "hooks-libraries",                Element::list },
-    { "expired-leases-processing",      Element::map },
-    { "server-id",                      Element::map },
-    { "dhcp4o6-port",                   Element::integer },
-    { "control-socket",                 Element::map },
-    { "dhcp-queue-control",             Element::map },
-    { "dhcp-ddns",                      Element::map },
-    { "user-context",                   Element::map },
-    { "comment",                        Element::string },
-    { "sanity-checks",                  Element::map },
-    { "reservations",                   Element::list },
-    { "config-control",                 Element::map },
-    { "server-tag",                     Element::string },
-    { "reservation-mode",               Element::string },
-    { "reservations-global",            Element::boolean },
-    { "reservations-in-subnet",         Element::boolean },
-    { "reservations-out-of-pool",       Element::boolean },
-    { "calculate-tee-times",            Element::boolean },
-    { "t1-percent",                     Element::real },
-    { "t2-percent",                     Element::real },
-    { "loggers",                        Element::list },
-    { "hostname-char-set",              Element::string },
-    { "hostname-char-replacement",      Element::string },
-    { "ddns-send-updates",              Element::boolean },
-    { "ddns-override-no-update",        Element::boolean },
-    { "ddns-override-client-update",    Element::boolean },
-    { "ddns-replace-client-name",       Element::string },
-    { "ddns-generated-prefix",          Element::string },
-    { "ddns-qualifying-suffix",         Element::string },
-    { "store-extended-info",            Element::boolean },
-    { "statistic-default-sample-count", Element::integer },
-    { "statistic-default-sample-age",   Element::integer },
-    { "multi-threading",                Element::map },
-    { "cache-threshold",                Element::real },
-    { "cache-max-age",                  Element::integer },
-    { "ip-reservations-unique",         Element::boolean },
-    { "ddns-update-on-renew",           Element::boolean },
-    { "ddns-use-conflict-resolution",   Element::boolean },
-    { "compatibility",                  Element::map },
-    { "parked-packet-limit",            Element::integer },
+    { "data-directory",                   Element::string },
+    { "preferred-lifetime",               Element::integer },
+    { "min-preferred-lifetime",           Element::integer },
+    { "max-preferred-lifetime",           Element::integer },
+    { "valid-lifetime",                   Element::integer },
+    { "min-valid-lifetime",               Element::integer },
+    { "max-valid-lifetime",               Element::integer },
+    { "renew-timer",                      Element::integer },
+    { "rebind-timer",                     Element::integer },
+    { "decline-probation-period",         Element::integer },
+    { "subnet6",                          Element::list },
+    { "shared-networks",                  Element::list },
+    { "interfaces-config",                Element::map },
+    { "lease-database",                   Element::map },
+    { "hosts-database",                   Element::map },
+    { "hosts-databases",                  Element::list },
+    { "mac-sources",                      Element::list },
+    { "relay-supplied-options",           Element::list },
+    { "host-reservation-identifiers",     Element::list },
+    { "client-classes",                   Element::list },
+    { "option-def",                       Element::list },
+    { "option-data",                      Element::list },
+    { "hooks-libraries",                  Element::list },
+    { "expired-leases-processing",        Element::map },
+    { "server-id",                        Element::map },
+    { "dhcp4o6-port",                     Element::integer },
+    { "control-socket",                   Element::map },
+    { "dhcp-queue-control",               Element::map },
+    { "dhcp-ddns",                        Element::map },
+    { "user-context",                     Element::map },
+    { "comment",                          Element::string },
+    { "sanity-checks",                    Element::map },
+    { "reservations",                     Element::list },
+    { "config-control",                   Element::map },
+    { "server-tag",                       Element::string },
+    { "reservation-mode",                 Element::string },
+    { "reservations-global",              Element::boolean },
+    { "reservations-in-subnet",           Element::boolean },
+    { "reservations-out-of-pool",         Element::boolean },
+    { "calculate-tee-times",              Element::boolean },
+    { "t1-percent",                       Element::real },
+    { "t2-percent",                       Element::real },
+    { "loggers",                          Element::list },
+    { "hostname-char-set",                Element::string },
+    { "hostname-char-replacement",        Element::string },
+    { "ddns-send-updates",                Element::boolean },
+    { "ddns-override-no-update",          Element::boolean },
+    { "ddns-override-client-update",      Element::boolean },
+    { "ddns-replace-client-name",         Element::string },
+    { "ddns-generated-prefix",            Element::string },
+    { "ddns-qualifying-suffix",           Element::string },
+    { "store-extended-info",              Element::boolean },
+    { "statistic-default-sample-count",   Element::integer },
+    { "statistic-default-sample-age",     Element::integer },
+    { "multi-threading",                  Element::map },
+    { "cache-threshold",                  Element::real },
+    { "cache-max-age",                    Element::integer },
+    { "early-global-reservations-lookup", Element::boolean },
+    { "ip-reservations-unique",           Element::boolean },
+    { "reservations-lookup-first",        Element::boolean },
+    { "ddns-update-on-renew",             Element::boolean },
+    { "ddns-use-conflict-resolution",     Element::boolean },
+    { "compatibility",                    Element::map },
+    { "parked-packet-limit",              Element::integer },
 };
 
 /// @brief This table defines default global values for DHCPv6
@@ -106,32 +108,34 @@ const SimpleKeywords SimpleParser6::GLOBAL6_PARAMETERS = {
 /// in Dhcp6) are optional. If not defined, the following values will be
 /// used.
 const SimpleDefaults SimpleParser6::GLOBAL6_DEFAULTS = {
-    { "preferred-lifetime",             Element::integer, "3600" },
-    { "valid-lifetime",                 Element::integer, "7200" },
-    { "decline-probation-period",       Element::integer, "86400" }, // 24h
-    { "dhcp4o6-port",                   Element::integer, "0" },
-    { "server-tag",                     Element::string,  "" },
-    { "reservations-global",            Element::boolean, "false" },
-    { "reservations-in-subnet",         Element::boolean, "true" },
-    { "reservations-out-of-pool",       Element::boolean, "false" },
-    { "calculate-tee-times",            Element::boolean, "true" },
-    { "t1-percent",                     Element::real,    ".50" },
-    { "t2-percent",                     Element::real,    ".80" },
-    { "ddns-send-updates",              Element::boolean, "true" },
-    { "ddns-override-no-update",        Element::boolean, "false" },
-    { "ddns-override-client-update",    Element::boolean, "false" },
-    { "ddns-replace-client-name",       Element::string,  "never" },
-    { "ddns-generated-prefix",          Element::string,  "myhost" },
-    { "ddns-qualifying-suffix",         Element::string,  "" },
-    { "hostname-char-set",              Element::string,  "[^A-Za-z0-9.-]" },
-    { "hostname-char-replacement",      Element::string,  "" },
-    { "store-extended-info",            Element::boolean, "false" },
-    { "statistic-default-sample-count", Element::integer, "20" },
-    { "statistic-default-sample-age",   Element::integer, "0" },
-    { "ip-reservations-unique",         Element::boolean, "true" },
-    { "ddns-update-on-renew",           Element::boolean, "false" },
-    { "ddns-use-conflict-resolution",   Element::boolean, "true" },
-    { "parked-packet-limit",            Element::integer, "256" }
+    { "preferred-lifetime",               Element::integer, "3600" },
+    { "valid-lifetime",                   Element::integer, "7200" },
+    { "decline-probation-period",         Element::integer, "86400" }, // 24h
+    { "dhcp4o6-port",                     Element::integer, "0" },
+    { "server-tag",                       Element::string,  "" },
+    { "reservations-global",              Element::boolean, "false" },
+    { "reservations-in-subnet",           Element::boolean, "true" },
+    { "reservations-out-of-pool",         Element::boolean, "false" },
+    { "calculate-tee-times",              Element::boolean, "true" },
+    { "t1-percent",                       Element::real,    ".50" },
+    { "t2-percent",                       Element::real,    ".80" },
+    { "ddns-send-updates",                Element::boolean, "true" },
+    { "ddns-override-no-update",          Element::boolean, "false" },
+    { "ddns-override-client-update",      Element::boolean, "false" },
+    { "ddns-replace-client-name",         Element::string,  "never" },
+    { "ddns-generated-prefix",            Element::string,  "myhost" },
+    { "ddns-qualifying-suffix",           Element::string,  "" },
+    { "hostname-char-set",                Element::string,  "[^A-Za-z0-9.-]" },
+    { "hostname-char-replacement",        Element::string,  "" },
+    { "store-extended-info",              Element::boolean, "false" },
+    { "statistic-default-sample-count",   Element::integer, "20" },
+    { "statistic-default-sample-age",     Element::integer, "0" },
+    { "early-global-reservations-lookup", Element::boolean, "false" },
+    { "ip-reservations-unique",           Element::boolean, "true" },
+    { "reservations-lookup-first",        Element::boolean, "false" },
+    { "ddns-update-on-renew",             Element::boolean, "false" },
+    { "ddns-use-conflict-resolution",     Element::boolean, "true" },
+    { "parked-packet-limit",              Element::integer, "256" }
 };
 
 /// @brief This table defines all option definition parameters.
