@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -358,6 +358,36 @@ PgSqlTaggedStatement tagged_statements[] = {
       "  FROM lease6_stat "
       "  WHERE subnet_id >= $1 and subnet_id <= $2 "
       "  ORDER BY subnet_id, lease_type, state" },
+
+    // CHECK_LEASE4_LIMITS
+    { 1, { OID_TEXT },
+      "check_lease4_limits",
+      "SELECT checkLease4Limits($1)" },
+
+    // CHECK_LEASE6_LIMITS
+    { 1, { OID_TEXT },
+      "check_lease6_limits",
+      "SELECT checkLease6Limits($1)" },
+
+    // IS_JSON_SUPPORTED
+    { 0, { OID_NONE },
+      "is_json_supported",
+      "SELECT isJsonSupported()" },
+
+    // GET_LEASE4_COUNT_BY_CLASS
+    { 1, { OID_VARCHAR },
+      "get_lease4_count_by_class",
+      "SELECT leases "
+          "FROM lease4_stat_by_client_class "
+          "WHERE client_class = $1"},
+
+    // GET_LEASE6_COUNT_BY_CLASS
+    { 2, { OID_VARCHAR, OID_INT2 },
+      "get_lease6_count_by_class",
+      "SELECT leases "
+          "FROM lease6_stat_by_client_class "
+          "WHERE client_class = $1 AND lease_type = $2"},
+
     // End of list sentinel
     { 0,  { 0 }, NULL, NULL}
 };
@@ -502,7 +532,12 @@ public:
             valid_lifetime_str_ = boost::lexical_cast<std::string>(lease->valid_lft_);
             bind_array.add(valid_lifetime_str_);
 
-            // Avoid overflow
+            // The lease structure holds the client last transmission time (cltt_)
+            // For convenience for external tools, this is converted to lease
+            // expiry time (expire).  The relationship is given by:
+            // expire = cltt_ + valid_lft_
+            // Avoid overflow with infinite valid lifetime by using
+            // expire = cltt_ when valid_lft_ = 0xffffffff
             if (lease_->valid_lft_ == Lease::INFINITY_LFT) {
                 expire_str_ = convertToDatabaseTime(lease->cltt_, 0);
             } else {
@@ -562,7 +597,7 @@ public:
 
             getColumnValue(r, row , SUBNET_ID_COL, subnet_id_);
 
-            // Recover from overflow
+            // Recover from overflow (see createBindForSend)
             if (valid_lifetime_ == Lease::INFINITY_LFT) {
                 cltt_ = expire_;
             } else {
@@ -744,7 +779,12 @@ public:
             valid_lifetime_str_ = boost::lexical_cast<std::string>(lease->valid_lft_);
             bind_array.add(valid_lifetime_str_);
 
-            // Avoid overflow
+            // The lease structure holds the client last transmission time (cltt_)
+            // For convenience for external tools, this is converted to lease
+            // expiry time (expire).  The relationship is given by:
+            // expire = cltt_ + valid_lft_
+            // Avoid overflow with infinite valid lifetime by using
+            // expire = cltt_ when valid_lft_ = 0xffffffff
             if (lease_->valid_lft_ == Lease::INFINITY_LFT) {
                 expire_str_ = convertToDatabaseTime(lease->cltt_, 0);
             } else {
@@ -855,7 +895,7 @@ public:
             expire_ = convertFromDatabaseTime(getRawColumnValue(r, row,
                                                                 EXPIRE_COL));
 
-            // Recover from overflow
+            // Recover from overflow (see createBindForSend)
             if (valid_lifetime_ == Lease::INFINITY_LFT) {
                 cltt_ = expire_;
             } else {
@@ -1064,14 +1104,12 @@ public:
             PsqlBindArray parms;
 
             // Add first_subnet_id used by both single and range.
-            std::string subnet_id_str = boost::lexical_cast<std::string>(getFirstSubnetID());
-            parms.add(subnet_id_str);
+            parms.addTempString(boost::lexical_cast<std::string>(getFirstSubnetID()));
 
             // Add last_subnet_id for range.
             if (getSelectMode() == SUBNET_RANGE) {
                 // Add last_subnet_id used by range.
-                std::string subnet_id_str = boost::lexical_cast<std::string>(getLastSubnetID());
-                parms.add(subnet_id_str);
+                parms.addTempString(boost::lexical_cast<std::string>(getLastSubnetID()));
             }
 
             // Run the query with where clause parameters.
@@ -1222,9 +1260,31 @@ PgSqlLeaseMgr::PgSqlLeaseMgr(const DatabaseConnection::ParameterMap& parameters)
     timer_name_ += boost::lexical_cast<std::string>(reinterpret_cast<uint64_t>(this));
     timer_name_ += "]DbReconnectTimer";
 
+    // Check TLS support.
+    size_t tls(0);
+    tls += parameters.count("trust-anchor");
+    tls += parameters.count("cert-file");
+    tls += parameters.count("key-file");
+    tls += parameters.count("cipher-list");
+#ifdef HAVE_PGSQL_SSL
+    if ((tls > 0) && !PgSqlConnection::warned_about_tls) {
+        PgSqlConnection::warned_about_tls = true;
+        LOG_INFO(dhcpsrv_logger, DHCPSRV_PGSQL_TLS_SUPPORT)
+            .arg(DatabaseConnection::redactedAccessString(parameters_));
+        PQinitSSL(1);
+    }
+#else
+    if (tls > 0) {
+        LOG_ERROR(dhcpsrv_logger, DHCPSRV_PGSQL_NO_TLS_SUPPORT)
+            .arg(DatabaseConnection::redactedAccessString(parameters_));
+        isc_throw(DbOpenError, "Attempt to configure TLS for PostgreSQL "
+                  << "backend (built with this feature disabled)");
+    }
+#endif
+
     // Validate schema version first.
-    std::pair<uint32_t, uint32_t> code_version(PG_SCHEMA_VERSION_MAJOR,
-                                               PG_SCHEMA_VERSION_MINOR);
+    std::pair<uint32_t, uint32_t> code_version(PGSQL_SCHEMA_VERSION_MAJOR,
+                                               PGSQL_SCHEMA_VERSION_MINOR);
     std::pair<uint32_t, uint32_t> db_version = getVersion();
     if (code_version != db_version) {
         isc_throw(DbOpenError,
@@ -1347,8 +1407,8 @@ PgSqlLeaseMgr::createContext() const {
 std::string
 PgSqlLeaseMgr::getDBVersion() {
     std::stringstream tmp;
-    tmp << "PostgreSQL backend " << PG_SCHEMA_VERSION_MAJOR;
-    tmp << "." << PG_SCHEMA_VERSION_MINOR;
+    tmp << "PostgreSQL backend " << PGSQL_SCHEMA_VERSION_MAJOR;
+    tmp << "." << PGSQL_SCHEMA_VERSION_MINOR;
     tmp << ", library " << PQlibVersion();
     return (tmp.str());
 }
@@ -1989,7 +2049,7 @@ PgSqlLeaseMgr::getExpiredLeasesCommon(LeaseCollection& expired_leases,
     bind_array.add(state_str);
 
     // Expiration timestamp.
-    std::string timestamp_str = PgSqlLeaseExchange::convertToDatabaseTime(time(NULL));
+    std::string timestamp_str = PgSqlLeaseExchange::convertToDatabaseTime(time(0));
     bind_array.add(timestamp_str);
 
     // If the number of leases is 0, we will return all leases. This is
@@ -2059,8 +2119,14 @@ PgSqlLeaseMgr::updateLease4(const Lease4Ptr& lease) {
     std::string addr4_str = boost::lexical_cast<std::string>(lease->addr_.toUint32());
     bind_array.add(addr4_str);
 
-    std::string expire_str = PgSqlLeaseExchange::convertToDatabaseTime(lease->current_cltt_,
-                                                                       lease->current_valid_lft_);
+    std::string expire_str;
+    // Avoid overflow (see createBindForSend)
+    if (lease->current_valid_lft_ == Lease::INFINITY_LFT) {
+        expire_str = PgSqlLeaseExchange::convertToDatabaseTime(lease->current_cltt_, 0);
+    } else {
+        expire_str = PgSqlLeaseExchange::convertToDatabaseTime(lease->current_cltt_,
+                                                               lease->current_valid_lft_);
+    }
     bind_array.add(expire_str);
 
     // Drop to common update code
@@ -2090,8 +2156,14 @@ PgSqlLeaseMgr::updateLease6(const Lease6Ptr& lease) {
     std::string addr_str = lease->addr_.toText();
     bind_array.add(addr_str);
 
-    std::string expire_str = PgSqlLeaseExchange::convertToDatabaseTime(lease->current_cltt_,
-                                                                       lease->current_valid_lft_);
+    std::string expire_str;
+    // Avoid overflow (see createBindForSend)
+    if (lease->current_valid_lft_ == Lease::INFINITY_LFT) {
+        expire_str = PgSqlLeaseExchange::convertToDatabaseTime(lease->current_cltt_, 0);
+    } else {
+        expire_str = PgSqlLeaseExchange::convertToDatabaseTime(lease->current_cltt_,
+                                                               lease->current_valid_lft_);
+    }
     bind_array.add(expire_str);
 
     // Drop to common update code
@@ -2132,8 +2204,14 @@ PgSqlLeaseMgr::deleteLease(const Lease4Ptr& lease) {
     std::string addr4_str = boost::lexical_cast<std::string>(addr.toUint32());
     bind_array.add(addr4_str);
 
-    std::string expire_str = PgSqlLeaseExchange::convertToDatabaseTime(lease->current_cltt_,
-                                                                       lease->current_valid_lft_);
+    std::string expire_str;
+    // Avoid overflow (see createBindForSend)
+    if (lease->current_valid_lft_ == Lease::INFINITY_LFT) {
+        expire_str = PgSqlLeaseExchange::convertToDatabaseTime(lease->current_cltt_, 0);
+    } else {
+        expire_str = PgSqlLeaseExchange::convertToDatabaseTime(lease->current_cltt_,
+                                                               lease->current_valid_lft_);
+    }
     bind_array.add(expire_str);
 
     auto affected_rows = deleteLeaseCommon(DELETE_LEASE4, bind_array);
@@ -2167,8 +2245,14 @@ PgSqlLeaseMgr::deleteLease(const Lease6Ptr& lease) {
     std::string addr6_str = addr.toText();
     bind_array.add(addr6_str);
 
-    std::string expire_str = PgSqlLeaseExchange::convertToDatabaseTime(lease->current_cltt_,
-                                                                       lease->current_valid_lft_);
+    std::string expire_str;
+    // Avoid overflow (see createBindForSend)
+    if (lease->current_valid_lft_ == Lease::INFINITY_LFT) {
+        expire_str = PgSqlLeaseExchange::convertToDatabaseTime(lease->current_cltt_, 0);
+    } else {
+        expire_str = PgSqlLeaseExchange::convertToDatabaseTime(lease->current_cltt_,
+                                                               lease->current_valid_lft_);
+    }
     bind_array.add(expire_str);
 
     auto affected_rows = deleteLeaseCommon(DELETE_LEASE6, bind_array);
@@ -2213,7 +2297,7 @@ PgSqlLeaseMgr::deleteExpiredReclaimedLeasesCommon(const uint32_t secs,
     bind_array.add(state_str);
 
     // Expiration timestamp.
-    std::string expiration_str = PgSqlLeaseExchange::convertToDatabaseTime(time(NULL) -
+    std::string expiration_str = PgSqlLeaseExchange::convertToDatabaseTime(time(0) -
         static_cast<time_t>(secs));
     bind_array.add(expiration_str);
 
@@ -2221,6 +2305,114 @@ PgSqlLeaseMgr::deleteExpiredReclaimedLeasesCommon(const uint32_t secs,
     return (deleteLeaseCommon(statement_index, bind_array));
 }
 
+string
+PgSqlLeaseMgr::checkLimits(ConstElementPtr const& user_context, StatementIndex const stindex) const {
+    // No user context means no limits means allocation allowed means empty string.
+    if (!user_context) {
+        return string();
+    }
+
+    // Get a context.
+    PgSqlLeaseContextAlloc get_context(*this);
+    PgSqlLeaseContextPtr ctx(get_context.ctx_);
+
+    // Create bindings.
+    PsqlBindArray bind_array;
+    std::string const user_context_str(user_context->str());
+    bind_array.add(user_context_str);
+
+    // Execute the select.
+    PgSqlResult r(PQexecPrepared(ctx->conn_,
+                                 tagged_statements[stindex].name,
+                                 tagged_statements[stindex].nbparams,
+                                 &bind_array.values_[0],
+                                 &bind_array.lengths_[0],
+                                 &bind_array.formats_[0], 0));
+    ctx->conn_.checkStatementError(r, tagged_statements[stindex]);
+
+    std::string limits;
+    PgSqlExchange::getColumnValue(r, 0, 0, limits);
+    return limits;
+}
+
+string
+PgSqlLeaseMgr::checkLimits4(ConstElementPtr const& user_context) const {
+    return checkLimits(user_context, CHECK_LEASE4_LIMITS);
+}
+
+string
+PgSqlLeaseMgr::checkLimits6(ConstElementPtr const& user_context) const {
+    return checkLimits(user_context, CHECK_LEASE6_LIMITS);
+}
+
+bool
+PgSqlLeaseMgr::isJsonSupported() const {
+    // Get a context.
+    PgSqlLeaseContextAlloc get_context(*this);
+    PgSqlLeaseContextPtr ctx(get_context.ctx_);
+
+    // Execute the select.
+    StatementIndex const stindex(IS_JSON_SUPPORTED);
+    PgSqlResult r(PQexecPrepared(ctx->conn_, tagged_statements[stindex].name,
+                                 0, 0, 0, 0, 0));
+    ctx->conn_.checkStatementError(r, tagged_statements[stindex]);
+
+    bool json_supported;
+    PgSqlExchange::getColumnValue(r, 0, 0, json_supported);
+    return json_supported;
+}
+
+size_t
+PgSqlLeaseMgr::getClassLeaseCount(const ClientClass& client_class,
+                                  const Lease::Type& ltype /* = Lease::TYPE_V4*/) const {
+    // Get a context.
+    PgSqlLeaseContextAlloc get_context(*this);
+    PgSqlLeaseContextPtr ctx(get_context.ctx_);
+
+    // Create bindings.
+    PsqlBindArray bind_array;
+    bind_array.add(client_class);
+    if (ltype != Lease::TYPE_V4) {
+        bind_array.add(ltype);
+    }
+
+    // Execute the select.
+    StatementIndex const stindex(ltype == Lease::TYPE_V4 ? GET_LEASE4_COUNT_BY_CLASS :
+                                                           GET_LEASE6_COUNT_BY_CLASS);
+    PgSqlResult r(PQexecPrepared(ctx->conn_,
+                                 tagged_statements[stindex].name,
+                                 tagged_statements[stindex].nbparams,
+                                 &bind_array.values_[0],
+                                 &bind_array.lengths_[0],
+                                 &bind_array.formats_[0], 0));
+    ctx->conn_.checkStatementError(r, tagged_statements[stindex]);
+
+    int rows = PQntuples(r);
+    if (rows == 0) {
+        // No entries means 0 leases.
+        return 0;
+    }
+
+    size_t count;
+    PgSqlExchange::getColumnValue(r, 0, 0, count);
+    return count;
+}
+
+void
+PgSqlLeaseMgr::recountClassLeases4() {
+    isc_throw(NotImplemented, "PgSqlLeaseMgr::recountClassLeases4() not implemented");
+}
+
+void
+PgSqlLeaseMgr::recountClassLeases6() {
+    isc_throw(NotImplemented, "PgSqlLeaseMgr::recountClassLeases6() not implemented");
+}
+
+void
+PgSqlLeaseMgr::clearClassLeaseCounts() {
+    isc_throw(NotImplemented, "PgSqlLeaseMgr::clearClassLeaseCounts() not implemented");
+}
+
 LeaseStatsQueryPtr
 PgSqlLeaseMgr::startLeaseStatsQuery4() {
     // Get a context
