@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -37,6 +37,7 @@ using namespace isc::dhcp;
 using namespace isc::dhcp::test;
 using namespace isc::stats;
 using namespace isc::test;
+using namespace isc::util;
 
 namespace {
 
@@ -508,7 +509,8 @@ TEST(CfgSubnets4Test, selectSharedNetworkByIface) {
     ASSERT_TRUE(network_returned);
     EXPECT_EQ(network, network_returned);
 
-    const Subnet4Collection* subnets_eth1 = network_returned->getAllSubnets();
+    const Subnet4SimpleCollection* subnets_eth1 =
+        network_returned->getAllSubnets();
     EXPECT_EQ(2, subnets_eth1->size());
     ASSERT_TRUE(network_returned->getSubnet(SubnetID(1)));
     ASSERT_TRUE(network_returned->getSubnet(SubnetID(2)));
@@ -1075,6 +1077,8 @@ TEST(CfgSubnets4Test, unparseSubnet) {
         "    \"rebind-timer\": 2,\n"
         "    \"relay\": { \"ip-addresses\": [ ] },\n"
         "    \"valid-lifetime\": 3,\n"
+        "    \"min-valid-lifetime\": 3,\n"
+        "    \"max-valid-lifetime\": 3,\n"
         "    \"client-class\": \"foo\",\n"
         "    \"4o6-interface\": \"\",\n"
         "    \"4o6-interface-id\": \"\",\n"
@@ -1090,6 +1094,8 @@ TEST(CfgSubnets4Test, unparseSubnet) {
         "    \"rebind-timer\": 2,\n"
         "    \"relay\": { \"ip-addresses\": [ \"10.0.0.1\" ] },\n"
         "    \"valid-lifetime\": 100,\n"
+        "    \"min-valid-lifetime\": 100,\n"
+        "    \"max-valid-lifetime\": 100,\n"
         "    \"4o6-interface\": \"\",\n"
         "    \"4o6-interface-id\": \"\",\n"
         "    \"4o6-subnet\": \"\",\n"
@@ -1168,6 +1174,8 @@ TEST(CfgSubnets4Test, unparsePool) {
         "    \"rebind-timer\": 2,\n"
         "    \"relay\": { \"ip-addresses\": [ ] },\n"
         "    \"valid-lifetime\": 3,\n"
+        "    \"min-valid-lifetime\": 3,\n"
+        "    \"max-valid-lifetime\": 3,\n"
         "    \"4o6-interface\": \"\",\n"
         "    \"4o6-interface-id\": \"\",\n"
         "    \"4o6-subnet\": \"\",\n"
@@ -1399,8 +1407,12 @@ TEST(CfgSubnets4Test, validLifetimeValidation) {
         data::ConstElementPtr value = repr->get("valid-lifetime");
         ASSERT_TRUE(value);
         EXPECT_EQ("100", value->str());
-        EXPECT_FALSE(repr->get("min-valid-lifetime"));
-        EXPECT_FALSE(repr->get("max-valid-lifetime"));
+        data::ConstElementPtr min_value = repr->get("min-valid-lifetime");
+        ASSERT_TRUE(min_value);
+        EXPECT_EQ("100", min_value->str());
+        data::ConstElementPtr max_value = repr->get("max-valid-lifetime");
+        ASSERT_TRUE(max_value);
+        EXPECT_EQ("100", max_value->str());
     }
 
     {
@@ -1419,9 +1431,12 @@ TEST(CfgSubnets4Test, validLifetimeValidation) {
         data::ConstElementPtr value = repr->get("valid-lifetime");
         ASSERT_TRUE(value);
         EXPECT_EQ("100", value->str());
-        // Bound only: forgot it was a bound.
-        EXPECT_FALSE(repr->get("min-valid-lifetime"));
-        EXPECT_FALSE(repr->get("max-valid-lifetime"));
+        data::ConstElementPtr min_value = repr->get("min-valid-lifetime");
+        ASSERT_TRUE(min_value);
+        EXPECT_EQ("100", min_value->str());
+        data::ConstElementPtr max_value = repr->get("max-valid-lifetime");
+        ASSERT_TRUE(max_value);
+        EXPECT_EQ("100", max_value->str());
     }
 
     {
@@ -1440,9 +1455,12 @@ TEST(CfgSubnets4Test, validLifetimeValidation) {
         data::ConstElementPtr value = repr->get("valid-lifetime");
         ASSERT_TRUE(value);
         EXPECT_EQ("100", value->str());
-        // Bound only: forgot it was a bound.
-        EXPECT_FALSE(repr->get("min-valid-lifetime"));
-        EXPECT_FALSE(repr->get("max-valid-lifetime"));
+        data::ConstElementPtr min_value = repr->get("min-valid-lifetime");
+        ASSERT_TRUE(min_value);
+        EXPECT_EQ("100", min_value->str());
+        data::ConstElementPtr max_value = repr->get("max-valid-lifetime");
+        ASSERT_TRUE(max_value);
+        EXPECT_EQ("100", max_value->str());
     }
 
     {
@@ -1466,7 +1484,9 @@ TEST(CfgSubnets4Test, validLifetimeValidation) {
         data::ConstElementPtr min_value = repr->get("min-valid-lifetime");
         ASSERT_TRUE(min_value);
         EXPECT_EQ("100", min_value->str());
-        EXPECT_FALSE(repr->get("max-valid-lifetime"));
+        data::ConstElementPtr max_value = repr->get("max-valid-lifetime");
+        ASSERT_TRUE(max_value);
+        EXPECT_EQ("200", max_value->str());
     }
 
     {
@@ -1487,10 +1507,12 @@ TEST(CfgSubnets4Test, validLifetimeValidation) {
         data::ConstElementPtr value = repr->get("valid-lifetime");
         ASSERT_TRUE(value);
         EXPECT_EQ("100", value->str());
+        data::ConstElementPtr min_value = repr->get("min-valid-lifetime");
+        ASSERT_TRUE(min_value);
+        EXPECT_EQ("100", min_value->str());
         data::ConstElementPtr max_value = repr->get("max-valid-lifetime");
         ASSERT_TRUE(max_value);
         EXPECT_EQ("200", max_value->str());
-        EXPECT_FALSE(repr->get("min-valid-lifetime"));
     }
 
     {
@@ -1556,7 +1578,7 @@ TEST(CfgSubnets4Test, validLifetimeValidation) {
     }
 
     {
-        SCOPED_TRACE("equal bounds are ignored");
+        SCOPED_TRACE("equal bounds are no longer ignored");
         data::ElementPtr copied = data::copy(elems);
         copied->set("min-valid-lifetime", data::Element::create(100));
         copied->set("valid-lifetime", data::Element::create(100));
@@ -1573,8 +1595,12 @@ TEST(CfgSubnets4Test, validLifetimeValidation) {
         data::ConstElementPtr value = repr->get("valid-lifetime");
         ASSERT_TRUE(value);
         EXPECT_EQ("100", value->str());
-        EXPECT_FALSE(repr->get("min-valid-lifetime"));
-        EXPECT_FALSE(repr->get("max-valid-lifetime"));
+        data::ConstElementPtr min_value = repr->get("min-valid-lifetime");
+        ASSERT_TRUE(min_value);
+        EXPECT_EQ("100", min_value->str());
+        data::ConstElementPtr max_value = repr->get("max-valid-lifetime");
+        ASSERT_TRUE(max_value);
+        EXPECT_EQ("100", max_value->str());
     }
 }
 
@@ -1731,7 +1757,7 @@ TEST(CfgSubnets4Test, iface) {
         "        {"
         "            \"id\": 1,\n"
         "            \"subnet\": \"10.1.2.0/24\", \n"
-        "            \"interface\": \"eth1\"\n"
+        "            \"interface\": \"eth1961\"\n"
         "        }";
 
     data::ElementPtr elems;
@@ -1744,7 +1770,7 @@ TEST(CfgSubnets4Test, iface) {
     EXPECT_NO_THROW(subnet = parser_no_check.parse(elems));
     ASSERT_TRUE(subnet);
     EXPECT_FALSE(subnet->getIface().unspecified());
-    EXPECT_EQ("eth1", subnet->getIface().get());
+    EXPECT_EQ("eth1961", subnet->getIface().get());
 
     // Retry with the interface check enabled.
     Subnet4ConfigParser parser;
@@ -1756,12 +1782,12 @@ TEST(CfgSubnets4Test, iface) {
     EXPECT_NO_THROW(subnet = parser_no_check.parse(elems));
     ASSERT_TRUE(subnet);
     EXPECT_FALSE(subnet->getIface().unspecified());
-    EXPECT_EQ("eth1", subnet->getIface().get());
+    EXPECT_EQ("eth1961", subnet->getIface().get());
 
     EXPECT_NO_THROW(subnet = parser.parse(elems));
     ASSERT_TRUE(subnet);
     EXPECT_FALSE(subnet->getIface().unspecified());
-    EXPECT_EQ("eth1", subnet->getIface().get());
+    EXPECT_EQ("eth1961", subnet->getIface().get());
 }
 
 // This test verifies that update statistics works as expected.
