@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -299,7 +299,7 @@ public:
                                      "universe=6" : "universe=4");
             LeaseMgrFactory::create(s.str());
         } catch (const std::exception& ex) {
-            std::cerr << "*** ERROR: unable to create instance of the Memfile\n"
+            std::cerr << "*** ERROR: unable to create instance of the Memfile"
                 " lease database backend: " << ex.what() << std::endl;
             throw;
         }
