@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -27,6 +27,7 @@
 using namespace isc::data;
 using namespace isc::dhcp;
 using namespace isc::asiolink;
+using namespace isc::util;
 
 namespace {
 
@@ -1437,5 +1438,17 @@ TEST_F(ClientClassDefParserTest, preferredLifetimeTests) {
     }
 }
 
+// Verifies that an invalid user-context fails to parse.
+TEST_F(ClientClassDefParserTest, invalidUserContext) {
+    std::string cfg_text =
+        "{ \n"
+        "    \"name\": \"one\", \n"
+        "   \"user-context\": \"i am not a map\" \n"
+        "} \n";
+
+    ClientClassDefPtr cclass;
+    ASSERT_THROW_MSG(cclass = parseClientClassDef(cfg_text, AF_INET),
+                 DhcpConfigError, "User context has to be a map (<string>:3:20)");
+}
 
 } // end of anonymous namespace
