@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -611,6 +611,20 @@ public:
         return (getCollection());
     }
 
+    ConstHostCollection getAllbyHostname(const std::string&) const {
+        return (getCollection());
+    }
+
+    ConstHostCollection getAllbyHostname4(const std::string&,
+                                          const SubnetID&) const {
+        return (getCollection());
+    }
+
+    ConstHostCollection getAllbyHostname6(const std::string&,
+                                          const SubnetID&) const {
+        return (getCollection());
+    }
+
     ConstHostCollection getPage4(const SubnetID&, size_t&, uint64_t,
                                  const HostPageSize&) const {
         return (getCollection());
@@ -621,6 +635,16 @@ public:
         return (getCollection());
     }
 
+    ConstHostCollection getPage4(size_t&, uint64_t,
+                                 const HostPageSize&) const {
+        return (getCollection());
+    }
+
+    ConstHostCollection getPage6(size_t&, uint64_t,
+                                 const HostPageSize&) const {
+        return (getCollection());
+    }
+
     ConstHostCollection getAll4(const IOAddress&) const {
         return (getCollection());
     }
@@ -634,6 +658,11 @@ public:
         return (getOne());
     }
 
+    ConstHostCollection
+    getAll4(const SubnetID&, const asiolink::IOAddress&) const {
+        return (getCollection());
+    }
+
     ConstHostPtr get6(const SubnetID&, const Host::IdentifierType&,
                       const uint8_t*, const size_t) const {
         return (getOne());
@@ -647,6 +676,11 @@ public:
         return (getOne());
     }
 
+    ConstHostCollection
+    getAll6(const SubnetID&, const IOAddress&) const {
+        return (getCollection());
+    }
+
     void add(const HostPtr&) {
     }
 
@@ -668,6 +702,10 @@ public:
         return ("one");
     }
 
+    bool setIPReservationsUnique(const bool) {
+        return (true);
+    }
+
     /// Specific methods
 
     /// @brief Set the entry
