@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -665,19 +665,20 @@ TEST_F(HostTest, setValues) {
     ASSERT_NO_THROW(host.reset(new Host("01:02:03:04:05:06", "hw-address",
                                         SubnetID(1), SubnetID(2),
                                         IOAddress("192.0.2.3"),
-                                        "some-host.example.org")));
+                                        "some-host.eXAMple.org")));
 
     ASSERT_EQ(1, host->getIPv4SubnetID());
     ASSERT_EQ(2, host->getIPv6SubnetID());
     ASSERT_EQ("192.0.2.3", host->getIPv4Reservation().toText());
-    ASSERT_EQ("some-host.example.org", host->getHostname());
+    ASSERT_EQ("some-host.eXAMple.org", host->getHostname());
+    ASSERT_EQ("some-host.example.org", host->getLowerHostname());
     ASSERT_FALSE(host->getContext());
     ASSERT_FALSE(host->getNegative());
 
     host->setIPv4SubnetID(SubnetID(123));
     host->setIPv6SubnetID(SubnetID(234));
     host->setIPv4Reservation(IOAddress("10.0.0.1"));
-    host->setHostname("other-host.example.org");
+    host->setHostname("other-host.eXAMple.org");
     host->setNextServer(IOAddress("192.0.2.2"));
     host->setServerHostname("server-hostname.example.org");
     host->setBootFileName("bootfile.efi");
@@ -690,7 +691,8 @@ TEST_F(HostTest, setValues) {
     EXPECT_EQ(123, host->getIPv4SubnetID());
     EXPECT_EQ(234, host->getIPv6SubnetID());
     EXPECT_EQ("10.0.0.1", host->getIPv4Reservation().toText());
-    EXPECT_EQ("other-host.example.org", host->getHostname());
+    EXPECT_EQ("other-host.eXAMple.org", host->getHostname());
+    ASSERT_EQ("other-host.example.org", host->getLowerHostname());
     EXPECT_EQ("192.0.2.2", host->getNextServer().toText());
     EXPECT_EQ("server-hostname.example.org", host->getServerHostname());
     EXPECT_EQ("bootfile.efi", host->getBootFileName());
@@ -969,7 +971,7 @@ TEST_F(HostTest, toText) {
                                        IOAddress("2001:db8:1::1")));
     );
 
-    // Add invisble user context
+    // Add invisible user context
     std::string user_context = "{ \"foo\": \"bar\" }";
     host->setContext(Element::fromJSON(user_context));
 
@@ -1091,24 +1093,24 @@ TEST_F(HostTest, unparse) {
     EXPECT_EQ("{ "
               "\"boot-file-name\": \"\", "
               "\"client-classes\": [  ], "
-              "\"comment\": \"a host reservation\", "
               "\"hostname\": \"myhost.example.com\", "
               "\"hw-address\": \"01:02:03:04:05:06\", "
               "\"ip-address\": \"192.0.2.3\", "
               "\"next-server\": \"0.0.0.0\", "
               "\"option-data\": [  ], "
-              "\"server-hostname\": \"\" "
+              "\"server-hostname\": \"\", "
+              "\"user-context\": { \"comment\": \"a host reservation\" } "
               "}",
               host->toElement4()->str());
 
     EXPECT_EQ("{ "
               "\"client-classes\": [  ], "
-              "\"comment\": \"a host reservation\", "
               "\"hostname\": \"myhost.example.com\", "
               "\"hw-address\": \"01:02:03:04:05:06\", "
               "\"ip-addresses\": [ \"2001:db8:1::cafe\", \"2001:db8:1::1\" ], "
               "\"option-data\": [  ], "
-              "\"prefixes\": [ \"2001:db8:1:1::/64\", \"2001:db8:1:2::/64\" ] "
+              "\"prefixes\": [ \"2001:db8:1:1::/64\", \"2001:db8:1:2::/64\" ], "
+              "\"user-context\": { \"comment\": \"a host reservation\" } "
               "}",
               host->toElement6()->str());
 
@@ -1120,23 +1122,23 @@ TEST_F(HostTest, unparse) {
     EXPECT_EQ("{ "
               "\"boot-file-name\": \"\", "
               "\"client-classes\": [  ], "
-              "\"comment\": \"a host reservation\", "
               "\"hostname\": \"\", "
               "\"hw-address\": \"01:02:03:04:05:06\", "
               "\"next-server\": \"0.0.0.0\", "
               "\"option-data\": [  ], "
-              "\"server-hostname\": \"\" "
+              "\"server-hostname\": \"\", "
+              "\"user-context\": { \"comment\": \"a host reservation\" } "
               "}",
               host->toElement4()->str());
 
     EXPECT_EQ("{ "
               "\"client-classes\": [  ], "
-              "\"comment\": \"a host reservation\", "
               "\"hostname\": \"\", "
               "\"hw-address\": \"01:02:03:04:05:06\", "
               "\"ip-addresses\": [ \"2001:db8:1::cafe\", \"2001:db8:1::1\" ], "
               "\"option-data\": [  ], "
-              "\"prefixes\": [ \"2001:db8:1:1::/64\", \"2001:db8:1:2::/64\" ] "
+              "\"prefixes\": [ \"2001:db8:1:1::/64\", \"2001:db8:1:2::/64\" ], "
+              "\"user-context\": { \"comment\": \"a host reservation\" } "
               "}",
               host->toElement6()->str());
 
@@ -1235,7 +1237,7 @@ TEST_F(HostTest, hostId) {
     EXPECT_EQ(12345, host->getHostId());
 }
 
-// Tets verifies if we can modify the host keys.
+// Test verifies if we can modify the host keys.
 TEST_F(HostTest, keys) {
     HostPtr host;
     ASSERT_NO_THROW(host.reset(new Host("01:02:03:04:05:06", "hw-address",
