@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -126,7 +126,7 @@ TEST_F(Lease4Test, constructor) {
         // Create the lease
         Lease4 lease(ADDRESS[i], hwaddr_, clientid_, VALID_LIFETIME,
                      current_time, SUBNET_ID, true, true,
-                     "hostname.example.com.");
+                     "Hostname.Example.Com.");
 
         EXPECT_EQ(ADDRESS[i], lease.addr_.toUint32());
         EXPECT_TRUE(util::equalValues(hwaddr_, lease.hwaddr_));
@@ -370,7 +370,7 @@ TEST_F(Lease4Test, operatorEquals) {
     EXPECT_TRUE(lease1 == lease2);  // Check that the reversion has made the
     EXPECT_FALSE(lease1 != lease2); // ... leases equal
 
-    lease1.hostname_ += std::string("Something random");
+    lease1.hostname_ += std::string("something random");
     EXPECT_FALSE(lease1 == lease2);
     EXPECT_TRUE(lease1 != lease2);
     lease1.hostname_ = lease2.hostname_;
@@ -493,7 +493,7 @@ TEST_F(Lease4Test, toElement) {
 
     const time_t current_time = 12345678;
     Lease4 lease(IOAddress("192.0.2.3"), hwaddr_, clientid_, 3600,
-                 current_time, 789, true, true, "urania.example.org");
+                 current_time, 789, true, true, "URANIA.example.org");
     lease.setContext(Element::fromJSON("{ \"foobar\": 1234 }"));
 
     std::string expected = "{"
@@ -535,7 +535,7 @@ TEST_F(Lease4Test, toElement) {
 
     expected = "{"
         "\"cltt\": 12345678,"
-        "\"comment\": \"a comment\","
+        "\"user-context\": { \"comment\": \"a comment\" },"
         "\"fqdn-fwd\": true,"
         "\"fqdn-rev\": true,"
         "\"hostname\": \"urania.example.org\","
@@ -556,7 +556,7 @@ TEST_F(Lease4Test, fromElement) {
         "\"cltt\": 12345678,"
         "\"fqdn-fwd\": true,"
         "\"fqdn-rev\": true,"
-        "\"hostname\": \"urania.example.org\","
+        "\"hostname\": \"urania.example.ORG\","
         "\"hw-address\": \"08:00:2b:02:3f:4e\","
         "\"ip-address\": \"192.0.2.3\","
         "\"state\": 0,"
@@ -577,7 +577,9 @@ TEST_F(Lease4Test, fromElement) {
     ASSERT_TRUE(lease->client_id_);
     EXPECT_EQ("17:34:e2:ff:09:92:54", lease->client_id_->toText());
     EXPECT_EQ(12345678, lease->cltt_);
+    EXPECT_EQ(lease->cltt_, lease->current_cltt_);
     EXPECT_EQ(3600, lease->valid_lft_);
+    EXPECT_EQ(lease->valid_lft_, lease->current_valid_lft_);
     EXPECT_TRUE(lease->fqdn_fwd_);
     EXPECT_TRUE(lease->fqdn_rev_);
     EXPECT_EQ("urania.example.org", lease->hostname_);
@@ -620,6 +622,7 @@ TEST_F(Lease4Test, fromElementInvalidValues) {
     testInvalidElement<Lease4>(json, "state", 1234, false);
     testInvalidElement<Lease4>(json, "subnet-id", std::string("xyz"));
     testInvalidElement<Lease4>(json, "subnet-id", -5, false);
+    testInvalidElement<Lease4>(json, "subnet-id", 0x100000000, false);
     testInvalidElement<Lease4>(json, "valid-lft", std::string("xyz"));
     testInvalidElement<Lease4>(json, "valid-lft", -3, false);
     testInvalidElement<Lease4>(json, "user-context", "[ ]", false);
@@ -634,7 +637,7 @@ TEST_F(Lease4Test, decline) {
     const time_t current_time = 12345678;
     Lease4 lease(IOAddress("192.0.2.3"), hwaddr_, clientid_, 3600,
                  current_time, 789);
-    lease.hostname_="foo.example.org";
+    lease.hostname_ = "foo.example.org";
     lease.fqdn_fwd_ = true;
     lease.fqdn_rev_ = true;
 
@@ -746,7 +749,7 @@ TEST(Lease6Test, Lease6ConstructorWithFQDN) {
         IOAddress addr(ADDRESS[i]);
         Lease6Ptr lease(new Lease6(Lease::TYPE_NA, addr,
                                    duid, iaid, 100, 200, subnet_id,
-                                   true, true, "host.example.com."));
+                                   true, true, "Host.Example.Com."));
 
         EXPECT_TRUE(lease->addr_ == addr);
         EXPECT_TRUE(*lease->duid_ == *duid);
@@ -862,7 +865,7 @@ TEST(Lease6Test, operatorEquals) {
     EXPECT_TRUE(lease1 == lease2);  // Check that the reversion has made the
     EXPECT_FALSE(lease1 != lease2); // ... leases equal
 
-    lease1.hostname_ += std::string("Something random");
+    lease1.hostname_ += std::string("something random");
     EXPECT_FALSE(lease1 == lease2);
     EXPECT_TRUE(lease1 != lease2);
     lease1.hostname_ = lease2.hostname_;
@@ -926,6 +929,11 @@ TEST(Lease6Test, Lease6Expired) {
     // Case 3: the lease is expired
     lease.cltt_ = time(NULL) - 102;
     EXPECT_TRUE(lease.expired());
+
+    // Case 4: the lease is static
+    lease.cltt_ = 1;
+    lease.valid_lft_ = Lease::INFINITY_LFT;
+    EXPECT_FALSE(lease.expired());
 }
 
 // Verify that the DUID can be returned as a vector object and if DUID is NULL
@@ -1114,7 +1122,7 @@ TEST(Lease6Test, toElementAddress) {
 
     expected = "{"
         "\"cltt\": 12345678,"
-        "\"comment\": \"a comment\","
+        "\"user-context\": { \"comment\": \"a comment\" },"
         "\"duid\": \"00:01:02:03:04:05:06:0a:0b:0c:0d:0e:0f\","
         "\"fqdn-fwd\": false,"
         "\"fqdn-rev\": false,"
@@ -1202,15 +1210,17 @@ TEST(Lease6Test, toElementPrefix) {
     l = lease.toElement();
     EXPECT_FALSE(l->contains("hw-address"));
     EXPECT_FALSE(l->contains("user-context"));
-    EXPECT_FALSE(l->contains("comment"));
 
     // And to finish try with a comment.
     lease.setContext(Element::fromJSON("{ \"comment\": \"a comment\" }"));
     l = lease.toElement();
     EXPECT_FALSE(l->contains("hw-address"));
-    EXPECT_FALSE(l->contains("user-context"));
-    ASSERT_TRUE(l->contains("comment"));
-    EXPECT_EQ("a comment", l->get("comment")->stringValue());
+    ConstElementPtr ctx = l->get("user-context");
+    ASSERT_TRUE(ctx);
+    ASSERT_EQ(Element::map, ctx->getType());
+    EXPECT_EQ(1, ctx->size());
+    ASSERT_TRUE(ctx->contains("comment"));
+    EXPECT_EQ("a comment", ctx->get("comment")->stringValue());
 }
 
 // Verify that the IA_NA can be created from JSON.
@@ -1220,7 +1230,7 @@ TEST(Lease6Test, fromElementNA) {
         "\"duid\": \"00:01:02:03:04:05:06:0a:0b:0c:0d:0e:0f\","
         "\"fqdn-fwd\": false,"
         "\"fqdn-rev\": false,"
-        "\"hostname\": \"urania.example.org\","
+        "\"hostname\": \"urania.EXAMPLE.org\","
         "\"hw-address\": \"08:00:2b:02:3f:4e\","
         "\"iaid\": 123456,"
         "\"ip-address\": \"2001:db8::1\","
@@ -1242,7 +1252,9 @@ TEST(Lease6Test, fromElementNA) {
     ASSERT_TRUE(lease->hwaddr_);
     EXPECT_EQ("hwtype=1 08:00:2b:02:3f:4e", lease->hwaddr_->toText());
     EXPECT_EQ(12345678, lease->cltt_);
+    EXPECT_EQ(lease->cltt_, lease->current_cltt_);
     EXPECT_EQ(800, lease->valid_lft_);
+    EXPECT_EQ(lease->valid_lft_, lease->current_valid_lft_);
     EXPECT_FALSE(lease->fqdn_fwd_);
     EXPECT_FALSE(lease->fqdn_rev_);
     EXPECT_EQ("urania.example.org", lease->hostname_);
@@ -1266,7 +1278,7 @@ TEST(Lease6Test, fromElementPD) {
         "\"duid\": \"00:01:02:03:04:05:06:0a:0b:0c:0d:0e:0f\","
         "\"fqdn-fwd\": false,"
         "\"fqdn-rev\": false,"
-        "\"hostname\": \"urania.example.org\","
+        "\"hostname\": \"uraniA.exaMple.orG\","
         "\"hw-address\": \"08:00:2b:02:3f:4e\","
         "\"iaid\": 123456,"
         "\"ip-address\": \"3000::\","
@@ -1288,7 +1300,9 @@ TEST(Lease6Test, fromElementPD) {
     ASSERT_TRUE(lease->hwaddr_);
     EXPECT_EQ("hwtype=1 08:00:2b:02:3f:4e", lease->hwaddr_->toText());
     EXPECT_EQ(12345678, lease->cltt_);
+    EXPECT_EQ(lease->cltt_, lease->current_cltt_);
     EXPECT_EQ(600, lease->valid_lft_);
+    EXPECT_EQ(lease->valid_lft_, lease->current_valid_lft_);
     EXPECT_FALSE(lease->fqdn_fwd_);
     EXPECT_FALSE(lease->fqdn_rev_);
     EXPECT_EQ("urania.example.org", lease->hostname_);
@@ -1349,6 +1363,7 @@ TEST(Lease6Test, fromElementInvalidValues) {
     testInvalidElement<Lease6>(json, "state", 1234, false);
     testInvalidElement<Lease6>(json, "subnet-id", std::string("xyz"));
     testInvalidElement<Lease6>(json, "subnet-id", -5, false);
+    testInvalidElement<Lease6>(json, "subnet-id", 0x100000000, false);
     testInvalidElement<Lease6>(json, "type", std::string("IA_XY"));
     testInvalidElement<Lease6>(json, "type", -3, false);
     testInvalidElement<Lease6>(json, "valid-lft", std::string("xyz"));
