@@ -1,4 +1,4 @@
-// Copyright (C) 2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2019-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -18,7 +18,7 @@ MySqlGenericBackendTest::MySqlGenericBackendTest()
 }
 
 size_t
-MySqlGenericBackendTest::countRows(MySqlConnection& conn, const std::string& table) const {
+MySqlGenericBackendTest::countRows(MySqlConnection& conn, const std::string& table) {
     // Execute a simple select query on all rows.
     std::string query = "SELECT * FROM " + table;
     auto status = mysql_query(conn.mysql_, query.c_str());
@@ -27,9 +27,11 @@ MySqlGenericBackendTest::countRows(MySqlConnection& conn, const std::string& tab
         return (0);
     }
 
-    // Get the number of rows returned and free the result.
-    MYSQL_RES * res = mysql_store_result(conn.mysql_);
+    // Get the number of rows returned.
+    MYSQL_RES* res = mysql_store_result(conn.mysql_);
     unsigned numrows = static_cast<unsigned>(mysql_num_rows(res));
+
+    // Free the result allocated.
     mysql_free_result(res);
 
     return (numrows);
