@@ -1,9 +1,11 @@
-// Copyright (C) 2012-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+#include <config.h>
+
 #include <dns/master_loader_callbacks.h>
 #include <dns/master_loader.h>
 #include <dns/rrtype.h>
@@ -15,10 +17,10 @@
 
 #include <gtest/gtest.h>
 
-#include <boost/bind.hpp>
 #include <boost/lexical_cast.hpp>
 #include <boost/scoped_ptr.hpp>
 
+#include <functional>
 #include <string>
 #include <vector>
 #include <list>
@@ -31,15 +33,16 @@ using std::list;
 using std::stringstream;
 using std::endl;
 using boost::lexical_cast;
+namespace ph = std::placeholders;
 
 namespace {
 class MasterLoaderTest : public ::testing::Test {
 public:
     MasterLoaderTest() :
-        callbacks_(boost::bind(&MasterLoaderTest::callback, this,
-                               &errors_, _1, _2, _3),
-                   boost::bind(&MasterLoaderTest::callback, this,
-                               &warnings_, _1, _2, _3))
+        callbacks_(std::bind(&MasterLoaderTest::callback, this,
+                             &errors_, ph::_1, ph::_2, ph::_3),
+                   std::bind(&MasterLoaderTest::callback, this,
+                             &warnings_, ph::_1, ph::_2, ph::_3))
     {}
 
     void TearDown() {
@@ -69,8 +72,9 @@ public:
                    const RRClass& rrclass, const MasterLoader::Options options)
     {
         loader_.reset(new MasterLoader(file, origin, rrclass, callbacks_,
-                                       boost::bind(&MasterLoaderTest::addRRset,
-                                                   this, _1, _2, _3, _4, _5),
+                                       std::bind(&MasterLoaderTest::addRRset,
+                                                 this, ph::_1, ph::_2, ph::_3,
+                                                 ph::_4, ph::_5),
                                        options));
     }
 
@@ -78,8 +82,9 @@ public:
                    const RRClass& rrclass, const MasterLoader::Options options)
     {
         loader_.reset(new MasterLoader(stream, origin, rrclass, callbacks_,
-                                       boost::bind(&MasterLoaderTest::addRRset,
-                                                   this, _1, _2, _3, _4, _5),
+                                       std::bind(&MasterLoaderTest::addRRset,
+                                                 this, ph::_1, ph::_2, ph::_3,
+                                                 ph::_4, ph::_5),
                                        options));
     }
 
@@ -146,7 +151,7 @@ TEST_F(MasterLoaderTest, basicLoad) {
     setLoader(TEST_DATA_SRCDIR "/example.org", Name("example.org."),
               RRClass::IN(), MasterLoader::MANY_ERRORS);
 
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
 
     // The following three should be set to 0 initially in case the loader
     // is constructed from a file name.
@@ -154,7 +159,7 @@ TEST_F(MasterLoaderTest, basicLoad) {
     EXPECT_EQ(0, loader_->getPosition());
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
 
     EXPECT_TRUE(errors_.empty());
     EXPECT_TRUE(warnings_.empty());
@@ -191,7 +196,7 @@ TEST_F(MasterLoaderTest, include) {
                   MasterLoader::MANY_ERRORS);
 
         loader_->load();
-        EXPECT_TRUE(loader_->loadedSucessfully());
+        EXPECT_TRUE(loader_->loadedSuccessfully());
         EXPECT_TRUE(errors_.empty());
         EXPECT_TRUE(warnings_.empty());
 
@@ -286,7 +291,7 @@ TEST_F(MasterLoaderTest, origin) {
                   MasterLoader::MANY_ERRORS);
 
         loader_->load();
-        EXPECT_TRUE(loader_->loadedSucessfully());
+        EXPECT_TRUE(loader_->loadedSuccessfully());
         EXPECT_TRUE(errors_.empty());
         // There's a relative origin in it, we warn about that.
         EXPECT_EQ(1, warnings_.size());
@@ -326,7 +331,7 @@ TEST_F(MasterLoaderTest, generate) {
                   MasterLoader::MANY_ERRORS);
 
         loader_->load();
-        EXPECT_TRUE(loader_->loadedSucessfully());
+        EXPECT_TRUE(loader_->loadedSuccessfully());
         EXPECT_TRUE(errors_.empty());
 
         // The "before" and "after" scaffolding below checks that no
@@ -349,7 +354,7 @@ TEST_F(MasterLoaderTest, generateRelativeLHS) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
 
     checkRR("example.org", RRType::NS(), "ns1.example.org.");
@@ -366,7 +371,7 @@ TEST_F(MasterLoaderTest, generateInFront) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
 
     checkRR("9host.example.org", RRType::TXT(), "9 pomegranate");
@@ -383,7 +388,7 @@ TEST_F(MasterLoaderTest, generateInMiddle) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
 
     checkRR("num9-host.example.org", RRType::TXT(), "This is 9 pomegranate");
@@ -400,7 +405,7 @@ TEST_F(MasterLoaderTest, generateAtEnd) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
 
     checkRR("num9-host.example.org", RRType::TXT(), "Pomegranate9");
@@ -416,7 +421,7 @@ TEST_F(MasterLoaderTest, generateStripsQuotes) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
 
     checkRR("example.org", RRType::MX(), "1 mx1.example.org.");
@@ -432,7 +437,7 @@ TEST_F(MasterLoaderTest, generateWithDoublePlaceholder) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
 
     checkRR("host9.example.org", RRType::TXT(), "This is $ pomegranate");
@@ -448,7 +453,7 @@ TEST_F(MasterLoaderTest, generateWithEscape) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
 
     checkRR("host9.example.org", RRType::TXT(), "This is \\$\\pomegranate");
@@ -469,7 +474,7 @@ TEST_F(MasterLoaderTest, generateWithParams) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
 
     checkRR("host2.example.org", RRType::A(), "192.0.2.2");
@@ -499,7 +504,7 @@ TEST_F(MasterLoaderTest, generateWithStep) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
 
     checkRR("host2.example.org", RRType::A(), "192.0.2.2");
@@ -558,7 +563,7 @@ TEST_F(MasterLoaderTest, generateWithModifiers) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
 
     checkRR("host3.example.org", RRType::A(), "192.0.2.1");
@@ -609,7 +614,7 @@ TEST_F(MasterLoaderTest, generateWithNoModifiers) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     ASSERT_EQ(2, errors_.size()); // For the broken GENERATE
     EXPECT_TRUE(warnings_.empty());
 
@@ -629,7 +634,7 @@ TEST_F(MasterLoaderTest, generateWithBadModifiers) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     ASSERT_EQ(2, errors_.size()); // For the broken GENERATE
     EXPECT_TRUE(warnings_.empty());
 
@@ -648,7 +653,7 @@ TEST_F(MasterLoaderTest, generateMissingRange) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_EQ(1, errors_.size()); // For the broken GENERATE
     EXPECT_TRUE(warnings_.empty());
 
@@ -665,7 +670,7 @@ TEST_F(MasterLoaderTest, generateMissingLHS) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_EQ(1, errors_.size()); // For the broken GENERATE
     EXPECT_TRUE(warnings_.empty());
 
@@ -682,7 +687,7 @@ TEST_F(MasterLoaderTest, generateMissingType) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_EQ(1, errors_.size()); // For the broken GENERATE
     EXPECT_TRUE(warnings_.empty());
 
@@ -699,7 +704,7 @@ TEST_F(MasterLoaderTest, generateMissingRHS) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_EQ(1, errors_.size()); // For the broken GENERATE
     EXPECT_TRUE(warnings_.empty());
 
@@ -716,7 +721,7 @@ TEST_F(MasterLoaderTest, generateWithBadRangeSyntax) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_EQ(1, errors_.size()); // For the broken GENERATE
     EXPECT_TRUE(warnings_.empty());
 
@@ -734,7 +739,7 @@ TEST_F(MasterLoaderTest, generateWithInvalidRange) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_EQ(1, errors_.size()); // For the broken GENERATE
     EXPECT_TRUE(warnings_.empty());
 
@@ -751,7 +756,7 @@ TEST_F(MasterLoaderTest, generateWithInvalidClass) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_EQ(1, errors_.size()); // For the broken GENERATE
     EXPECT_TRUE(warnings_.empty());
 
@@ -768,7 +773,7 @@ TEST_F(MasterLoaderTest, generateWithNoAvailableTTL) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_EQ(1, errors_.size()); // For the broken GENERATE
     EXPECT_TRUE(warnings_.empty());
 
@@ -787,7 +792,7 @@ TEST_F(MasterLoaderTest, popAfterError) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_EQ(1, errors_.size()); // For the broken RR
     EXPECT_EQ(1, warnings_.size()); // For missing EOLN
 
@@ -803,7 +808,7 @@ TEST_F(MasterLoaderTest, streamConstructor) {
     setLoader(zone_stream, Name("example.org."), RRClass::IN(),
               MasterLoader::MANY_ERRORS);
 
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
 
     // Unlike the basicLoad test, if we construct the loader from a stream
     // getSize() returns the data size in the stream immediately after the
@@ -812,7 +817,7 @@ TEST_F(MasterLoaderTest, streamConstructor) {
     EXPECT_EQ(0, loader_->getPosition());
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
 
     EXPECT_TRUE(errors_.empty());
     EXPECT_TRUE(warnings_.empty());
@@ -831,9 +836,9 @@ TEST_F(MasterLoaderTest, incrementalLoad) {
     setLoader(TEST_DATA_SRCDIR "/example.org", Name("example.org."),
               RRClass::IN(), MasterLoader::MANY_ERRORS);
 
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_FALSE(loader_->loadIncremental(2));
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
 
     EXPECT_TRUE(errors_.empty());
     EXPECT_TRUE(warnings_.empty());
@@ -848,7 +853,7 @@ TEST_F(MasterLoaderTest, incrementalLoad) {
 
     // Load the rest.
     EXPECT_TRUE(loader_->loadIncremental(20));
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
 
     EXPECT_TRUE(errors_.empty());
     EXPECT_TRUE(warnings_.empty());
@@ -861,7 +866,7 @@ TEST_F(MasterLoaderTest, incrementalLoad) {
 // saying so.
 TEST_F(MasterLoaderTest, invalidFile) {
     setLoader("This file doesn't exist at all",
-              Name("exmaple.org."), RRClass::IN(), MasterLoader::MANY_ERRORS);
+              Name("example.org."), RRClass::IN(), MasterLoader::MANY_ERRORS);
 
     // Nothing yet. The loader is dormant until invoked.
     // Is it really what we want?
@@ -941,7 +946,7 @@ struct ErrorCase {
     { "$INCLUDES " TEST_DATA_SRCDIR "/example.org",
         "Unknown directive 'INCLUDES'", "Include too long" },
     { "$INCLUDE", "unexpected end of input", "Missing include path" },
-    // The following two error messages are system dependant, omitting
+    // The following two error messages are system dependent, omitting
     { "$INCLUDE /file/not/found", NULL, "Include file not found" },
     { "$INCLUDE /file/not/found example.org. and here goes bunch of garbage",
         NULL, "Include file not found and garbage at the end of line" },
@@ -977,9 +982,9 @@ TEST_F(MasterLoaderTest, brokenZone) {
             stringstream zone_stream(zone);
             setLoader(zone_stream, Name("example.org."), RRClass::IN(),
                       MasterLoader::DEFAULT);
-            EXPECT_FALSE(loader_->loadedSucessfully());
+            EXPECT_FALSE(loader_->loadedSuccessfully());
             EXPECT_THROW(loader_->load(), MasterLoaderError);
-            EXPECT_FALSE(loader_->loadedSucessfully());
+            EXPECT_FALSE(loader_->loadedSuccessfully());
             EXPECT_EQ(1, errors_.size());
             if (ec->reason != NULL) {
                 checkCallbackMessage(errors_.at(0), ec->reason, 2);
@@ -999,9 +1004,9 @@ TEST_F(MasterLoaderTest, brokenZone) {
             stringstream zone_stream(zone);
             setLoader(zone_stream, Name("example.org."), RRClass::IN(),
                       MasterLoader::MANY_ERRORS);
-            EXPECT_FALSE(loader_->loadedSucessfully());
+            EXPECT_FALSE(loader_->loadedSuccessfully());
             EXPECT_NO_THROW(loader_->load());
-            EXPECT_FALSE(loader_->loadedSucessfully());
+            EXPECT_FALSE(loader_->loadedSuccessfully());
             EXPECT_EQ(1, errors_.size());
             EXPECT_TRUE(warnings_.empty());
             checkRR("example.org", RRType::SOA(), "ns1.example.org. "
@@ -1019,9 +1024,9 @@ TEST_F(MasterLoaderTest, brokenZone) {
             stringstream zone_stream(zoneEOF);
             setLoader(zone_stream, Name("example.org."), RRClass::IN(),
                       MasterLoader::MANY_ERRORS);
-            EXPECT_FALSE(loader_->loadedSucessfully());
+            EXPECT_FALSE(loader_->loadedSuccessfully());
             EXPECT_NO_THROW(loader_->load());
-            EXPECT_FALSE(loader_->loadedSucessfully());
+            EXPECT_FALSE(loader_->loadedSuccessfully());
             EXPECT_EQ(1, errors_.size()) << errors_[0] << "\n" << errors_[1];
             // The unexpected EOF warning
             EXPECT_EQ(1, warnings_.size());
@@ -1045,7 +1050,7 @@ TEST_F(MasterLoaderTest, includeWithGarbage) {
               MasterLoader::MANY_ERRORS);
 
     EXPECT_NO_THROW(loader_->load());
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     ASSERT_EQ(1, errors_.size());
     checkCallbackMessage(errors_.at(0), "Extra tokens at the end of line", 1);
     // It says something about extra tokens at the end
@@ -1064,7 +1069,7 @@ TEST_F(MasterLoaderTest, originWithGarbage) {
     setLoader(ss, Name("example.org."), RRClass::IN(),
               MasterLoader::MANY_ERRORS);
     EXPECT_NO_THROW(loader_->load());
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     ASSERT_EQ(1, errors_.size());
     checkCallbackMessage(errors_.at(0), "Extra tokens at the end of line", 1);
     EXPECT_TRUE(warnings_.empty());
@@ -1086,7 +1091,7 @@ TEST_F(MasterLoaderTest, includeAndOrigin) {
               MasterLoader::MANY_ERRORS);
     // Successfully load the data
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
     EXPECT_TRUE(warnings_.empty());
     // And check it's the correct data
@@ -1106,7 +1111,7 @@ TEST_F(MasterLoaderTest, includeAndBadOrigin) {
     setLoader(ss, Name("example.org"), RRClass::IN(),
               MasterLoader::MANY_ERRORS);
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_EQ(1, errors_.size());
     checkCallbackMessage(errors_.at(0), "duplicate period in example..org.",
                          1);
@@ -1125,7 +1130,7 @@ TEST_F(MasterLoaderTest, includeOriginRestore) {
               MasterLoader::MANY_ERRORS);
     // Successfully load the data
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
     EXPECT_TRUE(warnings_.empty());
     // And check it's the correct data
@@ -1144,7 +1149,7 @@ TEST_F(MasterLoaderTest, includeAndInitialWS) {
               MasterLoader::MANY_ERRORS);
     // Successfully load the data
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
     EXPECT_EQ(1, warnings_.size());
     checkCallbackMessage(warnings_.at(0),
@@ -1173,7 +1178,7 @@ TEST_F(MasterLoaderTest, ttlDirective) {
     setLoader(zone_stream, Name("example.org."), RRClass::IN(),
               MasterLoader::DEFAULT);
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     checkRR("example.org", RRType::A(), "192.0.2.1", RRTTL(1800));
     checkRR("a.example.org", RRType::A(), "192.0.2.2", RRTTL(100));
     checkRR("b.example.org", RRType::A(), "192.0.2.3", RRTTL(3600));
@@ -1190,7 +1195,7 @@ TEST_F(MasterLoaderTest, ttlFromSOA) {
     setLoader(zone_stream, Name("example.org."), RRClass::IN(),
               MasterLoader::DEFAULT);
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     checkRR("example.org", RRType::SOA(), ". . 0 0 0 0 1800", RRTTL(1800));
     checkRR("a.example.org", RRType::A(), "192.0.2.1", RRTTL(1800));
 
@@ -1209,7 +1214,7 @@ TEST_F(MasterLoaderTest, ttlFromPrevious) {
     setLoader(zone_stream, Name("example.org."), RRClass::IN(),
               MasterLoader::DEFAULT);
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     checkRR("a.example.org", RRType::A(), "192.0.2.1", RRTTL(1800));
     checkRR("b.example.org", RRType::A(), "192.0.2.2", RRTTL(1800));
     checkRR("c.example.org", RRType::A(), "192.0.2.3", RRTTL(1800));
@@ -1237,7 +1242,7 @@ TEST_F(MasterLoaderTest, RRParamsOrdering) {
     setLoader(zone_stream, Name("example.org."), RRClass::IN(),
               MasterLoader::DEFAULT);
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     checkRR("a.example.org", RRType::A(), "192.0.2.1", RRTTL(1800));
     checkRR("b.example.org", RRType::A(), "192.0.2.2", RRTTL(1800));
     checkRR("c.example.org", RRType::A(), "192.0.2.3", RRTTL(3600));
@@ -1257,7 +1262,7 @@ TEST_F(MasterLoaderTest, ttlFromPreviousSOA) {
     setLoader(zone_stream, Name("example.org."), RRClass::IN(),
               MasterLoader::DEFAULT);
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
 
     checkRR("example.org", RRType::SOA(), ". . 0 0 0 0 1800", RRTTL(100));
     checkRR("a.example.org", RRType::A(), "192.0.2.1", RRTTL(100));
@@ -1280,7 +1285,7 @@ TEST_F(MasterLoaderTest, ttlUnknownAndContinue) {
     setLoader(zone_stream, Name("example.org."), RRClass::IN(),
               MasterLoader::MANY_ERRORS);
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     checkRR("b.example.org", RRType::A(), "192.0.2.2", RRTTL(1800));
 
     EXPECT_TRUE(warnings_.empty());
@@ -1295,7 +1300,7 @@ TEST_F(MasterLoaderTest, ttlUnknownAndEOF) {
     setLoader(zone_stream, Name("example.org."), RRClass::IN(),
               MasterLoader::MANY_ERRORS);
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_TRUE(rrsets_.empty());
 
     EXPECT_EQ(1, errors_.size());
@@ -1312,14 +1317,15 @@ TEST_F(MasterLoaderTest, ttlOverflow) {
     stringstream zone_stream;
     zone_stream << "example.org. IN SOA . . 0 0 0 0 2147483648\n";
     zone_stream << "$TTL 3600\n"; // reset to an in-range value
-    zone_stream << "$TTL 2147483649\n" << "a.example.org. IN A 192.0.2.1\n";
+    zone_stream << "$TTL 2147483649\n";
+    zone_stream << "a.example.org. IN A 192.0.2.1\n";
     zone_stream << "$TTL 3600\n"; // reset to an in-range value
     zone_stream << "b.example.org. 2147483650 IN A 192.0.2.2\n";
     setLoader(zone_stream, Name("example.org."), RRClass::IN(),
               MasterLoader::DEFAULT);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_EQ(3, rrsets_.size());
 
     checkRR("example.org", RRType::SOA(), ". . 0 0 0 0 2147483648", RRTTL(0));
@@ -1380,7 +1386,7 @@ TEST_F(MasterLoaderTest, noEOLN) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
     // There should be one warning about the EOLN
     EXPECT_EQ(1, warnings_.size());
@@ -1396,7 +1402,7 @@ TEST_F(MasterLoaderTest, noPreviousName) {
     setLoader(ss, Name("example.org."), RRClass::IN(),
               MasterLoader::MANY_ERRORS);
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_EQ(1, errors_.size());
     checkCallbackMessage(errors_.at(0), "No previous name to use in place of "
                          "initial whitespace", 1);
@@ -1411,7 +1417,7 @@ TEST_F(MasterLoaderTest, previousInInclude) {
     setLoader(ss, Name("example.org"), RRClass::IN(),
               MasterLoader::MANY_ERRORS);
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
     // There should be one warning about the EOLN
     EXPECT_EQ(1, warnings_.size());
@@ -1429,7 +1435,7 @@ TEST_F(MasterLoaderTest, numericOwnerName) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
     EXPECT_TRUE(warnings_.empty());
 
