@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -453,7 +453,7 @@ protected:
 /// encapsulation
 ///
 /// This represents a reference to a given option similar to TokenOption
-/// but from within the information from a relay.  In the expresssion
+/// but from within the information from a relay.  In the expression
 /// relay6[nest-level].option[option-code], nest-level indicates which
 /// of the relays to examine and option-code which option to extract.
 ///
@@ -795,6 +795,50 @@ public:
     void evaluate(Pkt& pkt, ValueStack& values);
 };
 
+class TokenSplit : public Token {
+public:
+    /// @brief Constructor (does nothing)
+    TokenSplit() {}
+
+    /// @brief Extract a field from a delimited string
+    ///
+    /// Evaluation does not use packet information.  It requires at least
+    /// three values to be present on the stack.  It will consume the top
+    /// three values on the stack as parameters and push the resulting substring
+    /// onto the stack.  From the top it expects the values on the stack as:
+    /// -  field
+    /// -  delims
+    /// -  str
+    ///
+    /// str is the string to split.  If it is empty, an empty
+    /// string is pushed onto the value stack.
+    /// delims is string of character delimiters by which to split str. If it is
+    /// empty the entire value of str will be pushed on onto the value stack.
+    /// field is the field number (starting at 1) of the desired field.  If it is
+    /// out of range an empty string is pushed on the value stack.
+    ///
+    /// The following examples all use the base string "one.two..four" and shows
+    /// the value returned for a given field:
+    /// ```
+    /// field => value
+    /// --------------
+    /// -  0  => ""
+    /// -  1  => "one"
+    /// -  2  => "two"
+    /// -  3  => ""
+    /// -  4  => "four"
+    /// -  5  => ""
+    /// ```
+    ///
+    /// @throw EvalBadStack if there are less than 3 values on stack
+    /// @throw EvalTypeError if field is not a number
+    ///
+    /// @param pkt (unused)
+    /// @param values - stack of values (3 arguments will be popped, 1 result
+    ///        will be pushed)
+    void evaluate(Pkt& pkt, ValueStack& values);
+};
+
 /// @brief Token that represents concat operator (concatenates two other tokens)
 ///
 /// For example in the sub-expression "concat('foo','bar')" the result
@@ -1190,11 +1234,16 @@ protected:
 /// This class is derived from TokenOption and leverages its ability
 /// to operate on sub-options. It also adds additional capabilities.
 ///
+/// Note: @c TokenSubOption virtually derives @c TokenOption because both
+/// classes are inherited together in more complex classes in other parts of
+/// the code. This makes the base class @c TokenOption to exist only once in
+/// such complex classes.
+///
 /// It can represent the following expressions:
 /// option[149].exists - check if option 149 exists
 /// option[149].option[1].exists - check if suboption 1 exists in the option 149
 /// option[149].option[1].hex - return content of suboption 1 for option 149
-class TokenSubOption : public TokenOption {
+class TokenSubOption : public virtual TokenOption {
 public:
 
     /// @note Does not define its own representation type:
