@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -42,14 +42,14 @@ class LibraryManager;
 ///
 /// As described in the LibraryManager documentation, a CalloutHandle may end
 /// up with pointers to memory within the address space of a loaded library.
-/// If the library is unloaded before this address space is deleted, the
+/// If the library is closed before this address space is deleted, the
 /// deletion of the CalloutHandle may attempt to free memory into the newly-
 /// unmapped address space and cause a segmentation fault.
 ///
 /// To prevent this, each CalloutHandle maintains a shared pointer to the
 /// LibraryManagerCollection current when it was created.  In addition, the
-/// containing HooksManager object also maintains a shared pointer to it.  A
-/// a LibraryManagerCollection is never explicitly deleted: when a new set
+/// containing HooksManager object also maintains a shared pointer to it.
+/// A LibraryManagerCollection is never explicitly deleted: when a new set
 /// of libraries is loaded, the HooksManager clears its pointer to the
 /// collection.  The LibraryManagerCollection is only destroyed when all
 /// CallHandle objects referencing it are destroyed.
@@ -98,7 +98,7 @@ public:
     ///
     /// Returns a callout manager that can be used with this set of loaded
     /// libraries (even if the number of loaded libraries is zero).  This
-    /// method may only be caslled after loadLibraries() has been called.
+    /// method may only be called after loadLibraries() has been called.
     ///
     /// @return Pointer to a callout manager for this set of libraries.
     ///
@@ -139,11 +139,24 @@ public:
     ///
     /// @param libraries List of libraries to validate
     ///
-    /// @return Vector of libraries that faled to validate, or an empty vector
+    /// @return Vector of libraries that failed to validate, or an empty vector
     ///         if all validated.
     static std::vector<std::string>
     validateLibraries(const std::vector<std::string>& libraries);
 
+    /// @brief Prepare libaries unloading
+    ///
+    /// Utility function to call before closing libraries. It runs the
+    /// unload() function when it exists and removes associated callout.
+    /// When this function returns either there is only one owner
+    /// (the hook manager) or some visible dangling pointers so
+    /// libraries are not closed to lower the probability of a crash.
+    /// See @ref LibraryManager::prepareUnloadLibrary.
+    ///
+    /// @return true if all libraries unload were not found or run
+    /// successfully, false on an error.
+    bool prepareUnloadLibraries();
+
 protected:
     /// @brief Unload libraries
     ///
