@@ -1,4 +1,4 @@
-// Copyright (C) 2016-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -76,6 +76,9 @@ TEST_F(HttpRequestTest, minimal) {
     EXPECT_EQ("/isc/org", request_->getUri());
     EXPECT_EQ(1, request_->getHttpVersion().major_);
     EXPECT_EQ(1, request_->getHttpVersion().minor_);
+    EXPECT_TRUE(request_->getRemote().empty());
+    request_->setRemote("127.0.0.1");
+    EXPECT_EQ("127.0.0.1", request_->getRemote());
 
     EXPECT_THROW(request_->getHeaderValue("Content-Length"),
                  HttpMessageNonExistingHeader);
@@ -358,4 +361,62 @@ TEST_F(HttpRequestTest, toBriefString) {
     EXPECT_EQ("POST /isc/org HTTP/1.1", request_->toBriefString());
 }
 
+// This test verifies that no basic HTTP authentication is supported.
+TEST_F(HttpRequestTest, noBasicAuth) {
+    ASSERT_NO_THROW(request_.reset(new HttpRequest(HttpRequest::Method::HTTP_GET,
+                                                   "/isc/org",
+                                                   HttpVersion(1, 1),
+                                                   HostHttpHeader("www.example.org"))));
+
+    ASSERT_NO_THROW(request_->finalize());
+    ASSERT_THROW(request_->getHeader("Authorization"),
+                 HttpMessageNonExistingHeader);
+}
+
+// This test verifies that basic HTTP authentication works as expected.
+TEST_F(HttpRequestTest, basicAuth) {
+    BasicHttpAuthPtr basic_auth;
+    EXPECT_NO_THROW(basic_auth.reset(new BasicHttpAuth("foo", "bar")));
+    ASSERT_TRUE(basic_auth);
+
+    ASSERT_NO_THROW(request_.reset(new HttpRequest(HttpRequest::Method::HTTP_GET,
+                                                   "/isc/org",
+                                                   HttpVersion(1, 1),
+                                                   HostHttpHeader("www.example.org"),
+                                                   basic_auth)));
+
+    ASSERT_NO_THROW(request_->finalize());
+
+    std::string value;
+    EXPECT_NO_THROW(value = request_->getHeaderValue("Authorization"));
+    EXPECT_EQ(value, "Basic " + basic_auth->getCredential());
+}
+
+/// This test verifies that access parameters are handled as expected.
+TEST_F(HttpRequestTest, parameters) {
+    setContextBasics("GET", "/isc/org", HttpVersion(1, 1));
+    ASSERT_NO_THROW(request_->create());
+
+    EXPECT_TRUE(request_->getRemote().empty());
+    EXPECT_FALSE(request_->getTls());
+    EXPECT_TRUE(request_->getSubject().empty());
+    EXPECT_TRUE(request_->getIssuer().empty());
+    EXPECT_TRUE(request_->getBasicAuth().empty());
+    EXPECT_TRUE(request_->getCustom().empty());
+
+    request_->setRemote("my-remote");
+    request_->setTls(true);
+    request_->setSubject("my-subject");
+    request_->setIssuer("my-issuer");
+    request_->setBasicAuth("foo");
+    request_->setCustom("bar");
+
+    EXPECT_EQ("my-remote", request_->getRemote());
+    EXPECT_TRUE(request_->getTls());
+    EXPECT_EQ("my-subject", request_->getSubject());
+    EXPECT_EQ("my-issuer", request_->getIssuer());
+    EXPECT_EQ("foo", request_->getBasicAuth());
+    EXPECT_EQ("bar", request_->getCustom());
+}
+
 }
