@@ -1,4 +1,4 @@
-// Copyright (C) 2011-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -28,8 +28,8 @@
 #include <log/logger_specification.h>
 #include <log/message_initializer.h>
 #include <log/output_option.h>
-
 #include <log/tests/tempdir.h>
+#include <testutils/gtest_utils.h>
 
 #include <sys/types.h>
 #include <regex.h>
@@ -241,7 +241,7 @@ TEST_F(LoggerManagerTest, FileLogger) {
 
 // Check if the file rolls over when it gets above a certain size.
 TEST_F(LoggerManagerTest, FileSizeRollover) {
-    // Set to a suitable minimum that log4cplus can copy with
+    // Set to a suitable minimum that log4cplus can cope with.
     static const size_t SIZE_LIMIT = 204800;
 
     // Set up the name of the file.
@@ -254,7 +254,7 @@ TEST_F(LoggerManagerTest, FileSizeRollover) {
     opt->maxsize = SIZE_LIMIT;    // Bytes
     opt->maxver = 2;
 
-    // The current current output file does not exist (the creation of file_spec
+    // The current output file does not exist (the creation of file_spec
     // ensures that.  Check that previous versions don't either.
     vector<string> prev_name;
     for (int i = 0; i < 3; ++i) {
@@ -320,6 +320,29 @@ TEST_F(LoggerManagerTest, FileSizeRollover) {
     }
 }
 
+// Check if an exception is thrown if maxsize is too large.
+TEST_F(LoggerManagerTest, TooLargeMaxsize) {
+    // Set up the name of the file.
+    SpecificationForFileLogger file_spec;
+    LoggerSpecification& spec(file_spec.getSpecification());
+
+    // UINT64_MAX should be large enough.
+    LoggerSpecification::iterator opt = spec.begin();
+    EXPECT_TRUE(opt != spec.end());
+    opt->maxsize = std::numeric_limits<uint64_t>::max();  // bytes
+
+    // Set up the file logger.
+    LoggerManager manager;
+    EXPECT_THROW_MSG(manager.process(spec), BadValue,
+                     "expected maxsize < 2147483647MB, but instead got "
+                     "18446744073709MB");
+
+    opt->maxsize = 1000000LL * (std::numeric_limits<int32_t>::max() + 1LL);  // bytes
+    EXPECT_THROW_MSG(manager.process(spec), BadValue,
+                     "expected maxsize < 2147483647MB, but instead got "
+                     "2147483648MB");
+}
+
 namespace { // begin unnamed namespace
 
 // When we begin to use C++11, we could replace use of POSIX API with
