@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -9,10 +9,14 @@
 #include <exceptions/exceptions.h>
 #include <mysql/mysql_connection.h>
 #include <mysql/testutils/mysql_schema.h>
-#include <boost/date_time/posix_time/posix_time.hpp>
-#include <gtest/gtest.h>
+#include <testutils/gtest_utils.h>
+
 #include <array>
 
+#include <gtest/gtest.h>
+
+#include <boost/date_time/posix_time/posix_time.hpp>
+
 using namespace isc::db;
 using namespace isc::db::test;
 
@@ -185,8 +189,8 @@ public:
         ASSERT_FALSE(in_bindings[1]->amNull());
 
         // Store data in the database.
-        ASSERT_NO_THROW(conn_.insertQuery(MySqlConnectionTest::INSERT_VALUE,
-                                          in_bindings));
+        ASSERT_NO_THROW_LOG(conn_.insertQuery(MySqlConnectionTest::INSERT_VALUE,
+                                              in_bindings));
 
         // Create input binding for select query.
         MySqlBindingCollection bindings =
@@ -204,9 +208,9 @@ public:
 
         // Execute select statement. We expect one row to be returned. For this
         // returned row the lambda provided as 4th argument should be executed.
-        ASSERT_NO_THROW(conn_.selectQuery(MySqlConnectionTest::GET_BY_INT_VALUE,
-                                          bindings, out_bindings,
-                                          [&](MySqlBindingCollection& out_bindings) {
+        ASSERT_NO_THROW_LOG(conn_.selectQuery(MySqlConnectionTest::GET_BY_INT_VALUE,
+                                              bindings, out_bindings,
+                                              [&](MySqlBindingCollection& out_bindings) {
 
             // Compare received data with input data assuming they are both non-null.
 
@@ -436,7 +440,7 @@ public:
             MySqlBinding::createNull(),
         };
 
-        ASSERT_NO_THROW(
+        ASSERT_NO_THROW_LOG(
             conn_.insertQuery(MySqlConnectionTest::INSERT_VALUE, in_bindings));
 
         // This variable will be checked to see if the row has been deleted
@@ -446,14 +450,14 @@ public:
         // Execute delete query but use int_value of non existing row.
         // The row should not be deleted.
         in_bindings = {MySqlBinding::createInteger<uint32_t>(1)};
-        ASSERT_NO_THROW(
+        ASSERT_NO_THROW_LOG(
             deleted = conn_.updateDeleteQuery(
                 MySqlConnectionTest::DELETE_BY_INT_VALUE, in_bindings));
         ASSERT_FALSE(deleted);
 
         // This time use the correct value.
         in_bindings = {MySqlBinding::createInteger<uint32_t>(1024)};
-        ASSERT_NO_THROW(
+        ASSERT_NO_THROW_LOG(
             deleted = conn_.updateDeleteQuery(
                 MySqlConnectionTest::DELETE_BY_INT_VALUE, in_bindings));
         // The row should have been deleted.
@@ -469,15 +473,15 @@ public:
             MySqlBinding::createTimestamp(),
         };
 
-        ASSERT_NO_THROW(conn_.selectQuery(MySqlConnectionTest::GET_BY_INT_VALUE,
-                                          in_bindings, out_bindings,
-                                          [&deleted](MySqlBindingCollection&) {
-                                              // This will be executed if the
-                                              // row is returned as a result of
-                                              // select query. We expect that
-                                              // this is not executed.
-                                              deleted = false;
-                                          }));
+        ASSERT_NO_THROW_LOG(conn_.selectQuery(MySqlConnectionTest::GET_BY_INT_VALUE,
+                                              in_bindings, out_bindings,
+                                              [&deleted](MySqlBindingCollection&) {
+                                                  // This will be executed if the
+                                                  // row is returned as a result of
+                                                  // select query. We expect that
+                                                  // this is not executed.
+                                                  deleted = false;
+                                              }));
         // Make sure that select query returned nothing.
         EXPECT_TRUE(deleted);
     }
@@ -653,9 +657,133 @@ TEST_F(MySqlSchemaTest, checkVersion) {
     // Check version
     auto parameters = DatabaseConnection::parse(validMySQLConnectionString());
     std::pair<uint32_t, uint32_t> version;
-    ASSERT_NO_THROW(version = MySqlConnection::getVersion(parameters));
+    ASSERT_NO_THROW_LOG(version = MySqlConnection::getVersion(parameters));
     EXPECT_EQ(MYSQL_SCHEMA_VERSION_MAJOR, version.first);
     EXPECT_EQ(MYSQL_SCHEMA_VERSION_MINOR, version.second);
 }
 
+/// @brief Test fixture class for secure connection.
+class MySqlSecureConnectionTest : public ::testing::Test {
+public:
+
+    /// @brief Check if SSL/TLS support is available and configured.
+    bool hasMySQLTls() {
+        std::string tls = getMySQLTlsEnv();
+        if (tls.empty()) {
+            tls = getMySQLTlsServer();
+        }
+        return (tls == "YES");
+    }
+};
+
+/// @brief Check that we can get the MySQL support status.
+TEST_F(MySqlSecureConnectionTest, getMySQLTls) {
+    std::string env;
+    try {
+        env = getMySQLTlsEnv();
+        std::cout << "getMySQLTlsEnv returns '" << env << "'\n";
+    } catch (const isc::Exception& ex) {
+        std::cerr << "getMySQLTlsEnv fails with " << ex.what() << "\n";
+    }
+    if (!env.empty()) {
+        return;
+    }
+    try {
+        std::cout << "getMySQLTlsServer returns '" << getMySQLTlsServer() << "'\n";
+    } catch (const isc::Exception& ex) {
+        std::cerr << "getMySQLTlsServer fails with " << ex.what() << "\n";
+    }
+}
+
+/// @brief Check the enforced TCP connection.
+TEST_F(MySqlSecureConnectionTest, Tcp) {
+    std::string conn_str = connectionString(MYSQL_VALID_TYPE, VALID_NAME,
+                                            VALID_HOST_TCP, VALID_USER,
+                                            VALID_PASSWORD);
+    MySqlConnection conn(DatabaseConnection::parse(conn_str));
+    ASSERT_NO_THROW_LOG(conn.openDatabase());
+}
+
+/// @brief Check the SSL/TLS protected connection.
+TEST_F(MySqlSecureConnectionTest, Tls) {
+    if (!hasMySQLTls()) {
+        std::cout << "SSL/TLS support is not available or configured: "
+                  << "skipping this test\n";
+        return;
+    }
+    std::string conn_str = connectionString(MYSQL_VALID_TYPE, VALID_NAME,
+                                            VALID_HOST_TCP, VALID_SECURE_USER,
+                                            VALID_PASSWORD, 0, 0,
+                                            VALID_CERT, VALID_KEY, VALID_CA,
+                                            VALID_CIPHER);
+    MySqlConnection conn(DatabaseConnection::parse(conn_str));
+    ASSERT_NO_THROW_LOG(conn.openDatabase());
+    EXPECT_TRUE(conn.getTls());
+    std::string cipher = conn.getTlsCipher();
+    EXPECT_FALSE(cipher.empty());
+    std::cout << "TLS cipher is '" << cipher << "'\n";
+}
+
+/// @brief Check the SSL/TLS protected connection still requires the password.
+TEST_F(MySqlSecureConnectionTest, TlsInvalidPassword) {
+    if (!hasMySQLTls()) {
+        std::cout << "SSL/TLS support is not available or configured: "
+                  << "skipping this test\n";
+        return;
+    }
+    std::string conn_str = connectionString(MYSQL_VALID_TYPE, VALID_NAME,
+                                            VALID_HOST_TCP, VALID_SECURE_USER,
+                                            INVALID_PASSWORD, 0, 0,
+                                            VALID_CERT, VALID_KEY, VALID_CA,
+                                            VALID_CIPHER);
+    MySqlConnection conn(DatabaseConnection::parse(conn_str));
+    EXPECT_THROW(conn.openDatabase(), DbOpenError);
+}
+
+/// @brief Check the SSL/TLS protected connection requires crypto parameters.
+TEST_F(MySqlSecureConnectionTest, TlsNoCrypto) {
+    if (!hasMySQLTls()) {
+        std::cout << "SSL/TLS support is not available or configured: "
+                  << "skipping this test\n";
+        return;
+    }
+    std::string conn_str = connectionString(MYSQL_VALID_TYPE, VALID_NAME,
+                                            VALID_HOST_TCP, VALID_SECURE_USER,
+                                            VALID_PASSWORD);
+    MySqlConnection conn(DatabaseConnection::parse(conn_str));
+    EXPECT_THROW(conn.openDatabase(), DbOpenError);
+}
+
+/// @brief Check the SSL/TLS protected connection requires valid key.
+TEST_F(MySqlSecureConnectionTest, TlsInvalidKey) {
+    if (!hasMySQLTls()) {
+        std::cout << "SSL/TLS support is not available or configured: "
+                  << "skipping this test\n";
+        return;
+    }
+    std::string conn_str = connectionString(MYSQL_VALID_TYPE, VALID_NAME,
+                                            VALID_HOST_TCP, VALID_SECURE_USER,
+                                            VALID_PASSWORD, 0, 0,
+                                            VALID_CERT, INVALID_KEY, VALID_CA,
+                                            VALID_CIPHER);
+    MySqlConnection conn(DatabaseConnection::parse(conn_str));
+    EXPECT_THROW(conn.openDatabase(), DbOpenError);
+}
+
+/// @brief Check the SSL/TLS protected connection requires a key.
+TEST_F(MySqlSecureConnectionTest, TlsNoKey) {
+    if (!hasMySQLTls()) {
+        std::cout << "SSL/TLS support is not available or configured: "
+                  << "skipping this test\n";
+        return;
+    }
+    std::string conn_str = connectionString(MYSQL_VALID_TYPE, VALID_NAME,
+                                            VALID_HOST_TCP, VALID_SECURE_USER,
+                                            VALID_PASSWORD, 0, 0,
+                                            VALID_CERT, 0, VALID_CA,
+                                            VALID_CIPHER);
+    MySqlConnection conn(DatabaseConnection::parse(conn_str));
+    EXPECT_THROW(conn.openDatabase(), DbOpenError);
+}
+
 }  // namespace
