@@ -1,4 +1,4 @@
-// Copyright (C) 2016-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -8,6 +8,9 @@
 
 #include <pgsql/pgsql_connection.h>
 #include <pgsql/pgsql_exchange.h>
+#include <pgsql/testutils/pgsql_schema.h>
+#include <pgsql/tests/pgsql_basics.h>
+#include <testutils/gtest_utils.h>
 
 #include <boost/lexical_cast.hpp>
 
@@ -17,7 +20,12 @@
 #include <vector>
 
 using namespace isc;
+using namespace isc::data;
 using namespace isc::db;
+using namespace isc::db::test;
+using namespace isc::util;
+using namespace boost::posix_time;
+using namespace boost::gregorian;
 
 namespace {
 
@@ -71,6 +79,29 @@ TEST(PsqlBindArray, addDataTest) {
 
         // Add the vector
         b.add(bytes);
+
+        // Add an empty string.
+        b.add(std::string(""));
+
+        // Add a v4 address.
+        asiolink::IOAddress addr4("192.168.1.1");
+        b.addInet4(addr4);
+
+        // Add a v6 address.
+        asiolink::IOAddress addr6("3001::1");
+        b.addInet6(addr6);
+
+        // Add a double.  Not sure how portably reliable this test will be.
+        double dbl = 2.0;
+        b.add(dbl);
+
+        // Add a JSON.
+        ElementPtr elems = Element::fromJSON("{ \"foo\": \"bar\" }");
+        b.add(elems);
+
+        // Add a Temporary blob
+        std::vector<uint8_t> blob({0x0a,0x0b,0x0,0xc,0xd});
+        b.addTempBinary(blob);
     }
 
     // We've left bind scope, everything should be intact.
@@ -85,222 +116,208 @@ TEST(PsqlBindArray, addDataTest) {
         "7 : \"FALSE\"\n"
         "8 : \"3221360418\"\n"
         "9 : \"3001::1\"\n"
-        "10 : 0x0102030405060708090a\n";
+        "10 : 0x0102030405060708090a\n"
+        "11 : empty\n"
+        "12 : \"192.168.1.1\"\n"
+        "13 : \"3001::1\"\n"
+        "14 : \"2\"\n"
+        "15 : \"{ \"foo\": \"bar\" }\"\n"
+        "16 : 0x0a0b000c0d\n";
 
     EXPECT_EQ(expected, b.toText());
 }
 
-/// @brief Defines a pointer to a PgSqlConnection
-typedef boost::shared_ptr<PgSqlConnection> PgSqlConnectionPtr;
-/// @brief Defines a pointer to a PgSqlResult
-typedef boost::shared_ptr<PgSqlResult> PgSqlResultPtr;
+/// @brief Verifies the ability to add Triplets to
+/// the bind array.
+TEST(PsqlBindArray, addTriplet) {
 
-/// @brief Fixture for exercising basic PostgreSQL operations and data types
-///
-/// This class is intended to be used to verify basic operations and to
-/// verify that each PostgreSQL  data type currently used by Kea, can be
-/// correctly written to and read from PostgreSQL.  Rather than use tables
-/// that belong to Kea the schema proper,  it creates its own. Currently it
-/// consists of a single table, called "basics" which contains one column for
-/// each of the supported data types.
-///
-/// It creates the schema during construction, deletes it upon destruction, and
-/// provides functions for executing SQL statements, executing prepared
-/// statements, fetching all rows in the table, and deleting all the rows in
-/// the table.
-class PgSqlBasicsTest : public ::testing::Test {
-public:
-    /// @brief Column index for each column
-    enum BasicColIndex {
-        ID_COL,
-        BOOL_COL,
-        BYTEA_COL,
-        BIGINT_COL,
-        SMALLINT_COL,
-        INT_COL,
-        TEXT_COL,
-        TIMESTAMP_COL,
-        VARCHAR_COL,
-        NUM_BASIC_COLS
-    };
+    PsqlBindArray b;
 
-    /// @brief Constructor
-    ///
-    /// Creates the database connection, opens the database, and destroys
-    /// the table (if present) and then recreates it.
-    PgSqlBasicsTest() : expectedColNames_(NUM_BASIC_COLS) {
-        // Create database connection parameter list
-        DatabaseConnection::ParameterMap params;
-        params["name"] = "keatest";
-        params["user"] = "keatest";
-        params["password"] = "keatest";
+    // Add all the items within a different scope. Everything should
+    // still be valid once we exit this scope.
+    {
+        Triplet<uint32_t> empty;
+        Triplet<uint32_t> not_empty(1,2,3);
 
-        // Create and open the database connection
-        conn_.reset(new PgSqlConnection(params));
-        conn_->openDatabase();
-
-        // Create the list of expected column names
-        expectedColNames_[ID_COL] = "id";
-        expectedColNames_[BOOL_COL] = "bool_col";
-        expectedColNames_[BYTEA_COL] = "bytea_col";
-        expectedColNames_[BIGINT_COL] = "bigint_col";
-        expectedColNames_[SMALLINT_COL] = "smallint_col";
-        expectedColNames_[INT_COL] = "int_col";
-        expectedColNames_[TEXT_COL] = "text_col";
-        expectedColNames_[TIMESTAMP_COL] = "timestamp_col";
-        expectedColNames_[VARCHAR_COL] = "varchar_col";
-
-        destroySchema();
-        createSchema();
+        // Add triplets to the array.
+        b.add(empty);
+        b.add(not_empty);
+        b.addMin(empty);
+        b.addMin(not_empty);
+        b.addMax(empty);
+        b.addMax(not_empty);
     }
 
-    /// @brief Destructor
-    ///
-    /// Destroys the table. The database resources are freed and the connection
-    /// closed by the destruction of conn_.
-    virtual ~PgSqlBasicsTest () {
-        destroySchema();
+    // We've left bind scope, everything should be intact.
+    EXPECT_EQ(6, b.size());
+
+    // Verify contents are correct.
+    std::string expected =
+        "0 : empty\n"
+        "1 : \"2\"\n"
+        "2 : empty\n"
+        "3 : \"1\"\n"
+        "4 : empty\n"
+        "5 : \"3\"\n";
+
+    EXPECT_EQ(expected, b.toText());
+}
+
+/// @brief Verifies the ability to add Optional strings to
+/// the bind array.
+TEST(PsqlBindArray, addOptionalString) {
+
+    PsqlBindArray b;
+
+    // Add all the items within a different scope. Everything should
+    // still be valid once we exit this scope.
+    {
+        Optional<std::string> empty;
+        Optional<std::string> not_empty("whoopee!");
+
+        // Add strings to the array.
+        b.addOptional(empty);
+        b.addOptional(not_empty);
     }
 
-    /// @brief Gets the expected name of the column for a given column index
-    ///
-    /// Returns the name of column as we expect it to be when the column is
-    /// fetched from the database.
-    ///
-    /// @param col index of the desired column
-    ///
-    /// @return string containing the column name
-    ///
-    /// @throw BadValue if the index is out of range
-    const std::string& expectedColumnName(int col) {
-        if (col < 0 || col >= NUM_BASIC_COLS) {
-            isc_throw(BadValue,
-                      "definedColumnName: invalid column value" << col);
-        }
+    // We've left bind scope, everything should be intact.
+    EXPECT_EQ(2, b.size());
 
-        return (expectedColNames_[col]);
+    // Verify contents are correct.
+    std::string expected =
+        "0 : empty\n"
+        "1 : \"whoopee!\"\n";
+
+    EXPECT_EQ(expected, b.toText());
+}
+
+/// @brief Verifies the ability to add Optional booleans to
+/// the bind array.
+TEST(PsqlBindArray, addOptionalBool) {
+
+    PsqlBindArray b;
+
+    // Add all the items within a different scope. Everything should
+    // still be valid once we exit this scope.
+    {
+        Optional<bool> empty;
+        Optional<bool> am_false(false);
+        Optional<bool> am_true(true);
+
+        // Add booleans to the array.
+        b.addOptional(empty);
+        b.addOptional(am_false);
+        b.addOptional(am_true);
     }
 
-    /// @brief Creates the basics table
-    /// Asserts if the creation step fails
-    void createSchema() {
-        // One column for OID type, plus an auto-increment
-        const char* sql =
-            "CREATE TABLE basics ( "
-            "    id SERIAL PRIMARY KEY NOT NULL, "
-            "    bool_col BOOLEAN, "
-            "    bytea_col BYTEA, "
-            "    bigint_col  BIGINT, "
-            "    smallint_col  SMALLINT, "
-            "    int_col INT, "
-            "    text_col TEXT, "
-            "    timestamp_col TIMESTAMP WITH TIME ZONE, "
-            "    varchar_col VARCHAR(255) "
-            "); ";
+    // We've left bind scope, everything should be intact.
+    EXPECT_EQ(3, b.size());
 
-        PgSqlResult r(PQexec(*conn_, sql));
-        ASSERT_EQ(PQresultStatus(r), PGRES_COMMAND_OK)
-                 << " create basics table failed: " << PQerrorMessage(*conn_);
+    // Verify contents are correct.
+    std::string expected =
+        "0 : empty\n"
+        "1 : \"0\"\n"
+        "2 : \"1\"\n";
+
+    EXPECT_EQ(expected, b.toText());
+}
+
+
+/// @brief Verifies the ability to add OptionalIntegers to
+/// the bind array.
+TEST(PsqlBindArray, addOptionalInteger) {
+
+    PsqlBindArray b;
+
+    // Add all the items within a different scope. Everything should
+    // still be valid once we exit this scope.
+    {
+        Optional<uint32_t> empty;
+        Optional<uint32_t> not_empty(123);
+
+        // Add the integers to the array..
+        b.addOptional(empty);
+        b.addOptional(not_empty);
     }
 
-    /// @brief Destroys the basics table
-    /// Asserts if the destruction fails
-    void destroySchema() {
-        if (conn_) {
-            PgSqlResult r(PQexec(*conn_, "DROP TABLE IF EXISTS basics;"));
-            ASSERT_EQ(PQresultStatus(r), PGRES_COMMAND_OK)
-                 << " drop basics table failed: " << PQerrorMessage(*conn_);
-        }
+    // We've left bind scope, everything should be intact.
+    EXPECT_EQ(2, b.size());
+
+    // Verify contents are correct.
+    std::string expected =
+        "0 : empty\n"
+        "1 : \"123\"\n";
+
+    EXPECT_EQ(expected, b.toText());
+}
+
+/// @brief Verifies the ability to add Optional IPv4 addresses to
+/// the bind array.
+TEST(PsqlBindArray, addOptionalInet4) {
+
+    PsqlBindArray b;
+
+    // Add all the items within a different scope. Everything should
+    // still be valid once we exit this scope.
+    {
+        Optional<asiolink::IOAddress> empty;
+        Optional<asiolink::IOAddress> not_empty(asiolink::IOAddress("192.16.1.1"));
+
+        // Verify we cannot add a v6 address.
+        Optional<asiolink::IOAddress> not_v4(asiolink::IOAddress("3001::1"));
+        ASSERT_THROW_MSG(b.addOptionalInet4(not_v4), BadValue,
+                         "unable to add address to PsqlBindAray"
+                         " '3001::1' is not an IPv4 address");
+
+        // Add addresses to bind array.
+        b.addOptionalInet4(empty);
+        b.addOptionalInet4(not_empty);
     }
 
-    /// @brief Executes a SQL statement and tests for an expected outcome
-    ///
-    /// @param r pointer which will contain the result set returned by the
-    /// statement's execution.
-    /// @param sql string containing the SQL statement text.  Note that
-    /// PostgreSQL supports executing text which contains more than one SQL
-    /// statement separated by semicolons.
-    /// @param exp_outcome expected status value returned with within the
-    /// result set such as PGRES_COMMAND_OK, PGRES_TUPLES_OK.
-    /// @lineno line number from where the call was invoked
-    ///
-    /// Asserts if the result set status does not equal the expected outcome.
-    void runSql(PgSqlResultPtr& r, const std::string& sql, int exp_outcome,
-                int lineno) {
-        r.reset(new PgSqlResult(PQexec(*conn_, sql.c_str())));
-        ASSERT_EQ(PQresultStatus(*r), exp_outcome)
-                  << " runSql at line: " << lineno << " failed, sql:[" << sql
-                  << "]\n reason: " << PQerrorMessage(*conn_);
+    // We've left bind scope, everything should be intact.
+    EXPECT_EQ(2, b.size());
+
+    // Verify contents are correct.
+    std::string expected =
+        "0 : empty\n"
+        "1 : \"192.16.1.1\"\n";
+
+    EXPECT_EQ(expected, b.toText());
+}
+
+/// @brief Verifies the ability to add optional IPv6 addresses to
+/// the bind array.
+TEST(PsqlBindArray, addOptionalInet6) {
+
+    PsqlBindArray b;
+
+    // Add all the items within a different scope. Everything should
+    // still be valid once we exit this scope.
+    {
+        Optional<asiolink::IOAddress> empty;
+        Optional<asiolink::IOAddress> not_empty(asiolink::IOAddress("3001::1"));
+
+        // Verify we cannot add a v6 address.
+        Optional<asiolink::IOAddress> not_v6(asiolink::IOAddress("192.168.1.1"));
+        ASSERT_THROW_MSG(b.addOptionalInet6(not_v6), BadValue,
+                         "unable to add address to PsqlBindAray"
+                         " '192.168.1.1' is not an IPv6 address");
+
+        // Add addresses to bind array.
+        b.addOptionalInet6(empty);
+        b.addOptionalInet6(not_empty);
     }
 
-    /// @brief Executes a SQL statement and tests for an expected outcome
-    ///
-    /// @param r pointer which will contain the result set returned by the
-    /// statement's execution.
-    /// @param statement statement descriptor of the prepared statement
-    /// to execute.
-    /// @param bind_array bind array containing the input values to submit
-    /// along with the statement
-    /// @param exp_outcome expected status value returned with within the
-    /// result set such as PGRES_COMMAND_OK, PGRES_TUPLES_OK.
-    /// @lineno line number from where the call was invoked
-    ///
-    /// Asserts if the result set status does not equal the expected outcome.
-    void runPreparedStatement(PgSqlResultPtr& r,
-                              PgSqlTaggedStatement& statement,
-                              PsqlBindArrayPtr bind_array, int exp_outcome,
-                              int lineno) {
-        r.reset(new PgSqlResult(PQexecPrepared(*conn_, statement.name,
-                                statement.nbparams,
-                                &bind_array->values_[0],
-                                &bind_array->lengths_[0],
-                                &bind_array->formats_[0], 0)));
-        ASSERT_EQ(PQresultStatus(*r), exp_outcome)
-                  << " runPreparedStatement at line: " << lineno
-                  << " statement name:[" << statement.name
-                  << "]\n reason: " << PQerrorMessage(*conn_);
-    }
+    // We've left bind scope, everything should be intact.
+    EXPECT_EQ(2, b.size());
 
-    /// @brief Fetches all of the rows currently in the table
-    ///
-    /// Executes a select statement which returns all of the rows in the
-    /// basics table, in their order of insertion.   Each row contains all
-    /// of the defined columns, in the order they are defined.
-    ///
-    /// @param r pointer which will contain the result set returned by the
-    /// statement's execution.
-    /// @param exp_rows expected number of rows fetched. (This can be 0).
-    /// @lineno line number from where the call was invoked
-    ///
-    /// Asserts if the result set status does not equal the expected outcome.
-    void fetchRows(PgSqlResultPtr& r, int exp_rows, int line) {
-        std::string sql =
-            "SELECT"
-            "   id, bool_col, bytea_col, bigint_col, smallint_col, "
-            "   int_col, text_col,"
-            "   extract(epoch from timestamp_col)::bigint as timestamp_col,"
-            "   varchar_col FROM basics";
+    // Verify contents are correct.
+    std::string expected =
+        "0 : empty\n"
+        "1 : \"3001::1\"\n";
 
-        runSql(r, sql, PGRES_TUPLES_OK, line);
-        ASSERT_EQ(r->getRows(), exp_rows) << "fetch at line: " << line
-                  << " wrong row count, expected: " << exp_rows
-                  << " , have: " << r->getRows();
-
-    }
-
-    /// @brief Database connection
-    PgSqlConnectionPtr conn_;
-
-    /// @brief List of column names as we expect them to be in fetched rows
-    std::vector<std::string> expectedColNames_;
-};
-
-// Macros defined to ease passing invocation line number for output tracing
-// (Yes I could have used scoped tracing but that's so ugly in code...)
-#define RUN_SQL(a,b,c) (runSql(a,b,c, __LINE__))
-#define RUN_PREP(a,b,c,d) (runPreparedStatement(a,b,c,d, __LINE__))
-#define FETCH_ROWS(a,b) (fetchRows(a,b,__LINE__))
-#define WIPE_ROWS(a) (RUN_SQL(a, "DELETE FROM BASICS", PGRES_COMMAND_OK))
+    EXPECT_EQ(expected, b.toText());
+}
 
 /// @brief Verifies that PgResultSet row and column meta-data is correct
 TEST_F(PgSqlBasicsTest, rowColumnBasics) {
@@ -709,6 +726,33 @@ TEST_F(PgSqlBasicsTest, intTest) {
     ASSERT_TRUE(PgSqlExchange::isColumnNull(*r, 0, INT_COL));
 }
 
+/// @brief Verify that we use the methods used for testing:
+/// amNull(), getInteger<T>(), getType()
+TEST_F(PgSqlBasicsTest, get) {
+    PsqlBindArray bind_array;
+
+    // The array itself is empty, its first column is not null
+    EXPECT_THROW(bind_array.amNull(), OutOfRange);
+    EXPECT_THROW(bind_array.getInteger<uint32_t>(), OutOfRange);
+    EXPECT_THROW(bind_array.getType(), OutOfRange);
+
+    // Now try again with proper values.
+    bind_array.add(123); // This will be converted to "123" string.
+    bind_array.addNull();
+    bind_array.add("sagittarius");
+    EXPECT_FALSE(bind_array.amNull(0)); // first column is not null
+    EXPECT_TRUE(bind_array.amNull(1)); // second column is null
+    EXPECT_FALSE(bind_array.amNull(2)); // third column is not null
+
+    EXPECT_EQ(123, bind_array.getInteger<uint32_t>(0)); // first column is 123
+    EXPECT_THROW(bind_array.getInteger<uint32_t>(1), BadValue);
+    EXPECT_THROW(bind_array.getInteger<uint32_t>(2), boost::bad_lexical_cast);
+
+    EXPECT_EQ(PsqlBindArray::TEXT_FMT, bind_array.getType(0));
+    EXPECT_EQ(PsqlBindArray::TEXT_FMT, bind_array.getType(1));
+    EXPECT_EQ(PsqlBindArray::TEXT_FMT, bind_array.getType(2));
+}
+
 /// @brief Verify that we can read and write TEXT columns
 TEST_F(PgSqlBasicsTest, textTest) {
     // Create a prepared statement for inserting TEXT
@@ -920,4 +964,577 @@ TEST_F(PgSqlBasicsTest, timeStampTest) {
                                                       MAX_DB_TIME), BadValue);
 }
 
-}; // namespace
+/// @brief Verify that we can read and write ptime using TIMESTAMP columns.
+TEST_F(PgSqlBasicsTest, ptimeTimestamp) {
+    // Create a prepared statement for inserting a TIMESTAMP
+    PgSqlTaggedStatement statement[] = {
+        { 1, { OID_TIMESTAMP }, "timestamp_insert",
+          "INSERT INTO BASICS (localtime_col) values ($1)" }
+    };
+
+    ASSERT_NO_THROW(conn_->prepareStatement(statement[0]));
+
+    // Create an empty array.
+    PsqlBindArrayPtr bind_array(new PsqlBindArray());
+
+    // Make sure we catch values that are too big.
+    time_duration duration = hours(10) + minutes(14) + seconds(15);
+    ptime day_too_far(date(2038, Jan, 21), duration);
+    ASSERT_THROW_MSG(bind_array->addTimestamp(day_too_far), BadValue,
+                     "Time value is too large: 2147681655");
+
+    // Now add reasonable day, US National Ice Cream day.
+    ptime nice_day(date(2021, Jul, 18), duration);
+    bind_array->addTimestamp(nice_day);
+
+    PgSqlResultPtr r;
+    RUN_PREP(r, statement[0], bind_array, PGRES_COMMAND_OK);
+
+    // Fetch the newly inserted row.
+    FETCH_ROWS(r, 1);
+
+    // Timestamp column should not be null.
+    ASSERT_FALSE(PgSqlExchange::isColumnNull(*r, 0, LOCALTIME_COL));
+
+    // Convert fetched value into a ptime.
+    ptime fetched_time;
+    ASSERT_NO_THROW_LOG(PgSqlExchange::getColumnValue(*r, 0, LOCALTIME_COL,
+                        fetched_time));
+
+    ASSERT_EQ(fetched_time, nice_day);
+}
+
+/// @brief Verifies the ability to insert a string into
+/// the bind array.
+TEST(PsqlBindArray, insertString) {
+    PsqlBindArray b;
+
+    // Make a non-temporary string to insert.
+    std::string one("one");
+
+    // Add all the items within a different scope. Everything should
+    // still be valid once we exit this scope.
+    {
+        // Make sure you can "insert" at the front of an empty array.
+        b.insert("two", 0);
+
+        // Add a binding.
+        b.add("four");
+
+        // Verify an out of range index throws.
+        ASSERT_THROW_MSG(b.insert(std::string("too far"), 5), OutOfRange,
+                         "PsqlBindArray::insert - index: 5, "
+                         "is larger than the array size: 2");
+
+        // Insert a non-temporary string at the front.
+        b.insert(one, 0);
+
+        // Insert a temporary string.
+        b.insert("three", 2);
+
+        // Add another one.
+        b.add("five");
+    }
+
+    // We've left bind scope, everything should be intact.
+    EXPECT_EQ(5, b.size());
+
+    // Verify contents are correct.
+    std::string expected =
+        "0 : \"one\"\n"
+        "1 : \"two\"\n"
+        "2 : \"three\"\n"
+        "3 : \"four\"\n"
+        "4 : \"five\"\n";
+
+    EXPECT_EQ(expected, b.toText());
+}
+
+/// @brief Verifies the ability to pop bindings off
+/// the end of a bind array.
+TEST(PsqlBindArray, popBackTest) {
+    PsqlBindArray b;
+
+    // Popping on an empty array should throw.
+    ASSERT_THROW_MSG(b.popBack(), OutOfRange,
+                     "PsqlBindArray::pop_back - array empty");
+
+    // Add five integers.
+    for (int i = 1; i < 6; ++i) {
+        b.add(i);
+    }
+
+    // Verify size.
+    EXPECT_EQ(b.size(), 5);
+
+    // Pop one off.
+    ASSERT_NO_THROW_LOG(b.popBack());
+
+    // Verify size.
+    EXPECT_EQ(b.size(), 4);
+
+    // Pop another one off.
+    ASSERT_NO_THROW_LOG(b.popBack());
+
+    // Verify size.
+    EXPECT_EQ(b.size(), 3);
+
+    // This is what we should have left.
+    std::string expected =
+        "0 : \"1\"\n"
+        "1 : \"2\"\n"
+        "2 : \"3\"\n";
+
+    EXPECT_EQ(expected, b.toText());
+}
+
+/// @brief Verify that we can read and write IPv4 addresses
+/// using INET columns.
+TEST_F(PgSqlBasicsTest, inetTest4) {
+    // Create a prepared statement for inserting an IPv4 address
+    const char* st_name = "inet4_insert";
+    PgSqlTaggedStatement statement[] = {
+        { 1, { OID_TEXT }, st_name,
+          "INSERT INTO BASICS (inet4_col) values (cast($1 as inet))" },
+        { 1, { OID_TEXT }, "check_where",
+          "select * from BASICS where inet4_col = cast($1 as inet)" }
+    };
+
+    ASSERT_NO_THROW_LOG(conn_->prepareStatement(statement[0]));
+    ASSERT_NO_THROW_LOG(conn_->prepareStatement(statement[1]));
+
+    // Build our reference list of reference values
+    std::vector<asiolink::IOAddress>inets;
+    inets.push_back(asiolink::IOAddress("0.0.0.0"));
+    inets.push_back(asiolink::IOAddress("192.168.1.9"));
+    inets.push_back(asiolink::IOAddress("192.168.1.1"));
+
+    // Insert a row for each reference value
+    PsqlBindArrayPtr bind_array;
+    PgSqlResultPtr r;
+    for (int i = 0; i < inets.size(); ++i) {
+        bind_array.reset(new PsqlBindArray());
+        bind_array->addInet4(inets[i]);
+        RUN_PREP(r, statement[0], bind_array, PGRES_COMMAND_OK);
+    }
+
+    // Fetch the newly inserted rows.
+    FETCH_ROWS(r, inets.size());
+
+    // Iterate over the rows, verifying each value against its reference
+    int row = 0;
+    asiolink::IOAddress fetched_inet("0.0.0.0");
+    for ( ; row  < inets.size(); ++row ) {
+        // Verify the column is not null.
+        ASSERT_FALSE(PgSqlExchange::isColumnNull(*r, row, INET4_COL));
+
+        // Fetch and verify the column value
+        ASSERT_NO_THROW(fetched_inet = PgSqlExchange::getInetValue4(*r, row, INET4_COL));
+        EXPECT_EQ(fetched_inet, inets[row]);
+    }
+
+    // Verify that casting from string to inet works in where clauses.
+    r.reset();
+    bind_array.reset(new PsqlBindArray());
+    bind_array->addInet4(inets[1]);
+    RUN_PREP(r, statement[1], bind_array, PGRES_TUPLES_OK);
+    ASSERT_EQ(r->getRows(),1);
+    ASSERT_NO_THROW(fetched_inet = PgSqlExchange::getInetValue4(*r, 0, INET4_COL));
+    EXPECT_EQ(fetched_inet, inets[1]);
+
+    // Clean out the table
+    WIPE_ROWS(r);
+
+    // Verify we can insert a NULL value.
+    bind_array.reset(new PsqlBindArray());
+    bind_array->addNull();
+    RUN_PREP(r, statement[0], bind_array, PGRES_COMMAND_OK);
+
+    // Fetch the newly inserted row.
+    FETCH_ROWS(r, 1);
+
+    // Verify the column is null.
+    ASSERT_TRUE(PgSqlExchange::isColumnNull(*r, 0, INET4_COL));
+}
+
+/// @brief Verify that we can read and write IPv6 addresses
+/// using INET columns.
+TEST_F(PgSqlBasicsTest, inetTest6) {
+    // Create a prepared statement for inserting an IPv6 address
+    const char* st_name = "inet6_insert";
+    PgSqlTaggedStatement statement[] = {
+        { 1, { OID_TEXT }, st_name,
+          "INSERT INTO BASICS (inet6_col) values (cast($1 as inet))" },
+        { 1, { OID_TEXT }, "check_where",
+          "select * from BASICS where inet6_col = cast($1 as inet)" }
+    };
+
+    ASSERT_NO_THROW_LOG(conn_->prepareStatement(statement[0]));
+    ASSERT_NO_THROW_LOG(conn_->prepareStatement(statement[1]));
+
+    // Build our reference list of reference values
+    std::vector<asiolink::IOAddress>inets;
+    inets.push_back(asiolink::IOAddress("::"));
+    inets.push_back(asiolink::IOAddress("3001::1"));
+
+    // Insert a row for each reference value
+    PsqlBindArrayPtr bind_array;
+    PgSqlResultPtr r;
+    for (int i = 0; i < inets.size(); ++i) {
+        bind_array.reset(new PsqlBindArray());
+        bind_array->addInet6(inets[i]);
+        RUN_PREP(r, statement[0], bind_array, PGRES_COMMAND_OK);
+    }
+
+    // Fetch the newly inserted rows.
+    FETCH_ROWS(r, inets.size());
+
+    // Iterate over the rows, verifying each value against its reference
+    int row = 0;
+    asiolink::IOAddress fetched_inet("::");
+    for ( ; row  < inets.size(); ++row ) {
+        // Verify the column is not null.
+        ASSERT_FALSE(PgSqlExchange::isColumnNull(*r, row, INET6_COL));
+
+        // Fetch and verify the column value
+        ASSERT_NO_THROW(fetched_inet = PgSqlExchange::getInetValue6(*r, row, INET6_COL));
+        EXPECT_EQ(fetched_inet, inets[row]);
+    }
+
+    // Verify that casting from string to inet works in where clauses.
+    r.reset();
+    bind_array.reset(new PsqlBindArray());
+    bind_array->addInet6(inets[1]);
+    RUN_PREP(r, statement[1], bind_array, PGRES_TUPLES_OK);
+    ASSERT_EQ(r->getRows(),1);
+    ASSERT_NO_THROW(fetched_inet = PgSqlExchange::getInetValue6(*r, 0, INET6_COL));
+    EXPECT_EQ(fetched_inet, inets[1]);
+
+    // Clean out the table
+    WIPE_ROWS(r);
+
+    // Verify we can insert a NULL value.
+    bind_array.reset(new PsqlBindArray());
+    bind_array->addNull();
+    RUN_PREP(r, statement[0], bind_array, PGRES_COMMAND_OK);
+
+    // Fetch the newly inserted row.
+    FETCH_ROWS(r, 1);
+
+    // Verify the column is null.
+    ASSERT_TRUE(PgSqlExchange::isColumnNull(*r, 0, INET6_COL));
+}
+
+/// @brief Verify that we can read and write floats
+TEST_F(PgSqlBasicsTest, floatTest) {
+    // Create a prepared statement for inserting a FLOAT
+    const char* st_name = "float_insert";
+    PgSqlTaggedStatement statement[] = {
+        { 1, { OID_TEXT }, st_name,
+          "INSERT INTO BASICS (float_col) values (cast($1 as float))" }
+    };
+
+    ASSERT_NO_THROW(conn_->prepareStatement(statement[0]));
+
+    // Build our reference list of reference values
+    std::vector<double>floats;
+    floats.push_back(1.345);
+    floats.push_back(200);
+
+    // Insert a row for each reference value
+    PsqlBindArrayPtr bind_array;
+    PgSqlResultPtr r;
+    for (int i = 0; i < floats.size(); ++i) {
+        bind_array.reset(new PsqlBindArray());
+        bind_array->add(floats[i]);
+        RUN_PREP(r, statement[0], bind_array, PGRES_COMMAND_OK);
+    }
+
+    // Fetch the newly inserted rows.
+    FETCH_ROWS(r, floats.size());
+
+    // Iterate over the rows, verifying each value against its reference
+    int row = 0;
+    for ( ; row  < floats.size(); ++row ) {
+        // Verify the column is not null.
+        ASSERT_FALSE(PgSqlExchange::isColumnNull(*r, row, FLOAT_COL));
+
+        // Fetch and verify the column value
+        double fetched_float;
+        ASSERT_NO_THROW(PgSqlExchange::getColumnValue(*r, row, FLOAT_COL, fetched_float));
+        EXPECT_EQ(fetched_float, floats[row]);
+    }
+
+    // Clean out the table
+    WIPE_ROWS(r);
+
+    // Verify we can insert a NULL value.
+    bind_array.reset(new PsqlBindArray());
+    bind_array->addNull();
+    RUN_PREP(r, statement[0], bind_array, PGRES_COMMAND_OK);
+
+    // Fetch the newly inserted row.
+    FETCH_ROWS(r, 1);
+
+    // Verify the column is null.
+    ASSERT_TRUE(PgSqlExchange::isColumnNull(*r, 0, FLOAT_COL));
+}
+
+/// @brief Verify that we can read and write JSON columns.
+TEST_F(PgSqlBasicsTest, jsonTest) {
+    // Create a prepared statement for inserting a JSON
+    const char* st_name = "json_insert";
+    PgSqlTaggedStatement statement[] = {
+        { 1, { OID_TEXT }, st_name,
+          "INSERT INTO BASICS (json_col) values (cast($1 as json))" }
+    };
+
+    ASSERT_NO_THROW_LOG(conn_->prepareStatement(statement[0]));
+
+    // Build our reference list of reference values
+    std::vector<ElementPtr>elem_ptrs;
+    ASSERT_NO_THROW_LOG(elem_ptrs.push_back(Element::fromJSON("{ \"one\":1 }")));
+    ASSERT_NO_THROW_LOG(elem_ptrs.push_back(Element::fromJSON("{ \"two\":\"two\" }")));
+
+    // Insert a row for each reference value
+    PsqlBindArrayPtr bind_array;
+    PgSqlResultPtr r;
+    for (int i = 0; i < elem_ptrs.size(); ++i) {
+        bind_array.reset(new PsqlBindArray());
+        bind_array->add(elem_ptrs[i]);
+        RUN_PREP(r, statement[0], bind_array, PGRES_COMMAND_OK);
+    }
+
+    // Fetch the newly inserted rows.
+    FETCH_ROWS(r, elem_ptrs.size());
+
+    // Iterate over the rows, verifying each value against its reference
+    int row = 0;
+    for ( ; row  < elem_ptrs.size(); ++row ) {
+        // Verify the column is not null.
+        ASSERT_FALSE(PgSqlExchange::isColumnNull(*r, row, JSON_COL));
+
+        // Fetch and verify the column value
+        ElementPtr fetched_ptr;
+        ASSERT_NO_THROW_LOG(PgSqlExchange::getColumnValue(*r, row, JSON_COL, fetched_ptr));
+        EXPECT_TRUE(fetched_ptr->equals(*(elem_ptrs[row])));
+    }
+
+    // Clean out the table
+    WIPE_ROWS(r);
+
+    // Verify we can insert a NULL value.
+    bind_array.reset(new PsqlBindArray());
+    bind_array->addNull();
+    RUN_PREP(r, statement[0], bind_array, PGRES_COMMAND_OK);
+
+    // Fetch the newly inserted row.
+    FETCH_ROWS(r, 1);
+
+    // Verify the column is null.
+    ASSERT_TRUE(PgSqlExchange::isColumnNull(*r, 0, JSON_COL));
+}
+
+/// @brief Verify that we can read and write integer Triplets.
+TEST_F(PgSqlBasicsTest, tripleTest) {
+    // Create a prepared statement for inserting a Triplet
+    const char* st_name = "triplets_insert";
+    PgSqlTaggedStatement statement[] = {
+        { 3, { OID_INT4, OID_INT4, OID_INT4 }, st_name,
+          "INSERT INTO BASICS (int_col, min_int_col, max_int_col) values ($1, $2, $3)" }
+    };
+
+    ASSERT_NO_THROW_LOG(conn_->prepareStatement(statement[0]));
+
+    // Build our reference list of reference values
+    std::vector<Triplet<uint32_t>> triplets;
+    triplets.push_back(Triplet<uint32_t>());          // def column is null
+    triplets.push_back(Triplet<uint32_t>(10));        // only default column
+    triplets.push_back(Triplet<uint32_t>(5,10,15));   // all three columns
+    triplets.push_back(Triplet<uint32_t>(10,10,15));  // min column is null
+    triplets.push_back(Triplet<uint32_t>(5,10,10));   // max column is null
+
+    // Insert a row for each reference value
+    PsqlBindArrayPtr bind_array;
+    PgSqlResultPtr r;
+    for (auto triplet : triplets) {
+        bind_array.reset(new PsqlBindArray());
+        bind_array->add(triplet);
+        bind_array->addMin(triplet);
+        bind_array->addMax(triplet);
+        RUN_PREP(r, statement[0], bind_array, PGRES_COMMAND_OK);
+    }
+
+    // Fetch the newly inserted rows.
+    FETCH_ROWS(r, triplets.size());
+
+    // Iterate over the rows, verifying each value against its reference
+    int row = 0;
+    for (auto expected : triplets) {
+        Triplet<uint32_t> fetched;
+        // First we test making a triplet only with default value column.
+        ASSERT_NO_THROW_LOG(fetched = PgSqlExchange::getTripletValue(*r, row, INT_COL));
+        if (expected.unspecified()) {
+            EXPECT_TRUE(fetched.unspecified());
+        } else {
+            EXPECT_FALSE(fetched.unspecified());
+            EXPECT_EQ(fetched.get(), expected.get());
+        }
+
+        // Now test making a triplet with all three columns.
+        ASSERT_NO_THROW_LOG(
+            fetched = PgSqlExchange::getTripletValue(*r, row,
+                                                     INT_COL, MIN_INT_COL, MAX_INT_COL));
+        if (expected.unspecified()) {
+            EXPECT_TRUE(fetched.unspecified());
+        } else {
+            EXPECT_FALSE(fetched.unspecified());
+            EXPECT_EQ(fetched.get(), expected.get());
+            EXPECT_EQ(fetched.getMin(), expected.getMin());
+            EXPECT_EQ(fetched.getMax(), expected.getMax());
+        }
+
+        ++row;
+    }
+
+    // Clean out the table
+    WIPE_ROWS(r);
+}
+
+/// @brief Verify PgResultRowWorker operations.
+TEST_F(PgSqlBasicsTest, resultRowWorker) {
+    // Create a prepared statement for inserting multiple types
+    const char* st_name = "row_insert";
+    PgSqlTaggedStatement statement[] = {
+        { 14,
+          {
+            OID_BOOL,
+            OID_BYTEA,
+            OID_INT8,
+            OID_INT2,
+            OID_INT4,
+            OID_TEXT,
+            OID_TIMESTAMP,
+            OID_TEXT,
+            OID_TEXT,
+            OID_TEXT,
+            OID_TEXT,
+            OID_INT4,
+            OID_INT4,
+          }, st_name,
+          "INSERT INTO BASICS ("
+          " bool_col, "
+          " bytea_col, "
+          " bigint_col, "
+          " smallint_col, "
+          " int_col, "
+          " text_col, "
+          " localtime_col, "
+          " varchar_col, "
+          " inet4_col, "
+          " float_col, "
+          " json_col, "
+          " min_int_col, "
+          " max_int_col, "
+          " inet6_col) "
+          " VALUES ($1, $2, $3, $4, $5, $6, $7, $8, cast($9 as inet), "
+          "         cast($10 as float), cast($11 as json), $12, $13, $14)"
+        }
+    };
+
+    ASSERT_NO_THROW_LOG(conn_->prepareStatement(statement[0]));
+
+    // Create a bind array of input values.
+    PsqlBindArrayPtr b(new PsqlBindArray());
+
+    bool exp_bool(true);
+    b->add(exp_bool);
+
+    std::vector<uint8_t> exp_bytes({0x01, 0x02, 0x03, 0x04});
+    b->add(exp_bytes);
+
+    uint64_t exp_bigint = 9876;
+    b->add(exp_bigint);
+
+    uint16_t exp_smallint = 12;
+    b->add(exp_smallint);
+
+    uint32_t exp_int = 345;
+    b->add(exp_int);
+
+    std::string exp_text = "just some string";
+    b->add(exp_text);
+
+    time_duration duration = hours(7) + minutes(45) + seconds(9);
+    ptime exp_timestamp(date(2021, Jul, 18), duration);
+    b->addTimestamp(exp_timestamp);
+
+    const char* exp_varchar = "really just a string";
+    b->add(exp_varchar);
+
+    asiolink::IOAddress exp_inet4("192.168.1.35");
+    b->addInet4(exp_inet4);
+
+    double exp_double(2.5);
+    b->add(exp_double);
+
+    ElementPtr exp_elems = Element::fromJSON("{ \"foo\": \"bar\" }");
+    b->add(exp_elems);
+
+    uint32_t exp_min = 100;
+    b->add(exp_min);
+
+    uint32_t exp_max = 500;
+    b->add(exp_max);
+
+    asiolink::IOAddress exp_inet6("3001::77");
+    b->addInet6(exp_inet6);
+
+    PgSqlResultPtr r;
+    RUN_PREP(r, statement[0], b, PGRES_COMMAND_OK);
+
+    // Fetch the newly inserted row.
+    FETCH_ROWS(r, 1);
+
+    // Create a row worker.
+    PgSqlResultRowWorkerPtr worker;
+
+    // Creating the row worker for the first (and only) row should succeed.
+    ASSERT_NO_THROW_LOG(worker.reset(new PgSqlResultRowWorker(*r, 0)));
+
+    // Now let's test all the getters.
+    EXPECT_EQ(exp_bool, worker->getBool(BOOL_COL));
+
+    std::vector<uint8_t> fetched_bytes;
+    ASSERT_NO_THROW_LOG(worker->getBytes(BYTEA_COL, fetched_bytes));
+    EXPECT_EQ(exp_bytes, fetched_bytes);
+
+    EXPECT_EQ(exp_bigint, worker->getBigInt(BIGINT_COL));
+    EXPECT_EQ(exp_smallint, worker->getSmallInt(SMALLINT_COL));
+    EXPECT_EQ(exp_int, worker->getInt(INT_COL));
+    EXPECT_EQ(exp_text, worker->getString(TEXT_COL));
+    EXPECT_EQ(exp_timestamp, worker->getTimestamp(LOCALTIME_COL));
+    EXPECT_EQ(exp_varchar, worker->getString(VARCHAR_COL));
+    EXPECT_EQ(exp_inet4, worker->getInet4(INET4_COL));
+    EXPECT_EQ(exp_double, worker->getDouble(FLOAT_COL));
+    EXPECT_EQ(*exp_elems, *(worker->getJSON(JSON_COL)));
+    EXPECT_EQ(exp_min, worker->getInt(MIN_INT_COL));
+    EXPECT_EQ(exp_max, worker->getInt(MAX_INT_COL));
+    EXPECT_EQ(exp_inet6, worker->getInet6(INET6_COL));
+
+    // Get a triplet using int_col as the sole value.
+    Triplet<uint32_t>fetched_triplet = worker->getTriplet(INT_COL);
+    EXPECT_EQ(exp_int, fetched_triplet.get());
+
+    // Get a triplet using int_col, min_col, and max_col values.
+    fetched_triplet = worker->getTriplet(INT_COL, MIN_INT_COL, MAX_INT_COL);
+    EXPECT_EQ(exp_int, fetched_triplet.get());
+    EXPECT_EQ(exp_min, fetched_triplet.getMin());
+    EXPECT_EQ(exp_max, fetched_triplet.getMax());
+
+    // Attempting to access an invalid row should throw.
+    ASSERT_THROW_MSG(worker.reset(new PgSqlResultRowWorker(*r, 1)),
+                     DbOperationError, "row: 1, out of range: 0..1");
+}
+
+} // namespace
