@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -90,9 +90,7 @@ TEST_F(LoggingInfoTest, defaults) {
     std::string begin =
         "\"name\": \"kea\",\n"
         "\"output_options\": [ {\n"
-        " \"output\": \"stdout\",\n \"maxsize\": 10240000,\n"
-        " \"pattern\": \"\","
-        " \"maxver\": 1,\n \"flush\": true } ],\n"
+        " \"output\": \"stdout\", \"flush\": true, \"pattern\": \"\" } ],\n"
         "\"severity\": \"";
     std::string dbglvl = "\",\n\"debuglevel\": ";
     std::string trailer = "\n}\n";
@@ -205,7 +203,6 @@ TEST_F(LoggingInfoTest, equalityOperators) {
     // The should now be unequal.
     EXPECT_FALSE(info1 == info2);
     EXPECT_TRUE(info1 != info2);
-
 }
 
 } // end of anonymous namespace
