@@ -1,4 +1,4 @@
-// Copyright (C) 2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,16 +6,15 @@
 
 #include <config.h>
 
-#include <yang/translator_subnet.h>
 #include <yang/adaptor_pool.h>
+#include <yang/translator_subnet.h>
 #include <yang/yang_models.h>
+
 #include <sstream>
 
 using namespace std;
 using namespace isc::data;
-#ifndef HAVE_PRE_0_7_6_SYSREPO
 using namespace sysrepo;
-#endif
 
 namespace isc {
 namespace yang {
@@ -87,7 +86,9 @@ TranslatorSubnet::getSubnetIetf6(const string& xpath) {
         result->set("user-context", context);
     }
     /// missing a lot of things
-    AdaptorPool::toSubnet(model_, result, result->get("pools"));
+    if (result->get("pools")) {
+        AdaptorPool::toSubnet(model_, result, result->get("pools"));
+    }
     return (result);
 }
 
@@ -99,12 +100,29 @@ TranslatorSubnet::getSubnetKea(const string& xpath) {
         if (preferred) {
             result->set("preferred-lifetime", preferred);
         }
+        ConstElementPtr min_pref = getItem(xpath + "/min-preferred-lifetime");
+        if (min_pref) {
+            result->set("min-preferred-lifetime", min_pref);
+        }
+        ConstElementPtr max_pref = getItem(xpath + "/max-preferred-lifetime");
+        if (max_pref) {
+            result->set("max-preferred-lifetime", max_pref);
+        }
     }
     ConstElementPtr valid = getItem(xpath + "/valid-lifetime");
     if (valid) {
         result->set("valid-lifetime", valid);
     }
+    ConstElementPtr min_valid = getItem(xpath + "/min-valid-lifetime");
+    if (min_valid) {
+        result->set("min-valid-lifetime", min_valid);
+    }
+    ConstElementPtr max_valid = getItem(xpath + "/max-valid-lifetime");
+    if (max_valid) {
+        result->set("max-valid-lifetime", max_valid);
+    }
     ConstElementPtr renew = getItem(xpath + "/renew-timer");
+
     if (renew) {
         result->set("renew-timer", renew);
     }
@@ -112,6 +130,18 @@ TranslatorSubnet::getSubnetKea(const string& xpath) {
     if (rebind) {
         result->set("rebind-timer", rebind);
     }
+    ConstElementPtr calculate = getItem(xpath + "/calculate-tee-times");
+    if (calculate) {
+        result->set("calculate-tee-times", calculate);
+    }
+    ConstElementPtr t1_percent = getItem(xpath + "/t1-percent");
+    if (t1_percent) {
+        result->set("t1-percent", t1_percent);
+    }
+    ConstElementPtr t2_percent = getItem(xpath + "/t2-percent");
+    if (t2_percent) {
+        result->set("t2-percent", t2_percent);
+    }
     ConstElementPtr options = getOptionDataList(xpath);
     if (options && (options->size() > 0)) {
         result->set("option-data", options);
@@ -212,6 +242,22 @@ TranslatorSubnet::getSubnetKea(const string& xpath) {
     if (context) {
         result->set("user-context", Element::fromJSON(context->stringValue()));
     }
+    checkAndGetLeaf(result, xpath, "cache-max-age");
+    checkAndGetLeaf(result, xpath, "cache-threshold");
+    checkAndGetLeaf(result, xpath, "ddns-generated-prefix");
+    checkAndGetLeaf(result, xpath, "ddns-override-client-update");
+    checkAndGetLeaf(result, xpath, "ddns-override-no-update");
+    checkAndGetLeaf(result, xpath, "ddns-qualifying-suffix");
+    checkAndGetLeaf(result, xpath, "ddns-replace-client-name");
+    checkAndGetLeaf(result, xpath, "ddns-send-updates");
+    checkAndGetLeaf(result, xpath, "ddns-update-on-renew");
+    checkAndGetLeaf(result, xpath, "ddns-use-conflict-resolution");
+    checkAndGetLeaf(result, xpath, "hostname-char-replacement");
+    checkAndGetLeaf(result, xpath, "hostname-char-set");
+    checkAndGetLeaf(result, xpath, "reservations-global");
+    checkAndGetLeaf(result, xpath, "reservations-in-subnet");
+    checkAndGetLeaf(result, xpath, "reservations-out-of-pool");
+    checkAndGetLeaf(result, xpath, "store-extended-info");
     return (result);
 }
 
@@ -270,11 +316,27 @@ TranslatorSubnet::setSubnetKea(const string& xpath, ConstElementPtr elem) {
         if (preferred) {
             setItem(xpath + "/preferred-lifetime", preferred, SR_UINT32_T);
         }
+        ConstElementPtr min_pref = elem->get("min-preferred-lifetime");
+        if (min_pref) {
+            setItem(xpath + "/min-preferred-lifetime", min_pref, SR_UINT32_T);
+        }
+        ConstElementPtr max_pref = elem->get("max-preferred-lifetime");
+        if (max_pref) {
+            setItem(xpath + "/max-preferred-lifetime", max_pref, SR_UINT32_T);
+        }
     }
     ConstElementPtr valid = elem->get("valid-lifetime");
     if (valid) {
         setItem(xpath + "/valid-lifetime", valid, SR_UINT32_T);
     }
+    ConstElementPtr min_valid = elem->get("min-valid-lifetime");
+    if (min_valid) {
+        setItem(xpath + "/min-valid-lifetime", min_valid, SR_UINT32_T);
+    }
+    ConstElementPtr max_valid = elem->get("max-valid-lifetime");
+    if (max_valid) {
+        setItem(xpath + "/max-valid-lifetime", max_valid, SR_UINT32_T);
+    }
     ConstElementPtr renew = elem->get("renew-timer");
     if (renew) {
         setItem(xpath + "/renew-timer", renew, SR_UINT32_T);
@@ -283,6 +345,18 @@ TranslatorSubnet::setSubnetKea(const string& xpath, ConstElementPtr elem) {
     if (rebind) {
         setItem(xpath + "/rebind-timer", rebind, SR_UINT32_T);
     }
+    ConstElementPtr calculate = elem->get("calculate-tee-times");
+    if (calculate) {
+        setItem(xpath + "/calculate-tee-times", calculate, SR_BOOL_T);
+    }
+    ConstElementPtr t1_percent =  elem->get("t1-percent");
+    if (t1_percent) {
+        setItem(xpath + "/t1-percent", t1_percent, SR_DECIMAL64_T);
+    }
+    ConstElementPtr t2_percent =  elem->get("t2-percent");
+    if (t2_percent) {
+        setItem(xpath + "/t2-percent", t2_percent, SR_DECIMAL64_T);
+    }
     ConstElementPtr options = elem->get("option-data");
     if (options && (options->size() > 0)) {
         setOptionDataList(xpath, options);
@@ -348,6 +422,22 @@ TranslatorSubnet::setSubnetKea(const string& xpath, ConstElementPtr elem) {
             }
         }
     }
+    checkAndSetLeaf(elem, xpath, "cache-max-age", SR_UINT32_T);
+    checkAndSetLeaf(elem, xpath, "cache-threshold", SR_DECIMAL64_T);
+    checkAndSetLeaf(elem, xpath, "ddns-generated-prefix", SR_STRING_T);
+    checkAndSetLeaf(elem, xpath, "ddns-override-client-update", SR_BOOL_T);
+    checkAndSetLeaf(elem, xpath, "ddns-override-no-update", SR_BOOL_T);
+    checkAndSetLeaf(elem, xpath, "ddns-qualifying-suffix", SR_STRING_T);
+    checkAndSetLeaf(elem, xpath, "ddns-replace-client-name", SR_STRING_T);
+    checkAndSetLeaf(elem, xpath, "ddns-send-updates", SR_BOOL_T);
+    checkAndSetLeaf(elem, xpath, "ddns-update-on-renew", SR_BOOL_T);
+    checkAndSetLeaf(elem, xpath, "ddns-use-conflict-resolution", SR_BOOL_T);
+    checkAndSetLeaf(elem, xpath, "hostname-char-replacement", SR_STRING_T);
+    checkAndSetLeaf(elem, xpath, "hostname-char-set", SR_STRING_T);
+    checkAndSetLeaf(elem, xpath, "reservations-global", SR_BOOL_T);
+    checkAndSetLeaf(elem, xpath, "reservations-in-subnet", SR_BOOL_T);
+    checkAndSetLeaf(elem, xpath, "reservations-out-of-pool", SR_BOOL_T);
+    checkAndSetLeaf(elem, xpath, "store-extended-info", SR_BOOL_T);
     if (model_ == KEA_DHCP4_SERVER) {
         ConstElementPtr match = elem->get("match-client-id");
         if (match) {
@@ -427,20 +517,8 @@ TranslatorSubnets::getSubnets(const string& xpath) {
 ElementPtr
 TranslatorSubnets::getSubnetsCommon(const string& xpath,
                                     const std::string& subsel) {
-    ElementPtr result = Element::createList();
-    S_Iter_Value iter = getIter(xpath + "/" + subsel);
-    if (!iter) {
-        /// Can't happen.
-        isc_throw(Unexpected, "getSubnets: can't get iterator: " << xpath);
-    }
-    for (;;) {
-        const string& subnet = getNext(iter);
-        if (subnet.empty()) {
-            break;
-        }
-        result->add(getSubnet(subnet));
-    }
-    return (result);
+    return getList<TranslatorSubnet>(xpath + "/" + subsel, *this,
+                                     &TranslatorSubnet::getSubnet);
 }
 
 void
@@ -493,5 +571,5 @@ TranslatorSubnets::setSubnetsKea(const string& xpath, ConstElementPtr elem,
     }
 }
 
-}; // end of namespace isc::yang
-}; // end of namespace isc
+}  // namespace yang
+}  // namespace isc
